@@ -56,6 +56,9 @@
 #
 #   BUILD_JAVA        Default: 1
 #     Build and test java code if this is set to 1.
+#     Note: The Java code required for the C++ code and tests to run,
+#     such as the HMS plugin and the Java subprocess, will still be built
+#     even if set to 0.
 #
 #   BUILD_PYTHON       Default: 1
 #     Build and test the Python wrapper of the client API.
@@ -82,13 +85,41 @@
 #
 #   KUDU_REPORT_TEST_RESULTS Default: 0
 #     If non-zero, tests are reported to the central test server.
+#
+#   KUDU_ALLOW_SKIPPED_TESTS Default: 0
+#     If set to 1, commits with changes that do not impact the build or tests exit early.
+#     Additionally, commits with "DONT_BUILD" in the commit message will exit early.
+#
+#   PARALLEL    Default: number of available cores
+#     Parallelism to use when compiling; by default is set to the number of
+#     available cores as reported by `getconf _NPROCESSORS_ONLN`. The latter
+#     might report total number of cores available to the _host_ OS in case of
+#     containerized VM instances: for optimal results, it's useful to override
+#     this variable to reflect actual restrictions.
 
-# If a commit messages contains a line that says 'DONT_BUILD', exit
-# immediately.
-DONT_BUILD=$(git show|egrep '^\s{4}DONT_BUILD$')
-if [ "x$DONT_BUILD" != "x" ]; then
+if [ "$KUDU_ALLOW_SKIPPED_TESTS" == "1" ]; then
+  # If the commit only contains changes that do not impact the build or tests, exit immediately.
+  # This check is conservative and attempts to have no false positives, but may have false negatives.
+  if ! git diff-tree --no-commit-id --name-only -r HEAD |
+     grep -qvE "^\.dockerignore|^\.gitignore|^\.ycm_extra_conf.py|^docker/|^docs/|^kubernetes/|LICENSE\.txt|NOTICE\.txt|.*\.adoc|.*\.md"; then
+    echo
+    echo ------------------------------------------------------------
+    echo "*** Changes are only in files or directories that do not impact the build or tests. Exiting."
+    echo ------------------------------------------------------------
+    echo
+    exit 0
+  fi
+
+  # If a commit messages contains a line that says 'DONT_BUILD', exit immediately.
+  DONT_BUILD=$(git show|egrep '^\s{4}DONT_BUILD$')
+  if [ "x$DONT_BUILD" != "x" ]; then
+    echo
+    echo ------------------------------------------------------------
     echo "*** Build not requested. Exiting."
+    echo ------------------------------------------------------------
+    echo
     exit 1
+  fi
 fi
 
 set -e
@@ -116,6 +147,7 @@ export KUDU_FLAKY_TEST_ATTEMPTS=${KUDU_FLAKY_TEST_ATTEMPTS:-1}
 export KUDU_ALLOW_SLOW_TESTS=${KUDU_ALLOW_SLOW_TESTS:-$DEFAULT_ALLOW_SLOW_TESTS}
 export KUDU_COMPRESS_TEST_OUTPUT=${KUDU_COMPRESS_TEST_OUTPUT:-1}
 export TEST_TMPDIR=${TEST_TMPDIR:-/tmp/kudutest-$UID}
+export PARALLEL=${PARALLEL:-$(getconf _NPROCESSORS_ONLN)}
 BUILD_JAVA=${BUILD_JAVA:-1}
 BUILD_GRADLE=${BUILD_GRADLE:-1}
 BUILD_PYTHON=${BUILD_PYTHON:-1}
@@ -265,6 +297,8 @@ THIRDPARTY_TYPE=
 if [ "$BUILD_TYPE" = "TSAN" ]; then
   THIRDPARTY_TYPE=tsan
 fi
+
+# The settings for PARALLEL (see above) also affects the thirdparty build.
 $SOURCE_ROOT/build-support/enable_devtoolset.sh thirdparty/build-if-necessary.sh $THIRDPARTY_TYPE
 
 THIRDPARTY_BIN=$(pwd)/thirdparty/installed/common/bin
@@ -276,6 +310,27 @@ else
   CLANG=$(pwd)/thirdparty/clang-toolchain/bin/clang
 fi
 
+# Make sure we use JDK8
+if [ -n "$JAVA8_HOME" ]; then
+  export JAVA_HOME="$JAVA8_HOME"
+  export PATH="$JAVA_HOME/bin:$PATH"
+fi
+
+# Some portions of the C++ build may depend on Java code, so we may run Gradle
+# while building. Pass in some flags suitable for automated builds; these will
+# also be used in the Java build.
+# These should be set before CMAKE so that the Gradle command in the
+# generated make file has the correct flags.
+export EXTRA_GRADLE_FLAGS="--console=plain"
+EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --no-daemon"
+EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --continue"
+# Temporarily disable parallel builds for automated builds.
+EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --no-parallel"
+# KUDU-2524: temporarily disable scalafmt until we can work out its JDK
+# incompatibility issue.
+EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS -DskipFormat"
+EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS $GRADLE_FLAGS"
+
 # Assemble the cmake command line, starting with environment variables.
 
 # There's absolutely no reason to rebuild the thirdparty tree; we just ran
@@ -344,8 +399,35 @@ fi
 
 # Short circuit for LINT builds.
 if [ "$BUILD_TYPE" = "LINT" ]; then
-  make lint | tee $TEST_LOGDIR/lint.log
-  exit $?
+  LINT_FAILURES=""
+  LINT_RESULT=0
+
+  if [ "$BUILD_JAVA" == "1" ]; then
+    pushd $SOURCE_ROOT/java
+    if ! ./gradlew $EXTRA_GRADLE_FLAGS clean check -x test $EXTRA_GRADLE_TEST_FLAGS; then
+      LINT_RESULT=1
+      LINT_FAILURES="$LINT_FAILURES"$'Java Gradle check failed\n'
+    fi
+    popd
+  fi
+
+  if ! make lint | tee $TEST_LOGDIR/lint.log; then
+    LINT_RESULT=1
+    LINT_FAILURES="$LINT_FAILURES"$'make lint failed\n'
+  fi
+
+  if [ -n "$LINT_FAILURES" ]; then
+    echo
+    echo
+    echo ======================================================================
+    echo Lint Failure summary
+    echo ======================================================================
+    echo $LINT_FAILURES
+    echo
+    echo
+  fi
+
+  exit $LINT_RESULT
 fi
 
 # Short circuit for IWYU builds: run the include-what-you-use tool on the files
@@ -358,7 +440,7 @@ fi
 # Short circuit for TIDY builds: run the clang-tidy tool on the C++ source
 # files in the HEAD revision for the gerrit branch.
 if [ "$BUILD_TYPE" = "TIDY" ]; then
-  make -j$NUM_PROCS generated-headers 2>&1 | tee $TEST_LOGDIR/tidy.log
+  make -j$PARALLEL generated-headers 2>&1 | tee $TEST_LOGDIR/tidy.log
   $SOURCE_ROOT/build-support/clang_tidy_gerrit.py HEAD 2>&1 | \
       tee -a $TEST_LOGDIR/tidy.log
   exit $?
@@ -378,8 +460,7 @@ fi
 echo
 echo Building C++ code.
 echo ------------------------------------------------------------
-NUM_PROCS=$(getconf _NPROCESSORS_ONLN)
-make -j$NUM_PROCS 2>&1 | tee build.log
+make -j$PARALLEL 2>&1 | tee build.log
 
 # If compilation succeeds, try to run all remaining steps despite any failures.
 set +e
@@ -405,7 +486,7 @@ if [ "$ENABLE_DIST_TEST" == "1" ]; then
   EXTRA_TEST_FLAGS="$EXTRA_TEST_FLAGS -L no_dist_test"
 fi
 
-if ! $THIRDPARTY_BIN/ctest -j$NUM_PROCS $EXTRA_TEST_FLAGS ; then
+if ! $THIRDPARTY_BIN/ctest -j$PARALLEL $EXTRA_TEST_FLAGS ; then
   TESTS_FAILED=1
   FAILURES="$FAILURES"$'C++ tests failed\n'
 fi
@@ -430,20 +511,10 @@ if [ "$BUILD_JAVA" == "1" ]; then
   echo Building and testing java...
   echo ------------------------------------------------------------
 
-  # Make sure we use JDK8
-  export JAVA_HOME=$JAVA8_HOME
-  export PATH=$JAVA_HOME/bin:$PATH
   pushd $SOURCE_ROOT/java
   set -x
 
   # Run the full Gradle build.
-  export EXTRA_GRADLE_FLAGS="--console=plain"
-  EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --no-daemon"
-  EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS --continue"
-  # KUDU-2524: temporarily disable scalafmt until we can work out its JDK
-  # incompatibility issue.
-  EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS -DskipFormat"
-  EXTRA_GRADLE_FLAGS="$EXTRA_GRADLE_FLAGS $GRADLE_FLAGS"
   # If we're running distributed Java tests, submit them asynchronously.
   if [ "$ENABLE_DIST_TEST" == "1" ]; then
     echo
@@ -504,13 +575,16 @@ if [ "$BUILD_PYTHON" == "1" ]; then
   #
   # The absence of $PIP_FLAGS is intentional: older versions of pip may not
   # support the flags that we want to use.
-  pip install -i https://pypi.python.org/simple $PIP_INSTALL_FLAGS --upgrade 'pip < 10.0.0b1'
+  pip install -i https://pypi.python.org/simple $PIP_INSTALL_FLAGS --upgrade 'pip <10.0.0b1'
 
   # New versions of pip raise an exception when upgrading old versions of
   # setuptools (such as the one found in el6). The workaround is to upgrade
   # setuptools on its own, outside of requirements.txt, and with the pip version
   # check disabled.
-  pip $PIP_FLAGS install --disable-pip-version-check $PIP_INSTALL_FLAGS --upgrade 'setuptools >= 0.8'
+  #
+  # Setuptools 42.0.0 changes something that causes build_ext to fail with a
+  # missing wheel package. Let's pin to an older version known to work.
+  pip $PIP_FLAGS install --disable-pip-version-check $PIP_INSTALL_FLAGS --upgrade 'setuptools >=0.8,<42.0.0'
 
   # One of our dependencies is pandas, installed below. It depends on numpy, and
   # if we don't install numpy directly, the pandas installation will install the
@@ -521,7 +595,7 @@ if [ "$BUILD_PYTHON" == "1" ]; then
   # so we must pass in the current values of CC and CXX.
   #
   # See https://github.com/numpy/numpy/releases/tag/v1.12.0 for more details.
-  CC=$CLANG CXX=$CLANG++ pip $PIP_FLAGS install $PIP_INSTALL_FLAGS 'numpy < 1.12.0'
+  CC=$CLANG CXX=$CLANG++ pip $PIP_FLAGS install $PIP_INSTALL_FLAGS 'numpy <1.12.0'
 
   # We've got a new pip and new setuptools. We can now install the rest of the
   # Python client's requirements.
@@ -538,7 +612,7 @@ if [ "$BUILD_PYTHON" == "1" ]; then
   #
   # pandas 0.18 dropped support for python 2.6. See https://pandas.pydata.org/pandas-docs/version/0.23.0/whatsnew.html#v0-18-0-march-13-2016
   # for more details.
-  CC=$CLANG CXX=$CLANG++ pip $PIP_FLAGS install $PIP_INSTALL_FLAGS 'pandas < 0.18'
+  CC=$CLANG CXX=$CLANG++ pip $PIP_FLAGS install $PIP_INSTALL_FLAGS 'pandas <0.18'
 
   # Delete old Cython extensions to force them to be rebuilt.
   rm -Rf build kudu_python.egg-info kudu/*.so
@@ -593,13 +667,13 @@ if [ "$BUILD_PYTHON3" == "1" ]; then
   # support the flags that we want to use.
   #
   # 1. https://github.com/pypa/pip/issues/6175
-  pip install -i https://pypi.python.org/simple $PIP_INSTALL_FLAGS --upgrade 'pip < 19.0'
+  pip install -i https://pypi.python.org/simple $PIP_INSTALL_FLAGS --upgrade 'pip <19.0'
 
   # New versions of pip raise an exception when upgrading old versions of
   # setuptools (such as the one found in el6). The workaround is to upgrade
   # setuptools on its own, outside of requirements.txt, and with the pip version
   # check disabled.
-  pip $PIP_FLAGS install --disable-pip-version-check $PIP_INSTALL_FLAGS --upgrade 'setuptools >= 0.8'
+  pip $PIP_FLAGS install --disable-pip-version-check $PIP_INSTALL_FLAGS --upgrade 'setuptools >=0.8'
 
   # One of our dependencies is pandas, installed below. It depends on numpy, and
   # if we don't install numpy directly, the pandas installation will install the
@@ -611,7 +685,7 @@ if [ "$BUILD_PYTHON3" == "1" ]; then
   # so we must pass in the current values of CC and CXX.
   #
   # See https://github.com/numpy/numpy/releases/tag/v1.16.0rc1 for more details.
-  CC=$CLANG CXX=$CLANG++ pip $PIP_FLAGS install $PIP_INSTALL_FLAGS 'numpy < 1.16.0'
+  CC=$CLANG CXX=$CLANG++ pip $PIP_FLAGS install $PIP_INSTALL_FLAGS 'numpy <1.16.0'
 
   # We've got a new pip and new setuptools. We can now install the rest of the
   # Python client's requirements.
