@@ -356,7 +356,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       public List<InputSplit> getSplits(JobContext job) throws IOException, InterruptedException {
         int numMappers = job.getConfiguration().getInt(GENERATOR_NUM_MAPPERS_KEY, 1);
 
-        ArrayList<InputSplit> splits = new ArrayList<InputSplit>(numMappers);
+        ArrayList<InputSplit> splits = new ArrayList<>(numMappers);
 
         for (int i = 0; i < numMappers; i++) {
           splits.add(new GeneratorInputSplit());
@@ -406,7 +406,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       private byte[][] current = null;
       private String id;
       private long rowId = 0;
-      private int i;
+      private int position;
       private KuduClient client;
       private KuduTable table;
       private KuduSession session;
@@ -416,7 +416,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       private int width;
 
       @Override
-      protected void setup(Context context) throws KuduException {
+      protected void setup(Mapper.Context context) throws KuduException {
         id = "Job: " + context.getJobID() + " Task: " + context.getTaskAttemptID();
         Configuration conf = context.getConfiguration();
         CommandLineParser parser = new CommandLineParser(conf);
@@ -440,18 +440,19 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       }
 
       @Override
-      protected void cleanup(Context context) throws KuduException {
+      protected void cleanup(Mapper.Context context) throws KuduException {
         session.close();
         client.shutdown();
       }
 
       @Override
-      protected void map(BytesWritable key, NullWritable value, Context output) throws IOException {
-        current[i] = new byte[key.getLength()];
-        System.arraycopy(key.getBytes(), 0, current[i], 0, key.getLength());
-        if (++i == current.length) {
+      protected void map(BytesWritable key, NullWritable value, Mapper.Context output)
+              throws IOException {
+        current[position] = new byte[key.getLength()];
+        System.arraycopy(key.getBytes(), 0, current[position], 0, key.getLength());
+        if (++position == current.length) {
           persist(output, current, false);
-          i = 0;
+          position = 0;
 
           // Keep track of the first row so that we can point to it at the end.
           if (first == null) {
@@ -490,7 +491,8 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
         first[first.length - 1] = ez;
       }
 
-      private void persist(Context output, byte[][] data, boolean update) throws KuduException {
+      private void persist(Mapper.Context output, byte[][] data, boolean update)
+              throws KuduException {
         for (int i = 0; i < data.length; i++) {
           Operation put = update ? table.newUpdate() : table.newInsert();
           PartialRow row = put.getRow();
@@ -581,6 +583,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       client.createTable(tableName, schema, builder);
     }
 
+    @SuppressWarnings("deprecation")
     public int runRandomInputGenerator(int numMappers, long numNodes, Path tmpOutput,
                                        Integer width, Integer wrapMultiplier) throws Exception {
       LOG.info("Running RandomInputGenerator with numMappers=" + numMappers +
@@ -607,6 +610,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       return success ? 0 : 1;
     }
 
+    @SuppressWarnings("deprecation")
     public int runGenerator(int numMappers, long numNodes, int numTablets, Path tmpOutput,
                             Integer width, Integer wrapMultiplier) throws Exception {
       LOG.info("Running Generator with numMappers=" + numMappers + ", numNodes=" + numNodes);
@@ -667,6 +671,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       private BytesWritable ref = new BytesWritable();
 
       @Override
+      @SuppressWarnings("unchecked")
       protected void map(NullWritable key, RowResult value, Mapper.Context context)
           throws IOException, InterruptedException {
         Bytes.setLong(rowKey, value.getLong(0));
@@ -687,11 +692,12 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       }
     }
 
-    public static class VerifyReducer extends Reducer<BytesWritable,BytesWritable,Text,Text> {
-      private ArrayList<byte[]> refs = new ArrayList<byte[]>();
+    public static class VerifyReducer extends Reducer<BytesWritable, BytesWritable, Text, Text> {
+      private ArrayList<byte[]> refs = new ArrayList<>();
 
       @Override
-      public void reduce(BytesWritable key, Iterable<BytesWritable> values, Context context)
+      public void reduce(BytesWritable key, Iterable<BytesWritable> values,
+                         Reducer<BytesWritable, BytesWritable, Text, Text>.Context context)
           throws IOException, InterruptedException {
 
         int defCount = 0;
@@ -733,9 +739,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
           context.getCounter(Counts.UNREFERENCED).increment(1);
         } else {
           if (refs.size() > 1) {
-            if (refsList != null) {
-              context.write(new Text(keyString), new Text(refsList.toString()));
-            }
+            context.write(new Text(keyString), new Text(refsList.toString()));
             context.getCounter(Counts.EXTRAREFERENCES).increment(refs.size() - 1);
           }
           // node is defined and referenced
@@ -764,6 +768,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       return run(new Path(outputDir), numReducers);
     }
 
+    @SuppressWarnings("deprecation")
     public int run(Path outputDir, int numReducers) throws Exception {
       LOG.info("Running Verify with outputDir=" + outputDir + ", numReducers=" + numReducers);
 
@@ -793,7 +798,6 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       return success ? 0 : 1;
     }
 
-    @SuppressWarnings("deprecation")
     public boolean verify(long expectedReferenced) throws Exception {
       if (job == null) {
         throw new IllegalStateException("You should call run() first");
@@ -977,7 +981,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
         System.err.println();
         HelpFormatter formatter = new HelpFormatter();
         formatter.printHelp(getClass().getSimpleName(), options);
-        System.exit(-1);
+        throw new RuntimeException(e);
       }
 
       CommandLineParser cmdLineParser = new CommandLineParser(getConf());
@@ -1088,7 +1092,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       private List<Pair<Long, Long>> headsCache;
 
       @Override
-      protected void setup(Context context) throws KuduException {
+      protected void setup(Mapper.Context context) throws KuduException {
         Configuration conf = context.getConfiguration();
         CommandLineParser parser = new CommandLineParser(conf);
         client = parser.getClient();
@@ -1104,8 +1108,8 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
         // Add as many heads as we need, then we skip the rest.
         do {
           if (headsCache.size() < numUpdatesPerMapper) {
-            value = (RowResult)context.getCurrentValue();
-            headsCache.add(new Pair<Long, Long>(value.getLong(0), value.getLong(1)));
+            RowResult realValue = (RowResult) context.getCurrentValue();
+            headsCache.add(new Pair<>(realValue.getLong(0), realValue.getLong(1)));
           }
         } while (context.nextKeyValue());
 
@@ -1137,14 +1141,14 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
         do {
           RowResult prev = nextNode(prevKeyOne, prevKeyTwo);
           if (prev == null) {
-            context.getCounter(Counts.BROKEN_LINKS).increment(1);
+            context.getCounter(Updater.Counts.BROKEN_LINKS).increment(1);
             LOG.warn(getStringFromKeys(prevKeyOne, prevKeyTwo) + " doesn't exist");
             break;
           }
 
           // It's possible those columns are null, let's not break trying to read them.
           if (prev.isNull(0) || prev.isNull(1)) {
-            context.getCounter(Counts.BROKEN_LINKS).increment(1);
+            context.getCounter(Updater.Counts.BROKEN_LINKS).increment(1);
             LOG.warn(getStringFromKeys(prevKeyOne, prevKeyTwo) + " isn't referencing anywhere");
             break;
           }
@@ -1159,7 +1163,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
           }
 
           if (prevCount != currentCount) {
-            context.getCounter(Counts.BAD_UPDATE_COUNTS).increment(1);
+            context.getCounter(Updater.Counts.BAD_UPDATE_COUNTS).increment(1);
             LOG.warn(getStringFromKeys(prevKeyOne, prevKeyTwo) + " has a wrong updateCount, " +
                 prevCount + " instead of " + currentCount);
             // Game over, there's corruption.
@@ -1167,14 +1171,14 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
           }
 
           if (!prevClient.equals(client)) {
-            context.getCounter(Counts.BROKEN_LINKS).increment(1);
+            context.getCounter(Updater.Counts.BROKEN_LINKS).increment(1);
             LOG.warn(getStringFromKeys(prevKeyOne, prevKeyTwo) + " has the wrong client, " +
                 "bad reference? Bad client= " + prevClient);
             break;
           }
 
           updateRow(prevKeyOne, prevKeyTwo, newCount);
-          context.getCounter(Counts.UPDATED_NODES).increment(1);
+          context.getCounter(Updater.Counts.UPDATED_NODES).increment(1);
           if (prevKeyOne % 10 == 0) {
             context.progress();
           }
@@ -1183,7 +1187,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
         } while (headKeyOne != prevKeyOne && headKeyTwo != prevKeyTwo);
 
         updateStatCounters(context, newCount);
-        context.getCounter(Counts.UPDATED_LINKS).increment(1);
+        context.getCounter(Updater.Counts.UPDATED_LINKS).increment(1);
       }
 
       /**
@@ -1216,19 +1220,19 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
             // TODO We didn't event get the first node?
             break;
           case 1:
-            context.getCounter(Counts.FIRST_UPDATE).increment(1);
+            context.getCounter(Updater.Counts.FIRST_UPDATE).increment(1);
             break;
           case 2:
-            context.getCounter(Counts.SECOND_UPDATE).increment(1);
+            context.getCounter(Updater.Counts.SECOND_UPDATE).increment(1);
             break;
           case 3:
-            context.getCounter(Counts.THIRD_UPDATE).increment(1);
+            context.getCounter(Updater.Counts.THIRD_UPDATE).increment(1);
             break;
           case 4:
-            context.getCounter(Counts.FOURTH_UPDATE).increment(1);
+            context.getCounter(Updater.Counts.FOURTH_UPDATE).increment(1);
             break;
           default:
-            context.getCounter(Counts.MORE_THAN_FOUR_UPDATES).increment(1);
+            context.getCounter(Updater.Counts.MORE_THAN_FOUR_UPDATES).increment(1);
             break;
         }
       }
@@ -1240,6 +1244,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
       }
     }
 
+    @SuppressWarnings("deprecation")
     public int run(long maxLinkUpdatesPerMapper) throws Exception {
       LOG.info("Running Updater with maxLinkUpdatesPerMapper=" + maxLinkUpdatesPerMapper);
 
@@ -1279,8 +1284,8 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
 
       if (success) {
         // Let's not continue looping if we have broken linked lists.
-        Counter brokenLinks = counters.findCounter(Counts.BROKEN_LINKS);
-        Counter badUpdates = counters.findCounter(Counts.BAD_UPDATE_COUNTS);
+        Counter brokenLinks = counters.findCounter(Updater.Counts.BROKEN_LINKS);
+        Counter badUpdates = counters.findCounter(Updater.Counts.BAD_UPDATE_COUNTS);
         if (brokenLinks.getValue() > 0 || badUpdates.getValue() > 0) {
           LOG.error("Corruption was detected, see the job's counters. Ending the update loop.");
           success = false;
@@ -1323,9 +1328,8 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
     }
   }
 
-  /**
-   * A stand alone program that deletes a single node.
-   * TODO
+  /*
+   * TODO: A stand alone program that deletes a single node.
    */
   /*private static class Delete extends Configured implements Tool {
     @Override
@@ -1464,7 +1468,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
   }
 
   private static void printCINodeString(CINode node) {
-    System.out.printf("%s:%s:%012d:%s:%s\n", node.key, node.prev, node.rowId, node.client,
+    System.out.printf("%s:%s:%012d:%s:%s%n", node.key, node.prev, node.rowId, node.client,
         node.updateCount);
   }
 
@@ -1487,7 +1491,7 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
   private void usage() {
     System.err.println("Usage: " + this.getClass().getSimpleName() + " COMMAND [COMMAND options]");
     System.err.println("  where COMMAND is one of:");
-    System.err.println("");
+    System.err.println();
     System.err.println("  Generator                  A map only job that generates data.");
     System.err.println("  Verify                     A map reduce job that looks for holes");
     System.err.println("                             Look at the counts after running");
@@ -1521,24 +1525,30 @@ public class IntegrationTestBigLinkedList extends Configured implements Tool {
   public int run(String[] args) throws Exception {
     Tool tool;
     processOptions(args);
-    if (toRun.equals("Generator")) {
-      tool = new Generator();
-    } else if (toRun.equals("Verify")) {
-      tool = new Verify();
-    } else if (toRun.equals("Loop")) {
-      Loop loop = new Loop();
-      loop.it = this;
-      tool = loop;
-
-    } else if (toRun.equals("Print")) {
-      tool = new Print();
-    } else if (toRun.equals("Update")) {
-      tool = new Updater();
-    } else if (toRun.equals("Walker")) {
-      tool = new Walker();
-    } else {
-      usage();
-      throw new RuntimeException("Unknown arg");
+    switch (toRun) {
+      case "Generator":
+        tool = new Generator();
+        break;
+      case "Verify":
+        tool = new Verify();
+        break;
+      case "Loop":
+        Loop loop = new Loop();
+        loop.it = this;
+        tool = loop;
+        break;
+      case "Print":
+        tool = new Print();
+        break;
+      case "Update":
+        tool = new Updater();
+        break;
+      case "Walker":
+        tool = new Walker();
+        break;
+      default:
+        usage();
+        throw new RuntimeException("Unknown arg");
     }
 
     return ToolRunner.run(getConf(), tool, otherArgs);
