@@ -29,6 +29,7 @@ package org.apache.kudu.client;
 import static com.google.common.base.Preconditions.checkArgument;
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.kudu.tserver.Tserver.NewScanRequestPB;
+import static org.apache.kudu.tserver.Tserver.ResourceMetricsPB;
 import static org.apache.kudu.tserver.Tserver.ScanRequestPB;
 import static org.apache.kudu.tserver.Tserver.ScanResponsePB;
 import static org.apache.kudu.tserver.Tserver.TabletServerErrorPB;
@@ -221,6 +222,8 @@ public final class AsyncKuduScanner {
 
   private boolean reuseRowResult = false;
 
+  private final ResourceMetrics resourceMetrics = new ResourceMetrics();
+
   private boolean closed = false;
 
   private boolean hasMore = true;
@@ -308,7 +311,7 @@ public final class AsyncKuduScanner {
 
     // Map the column names to actual columns in the table schema.
     // If the user set this to 'null', we scan all columns.
-    List<ColumnSchema> columns = new ArrayList<ColumnSchema>();
+    List<ColumnSchema> columns = new ArrayList<>();
     if (projectedNames != null) {
       for (String columnName : projectedNames) {
         ColumnSchema originalColumn = table.getSchema().getColumn(columnName);
@@ -355,12 +358,12 @@ public final class AsyncKuduScanner {
    * @return a ColumnSchema for the virtual IS_DELETED column
    */
   private static ColumnSchema generateIsDeletedColumn(Schema schema) {
-    String columnName = DEFAULT_IS_DELETED_COL_NAME;
+    StringBuilder columnName = new StringBuilder(DEFAULT_IS_DELETED_COL_NAME);
     // If the column already exists and we need to pick an alternate column name.
-    while (schema.hasColumn(columnName)) {
-      columnName += "_";
+    while (schema.hasColumn(columnName.toString())) {
+      columnName.append("_");
     }
-    return new ColumnSchema.ColumnSchemaBuilder(columnName, Type.BOOL)
+    return new ColumnSchema.ColumnSchemaBuilder(columnName.toString(), Type.BOOL)
             .wireType(Common.DataType.IS_DELETED)
             .defaultValue(false)
             .nullable(false)
@@ -452,6 +455,15 @@ public final class AsyncKuduScanner {
     return this.startTimestamp;
   }
 
+  /**
+   * Returns the {@code ResourceMetrics} for this scanner. These metrics are
+   * updated with each batch of rows returned from the server.
+   * @return the resource metrics for this scanner
+   */
+  public ResourceMetrics getResourceMetrics() {
+    return this.resourceMetrics;
+  }
+
   long getSnapshotTimestamp() {
     return this.htTimestamp;
   }
@@ -520,6 +532,9 @@ public final class AsyncKuduScanner {
           }
 
           numRowsReturned += resp.data.getNumRows();
+          if (resp.resourceMetricsPb != null) {
+            resourceMetrics.update(resp.resourceMetricsPb);
+          }
 
           if (!resp.more || resp.scannerId == null) {
             scanFinished();
@@ -561,7 +576,7 @@ public final class AsyncKuduScanner {
           } else {
             LOG.debug("Can not open scanner", e);
             // Don't let the scanner think it's opened on this tablet.
-            return Deferred.fromError(e); // Let the error propogate.
+            return Deferred.fromError(e); // Let the error propagate.
           }
         }
 
@@ -877,18 +892,22 @@ public final class AsyncKuduScanner {
 
     private final byte[] lastPrimaryKey;
 
+    private final ResourceMetricsPB resourceMetricsPb;
+
     Response(final byte[] scannerId,
              final RowResultIterator data,
              final boolean more,
              final long scanTimestamp,
              final long propagatedTimestamp,
-             final byte[] lastPrimaryKey) {
+             final byte[] lastPrimaryKey,
+             final ResourceMetricsPB resourceMetricsPb) {
       this.scannerId = scannerId;
       this.data = data;
       this.more = more;
       this.scanTimestamp = scanTimestamp;
       this.propagatedTimestamp = propagatedTimestamp;
       this.lastPrimaryKey = lastPrimaryKey;
+      this.resourceMetricsPb = resourceMetricsPb;
     }
 
     @Override
@@ -955,7 +974,7 @@ public final class AsyncKuduScanner {
       readProtobuf(callResponse.getPBMessage(), builder);
       ScannerKeepAliveResponsePB resp = builder.build();
       TabletServerErrorPB error = resp.hasError() ? resp.getError() : null;
-      return new Pair<Void, Object>(null, error);
+      return new Pair<>(null, error);
     }
   }
 
@@ -1104,7 +1123,7 @@ public final class AsyncKuduScanner {
           case TABLET_NOT_RUNNING:
             if (state == State.OPENING || (state == State.NEXT && isFaultTolerant)) {
               // Doing this will trigger finding the new location.
-              return new Pair<Response, Object>(null, error);
+              return new Pair<>(null, error);
             } else {
               Status statusIncomplete = Status.Incomplete("Cannot continue scanning, " +
                   "the tablet has moved and this isn't a fault tolerant scan");
@@ -1122,7 +1141,8 @@ public final class AsyncKuduScanner {
       }
       // TODO: Find a clean way to plumb in reuseRowResult.
       RowResultIterator iterator = RowResultIterator.makeRowResultIterator(
-          timeoutTracker.getElapsedMillis(), tsUUID, schema, resp.getData(), callResponse, reuseRowResult);
+          timeoutTracker.getElapsedMillis(), tsUUID, schema, resp.getData(), callResponse,
+          reuseRowResult);
 
       boolean hasMore = resp.getHasMoreResults();
       if (id.length != 0 && scannerId != null && !Bytes.equals(scannerId, id)) {
@@ -1131,16 +1151,18 @@ public final class AsyncKuduScanner {
             Bytes.pretty(scannerId));
         throw new NonRecoverableException(statusIllegalState);
       }
+      ResourceMetricsPB resourceMetricsPB = resp.hasResourceMetrics() ?
+          resp.getResourceMetrics() : null;
       Response response = new Response(id, iterator, hasMore,
           resp.hasSnapTimestamp() ? resp.getSnapTimestamp()
                                   : AsyncKuduClient.NO_TIMESTAMP,
           resp.hasPropagatedTimestamp() ? resp.getPropagatedTimestamp()
                                         : AsyncKuduClient.NO_TIMESTAMP,
-          resp.getLastPrimaryKey().toByteArray());
+          resp.getLastPrimaryKey().toByteArray(), resourceMetricsPB);
       if (LOG.isDebugEnabled()) {
         LOG.debug("{} for scanner {}", response.toString(), AsyncKuduScanner.this);
       }
-      return new Pair<Response, Object>(response, error);
+      return new Pair<>(response, error);
     }
 
     @Override
