@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+
 package org.apache.kudu.client;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
@@ -21,6 +22,7 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
 import java.math.BigDecimal;
+import java.sql.Date;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -37,6 +39,8 @@ import org.apache.kudu.Type;
 import org.apache.kudu.client.PartitionSchema.HashBucketSchema;
 import org.apache.kudu.client.PartitionSchema.RangeSchema;
 import org.apache.kudu.test.KuduTestHarness;
+import org.apache.kudu.util.CharUtil;
+import org.apache.kudu.util.DateUtil;
 import org.apache.kudu.util.DecimalUtil;
 
 public class TestKeyEncoding {
@@ -91,7 +95,7 @@ public class TestKeyEncoding {
     }
     return new PartitionSchema(
         new PartitionSchema.RangeSchema(columnIds),
-        ImmutableList.<PartitionSchema.HashBucketSchema>of(), schema);
+        ImmutableList.of(), schema);
   }
 
   /**
@@ -186,6 +190,8 @@ public class TestKeyEncoding {
             .typeAttributes(DecimalUtil.typeAttributes(DecimalUtil.MAX_DECIMAL64_PRECISION, 0)),
         new ColumnSchemaBuilder("decimal128", Type.DECIMAL).key(true)
             .typeAttributes(DecimalUtil.typeAttributes(DecimalUtil.MAX_DECIMAL128_PRECISION, 0)),
+        new ColumnSchemaBuilder("varchar", Type.VARCHAR).key(true)
+            .typeAttributes(CharUtil.typeAttributes(10)),
         new ColumnSchemaBuilder("string", Type.STRING).key(true),
         new ColumnSchemaBuilder("binary", Type.BINARY).key(true));
 
@@ -199,6 +205,7 @@ public class TestKeyEncoding {
     rowA.addDecimal("decimal32", BigDecimal.valueOf(5));
     rowA.addDecimal("decimal64", BigDecimal.valueOf(6));
     rowA.addDecimal("decimal128", BigDecimal.valueOf(7));
+    rowA.addVarchar("varchar", "");
     rowA.addString("string", "");
     rowA.addBinary("binary", "".getBytes(UTF_8));
 
@@ -212,6 +219,7 @@ public class TestKeyEncoding {
                           (byte) 0x80, 0, 0, 5,
                           (byte) 0x80, 0, 0, 0, 0, 0, 0, 6,
                           (byte) 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7,
+                          0, 0,
                           0, 0
                       });
     assertEquals(rowA.stringifyRowKey(),
@@ -227,6 +235,7 @@ public class TestKeyEncoding {
     rowB.addDecimal("decimal32", BigDecimal.valueOf(5));
     rowB.addDecimal("decimal64", BigDecimal.valueOf(6));
     rowB.addDecimal("decimal128", BigDecimal.valueOf(7));
+    rowB.addVarchar("varchar", "abc\1\0defghij");
     rowB.addString("string", "abc\1\0def");
     rowB.addBinary("binary", "\0\1binary".getBytes(UTF_8));
 
@@ -240,6 +249,7 @@ public class TestKeyEncoding {
                           (byte) 0x80, 0, 0, 5,
                           (byte) 0x80, 0, 0, 0, 0, 0, 0, 6,
                           (byte) 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7,
+                          'a', 'b', 'c', 1, 0, 1, 'd', 'e', 'f', 'g', 'h', 0, 0,
                           'a', 'b', 'c', 1, 0, 1, 'd', 'e', 'f', 0, 0,
                           0, 1, 'b', 'i', 'n', 'a', 'r', 'y',
                       });
@@ -254,6 +264,7 @@ public class TestKeyEncoding {
     rowC.addDecimal("decimal32", BigDecimal.valueOf(5));
     rowC.addDecimal("decimal64", BigDecimal.valueOf(6));
     rowC.addDecimal("decimal128", BigDecimal.valueOf(7));
+    rowC.addVarchar("varchar", "abc\n12345678");
     rowC.addString("string", "abc\n123");
     rowC.addBinary("binary", "\0\1\2\3\4\5".getBytes(UTF_8));
 
@@ -267,6 +278,7 @@ public class TestKeyEncoding {
                           (byte) 0x80, 0, 0, 5,
                           (byte) 0x80, 0, 0, 0, 0, 0, 0, 6,
                           (byte) 0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7,
+                          'a', 'b', 'c', '\n', '1', '2', '3', '4', '5', '6', 0, 0,
                           'a', 'b', 'c', '\n', '1', '2', '3', 0, 0,
                           0, 1, 2, 3, 4, 5,
                       });
@@ -281,6 +293,7 @@ public class TestKeyEncoding {
     rowD.addDecimal("decimal32", BigDecimal.valueOf(-5));
     rowD.addDecimal("decimal64", BigDecimal.valueOf(-6));
     rowD.addDecimal("decimal128", BigDecimal.valueOf(-7));
+    rowD.addVarchar("varchar", "\0abc\n\1\1\0 123\1\0");
     rowD.addString("string", "\0abc\n\1\1\0 123\1\0");
     rowD.addBinary("binary", "\0\1\2\3\4\5\0".getBytes(UTF_8));
 
@@ -294,6 +307,7 @@ public class TestKeyEncoding {
                           (byte) 127, -1, -1, -5,
                           (byte) 127, -1, -1, -1, -1, -1, -1, -6,
                           (byte) 127, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -7,
+                          0, 1, 'a', 'b', 'c', '\n', 1, 1, 0, 1, ' ', '1', 0, 0,
                           0, 1, 'a', 'b', 'c', '\n', 1, 1, 0, 1, ' ', '1', '2', '3', 1, 0, 1, 0, 0,
                           0, 1, 2, 3, 4, 5, 0,
                       });
@@ -308,7 +322,7 @@ public class TestKeyEncoding {
         new ColumnSchemaBuilder("b", Type.STRING).key(true),
         new ColumnSchemaBuilder("c", Type.STRING).key(true));
 
-    PartitionSchema partitionSchema =
+    final PartitionSchema partitionSchema =
         new PartitionSchema(new RangeSchema(ImmutableList.of(0, 1, 2)),
                             ImmutableList.of(
                                 new HashBucketSchema(ImmutableList.of(0, 1), 32, 0),
@@ -382,29 +396,34 @@ public class TestKeyEncoding {
           .typeAttributes(DecimalUtil.typeAttributes(DecimalUtil.MAX_DECIMAL64_PRECISION, 0)),
         new ColumnSchemaBuilder("decimal128", Type.DECIMAL).key(true)
           .typeAttributes(DecimalUtil.typeAttributes(DecimalUtil.MAX_DECIMAL128_PRECISION, 0)),
+        new ColumnSchemaBuilder("varchar", Type.VARCHAR).key(true)
+          .typeAttributes(CharUtil.typeAttributes(10)),
+        new ColumnSchemaBuilder("date", Type.DATE).key(true),
         new ColumnSchemaBuilder("bool", Type.BOOL),       // not primary key type
         new ColumnSchemaBuilder("float", Type.FLOAT),     // not primary key type
         new ColumnSchemaBuilder("double", Type.DOUBLE));  // not primary key type
 
     KuduTable table = client.createTable("testAllPrimaryKeyTypes-" + System.currentTimeMillis(),
         schema, defaultCreateTableOptions(schema));
-    KuduSession session = client.newSession();
+    final KuduSession session = client.newSession();
 
     Insert insert = table.newInsert();
     PartialRow row = insert.getRow();
     row.addByte(0, (byte) 1);
     row.addShort(1, (short) 2);
     row.addInt(2, 3);
-    row.addLong(3, 4l);
+    row.addLong(3, 4L);
     row.addString(4, "foo");
     row.addBinary(5, "bar".getBytes(UTF_8));
-    row.addLong(6, 6l);
+    row.addLong(6, 6L);
     row.addDecimal(7, BigDecimal.valueOf(DecimalUtil.MAX_UNSCALED_DECIMAL32));
     row.addDecimal(8, BigDecimal.valueOf(DecimalUtil.MAX_UNSCALED_DECIMAL64));
     row.addDecimal(9, new BigDecimal(DecimalUtil.MAX_UNSCALED_DECIMAL128));
-    row.addBoolean(10, true);
-    row.addFloat(11, 8.8f);
-    row.addDouble(12, 9.9);
+    row.addVarchar(10, "varchar bar");
+    row.addDate(11, DateUtil.epochDaysToSqlDate(0));
+    row.addBoolean(12, true);
+    row.addFloat(13, 7.8f);
+    row.addDouble(14, 9.9);
     session.apply(insert);
     session.close();
 
@@ -416,19 +435,21 @@ public class TestKeyEncoding {
       assertEquals((byte) 0x01, rr.getByte(0));
       assertEquals((short) 2, rr.getShort(1));
       assertEquals(3, rr.getInt(2));
-      assertEquals(4l, rr.getLong(3));
+      assertEquals(4L, rr.getLong(3));
       assertBytesEquals(rr.getBinaryCopy(4), "foo");
       assertBytesEquals(rr.getBinaryCopy(5), "bar");
-      assertEquals(6l, rr.getLong(6));
+      assertEquals(6L, rr.getLong(6));
       assertTrue(BigDecimal.valueOf(DecimalUtil.MAX_UNSCALED_DECIMAL32)
           .compareTo(rr.getDecimal(7)) == 0);
       assertTrue(BigDecimal.valueOf(DecimalUtil.MAX_UNSCALED_DECIMAL64)
           .compareTo(rr.getDecimal(8)) == 0);
       assertTrue(new BigDecimal(DecimalUtil.MAX_UNSCALED_DECIMAL128)
           .compareTo(rr.getDecimal(9)) == 0);
-      assertTrue(rr.getBoolean(10));
-      assertEquals(8.8f, rr.getFloat(11), .001f);
-      assertEquals(9.9, rr.getDouble(12), .001);
+      assertEquals("varchar ba", rr.getVarchar(10));
+      assertEquals(0, rr.getInt(11));
+      assertTrue(rr.getBoolean(12));
+      assertEquals(7.8f, rr.getFloat(13), .001f);
+      assertEquals(9.9, rr.getDouble(14), .001);
     }
   }
 }
