@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+
 package org.apache.kudu.client;
 
 import static org.apache.kudu.client.AsyncKuduScanner.DEFAULT_IS_DELETED_COL_NAME;
@@ -123,7 +124,7 @@ public class TestKuduScanner {
       "--scanner_gc_check_interval_us=500000"}) // 10% of the TTL.
   public void testKeepAlive() throws Exception {
     int rowCount = 500;
-    int shortScannerTtlMs = 5000;
+    long shortScannerTtlMs = 5000;
 
     // Create a simple table with a single partition.
     Schema tableSchema = new Schema(Collections.singletonList(
@@ -153,7 +154,7 @@ public class TestKuduScanner {
     // Test that a keepAlivePeriodMs greater than the scanner ttl fails.
     KuduScanner badScanner = client.newScannerBuilder(table)
         .batchSizeBytes(100) // Set a small batch size so the first scan doesn't read all the rows.
-        .keepAlivePeriodMs(shortScannerTtlMs * 2)
+        .keepAlivePeriodMs(shortScannerTtlMs * 2L)
         .build();
     try {
       processKeepAliveScanner(badScanner, shortScannerTtlMs);
@@ -163,10 +164,12 @@ public class TestKuduScanner {
     }
   }
 
-  private void processKeepAliveScanner(KuduScanner scanner, int scannerTtlMs) throws Exception {
+  private void processKeepAliveScanner(KuduScanner scanner, long scannerTtlMs) throws Exception {
     int i = 0;
+    KuduScannerIterator iterator = scanner.iterator();
     // Ensure reading takes longer than the scanner ttl.
-    for (RowResult unused : scanner) {
+    while (iterator.hasNext()) {
+      iterator.next();
       // Sleep for half the ttl for the first few rows. This ensures
       // we are on the same tablet and will go past the ttl without
       // a new scan request. It also ensures a single row doesn't go
@@ -181,8 +184,8 @@ public class TestKuduScanner {
   @Test(timeout = 100000)
   public void testOpenScanWithDroppedPartition() throws Exception {
     // Create a table with 2 range partitions.
-    Schema basicSchema = getBasicSchema();
-    String tableName = "testOpenScanWithDroppedPartition";
+    final Schema basicSchema = getBasicSchema();
+    final String tableName = "testOpenScanWithDroppedPartition";
     PartialRow bottom = basicSchema.newPartialRow();
     bottom.addInt("key", 0);
     PartialRow middle = basicSchema.newPartialRow();
@@ -258,13 +261,14 @@ public class TestKuduScanner {
     long startHT = client.getLastPropagatedTimestamp() + 1;
     LOG.info("startHT: {}", startHT);
 
-    // Generate row mutations. The mutations performed here are what should be seen by the diff scan.
+    // Generate row mutations.
+    // The mutations performed here are what should be seen by the diff scan.
     int mutationBounds = 10;
     int expectedNumInserts = random.nextInt(mutationBounds);
     int expectedNumUpdates = random.nextInt(mutationBounds);
     int expectedNumDeletes = random.nextInt(mutationBounds);
-    List<Operation> operations =
-        generateMutationOperations(table, expectedNumInserts, expectedNumUpdates, expectedNumDeletes);
+    List<Operation> operations = generateMutationOperations(table, expectedNumInserts,
+                                                            expectedNumUpdates, expectedNumDeletes);
     Map<Integer, ChangeType> mutations = applyOperations(operations);
     LOG.info("Mutations: {}", mutations);
 
