@@ -39,7 +39,6 @@ import org.apache.hadoop.hive.metastore.api.SerDeInfo;
 import org.apache.hadoop.hive.metastore.api.StorageDescriptor;
 import org.apache.hadoop.hive.metastore.api.Table;
 import org.apache.hadoop.hive.metastore.api.hive_metastoreConstants;
-import org.apache.kudu.test.junit.RetryRule;
 import org.apache.thrift.TException;
 import org.junit.After;
 import org.junit.Before;
@@ -48,6 +47,8 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.kudu.test.junit.RetryRule;
+
 public class TestKuduMetastorePlugin {
   private static final Logger LOG = LoggerFactory.getLogger(TestKuduMetastorePlugin.class);
 
@@ -104,7 +105,7 @@ public class TestKuduMetastorePlugin {
         "jdbc:derby:memory:" + metadb.toString() + ";create=true");
     // Set the derby log file.
     Path derbyLogFile = hiveTestDir.resolve("derby.log");
-    derbyLogFile.toFile().createNewFile();
+    assertTrue(derbyLogFile.toFile().createNewFile());
     System.setProperty("derby.stream.error.file", derbyLogFile.toString());
 
     int msPort = MetaStoreUtils.startMetaStore(metastoreConf);
@@ -232,10 +233,10 @@ public class TestKuduMetastorePlugin {
   @Test
   public void testAlterTableHandler() throws Exception {
     // Test altering a Kudu (or a legacy) table.
-    Table table = newTable("table");
-    client.createTable(table, masterContext());
+    Table initTable = newTable("table");
+    client.createTable(initTable, masterContext());
     // Get the table from the HMS in case any translation occurred.
-    table = client.getTable(table.getDbName(), table.getTableName());
+    Table table = client.getTable(initTable.getDbName(), initTable.getTableName());
     Table legacyTable = newLegacyTable("legacy_table");
     client.createTable(legacyTable, masterContext());
     // Get the table from the HMS in case any translation occurred.
@@ -288,8 +289,7 @@ public class TestKuduMetastorePlugin {
         client.alter_table(table.getDbName(), table.getTableName(), alteredTable);
         fail();
       } catch (TException e) {
-        assertTrue(e.getMessage().contains(
-            "Kudu table type may not be altered"));
+        assertTrue(e.getMessage().contains("Kudu table type may not be altered"));
       }
 
       // Alter the Kudu table to a different type by setting the external property fails.
@@ -299,8 +299,7 @@ public class TestKuduMetastorePlugin {
         client.alter_table(table.getDbName(), table.getTableName(), alteredTable);
         fail();
       } catch (TException e) {
-        assertTrue(e.getMessage().contains(
-           "Kudu table type may not be altered"));
+        assertTrue(e.getMessage().contains("Kudu table type may not be altered"));
       }
 
       // Alter the Kudu table to the same type by setting the table property works.
@@ -332,8 +331,7 @@ public class TestKuduMetastorePlugin {
         client.alter_table(table.getDbName(), table.getTableName(), alteredTable);
         fail();
       } catch (TException e) {
-        assertTrue(e.getMessage().contains(
-            "Kudu table type may not be altered"));
+        assertTrue(e.getMessage().contains("Kudu table type may not be altered"));
       }
 
       // Alter the Kudu table to an external type with the master context succeeds.
@@ -352,8 +350,7 @@ public class TestKuduMetastorePlugin {
         client.alter_table(table.getDbName(), table.getTableName(), table);
         fail();
       } catch (TException e) {
-        assertTrue(e.getMessage().contains(
-            "Kudu table columns may not be altered through Hive"));
+        assertTrue(e.getMessage().contains("Kudu table columns may not be altered through Hive"));
       }
 
       // Check that adding a column succeeds with the master event property set.
@@ -383,8 +380,10 @@ public class TestKuduMetastorePlugin {
 
     // Test altering a non-Kudu table.
     {
+      table = initTable.deepCopy();
       table.getParameters().clear();
       client.createTable(table);
+      table = client.getTable(table.getDbName(), table.getTableName());
       try {
 
         // Try to alter the table and add a Kudu table ID.
@@ -434,11 +433,12 @@ public class TestKuduMetastorePlugin {
 
     // Test altering an unsynchronized table is accepted.
     {
-      table.getParameters().clear();
+      table = initTable.deepCopy();
       table.setTableType(TableType.EXTERNAL_TABLE.name());
       table.putToParameters(KuduMetastorePlugin.EXTERNAL_TABLE_KEY, "TRUE");
       table.putToParameters(KuduMetastorePlugin.EXTERNAL_PURGE_KEY, "FALSE");
       client.createTable(table);
+      table = client.getTable(table.getDbName(), table.getTableName());
       try {
         client.alter_table(table.getDbName(), table.getTableName(), table);
       } finally {
