@@ -1,23 +1,24 @@
-/*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
+// Licensed to the Apache Software Foundation (ASF) under one
+// or more contributor license agreements.  See the NOTICE file
+// distributed with this work for additional information
+// regarding copyright ownership.  The ASF licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
+//
+//   http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
 
 package org.apache.kudu.spark.kudu
 
 import java.net.InetAddress
+import java.util.Locale
 
 import scala.collection.JavaConverters._
 import scala.util.Try
@@ -68,6 +69,7 @@ class DefaultSource
   val BATCH_SIZE = "kudu.batchSize"
   val KEEP_ALIVE_PERIOD_MS = "kudu.keepAlivePeriodMs"
   val SPLIT_SIZE_BYTES = "kudu.splitSizeBytes"
+  val HANDLE_SCHEMA_DRIFT = "kudu.handleSchemaDrift"
 
   /**
    * A nice alias for the data source so that when specifying the format
@@ -202,7 +204,14 @@ class DefaultSource
       parameters.get(REPARTITION).map(_.toBoolean).getOrElse(defaultRepartition)
     val repartitionSort =
       parameters.get(REPARTITION_SORT).map(_.toBoolean).getOrElse(defaultRepartitionSort)
-    KuduWriteOptions(ignoreDuplicateRowErrors, ignoreNull, repartition, repartitionSort)
+    val handleSchemaDrift =
+      parameters.get(HANDLE_SCHEMA_DRIFT).map(_.toBoolean).getOrElse(defaultHandleSchemaDrift)
+    KuduWriteOptions(
+      ignoreDuplicateRowErrors,
+      ignoreNull,
+      repartition,
+      repartitionSort,
+      handleSchemaDrift)
   }
 
   private def getMasterAddrs(parameters: Map[String, String]): String = {
@@ -210,7 +219,7 @@ class DefaultSource
   }
 
   private def getScanLocalityType(opParam: String): ReplicaSelection = {
-    opParam.toLowerCase match {
+    opParam.toLowerCase(Locale.ENGLISH) match {
       case "leader_only" => ReplicaSelection.LEADER_ONLY
       case "closest_replica" => ReplicaSelection.CLOSEST_REPLICA
       case _ =>
@@ -223,7 +232,7 @@ class DefaultSource
   }
 
   private def stringToOperationType(opParam: String): OperationType = {
-    opParam.toLowerCase match {
+    opParam.toLowerCase(Locale.ENGLISH) match {
       case "insert" => Insert
       case "insert-ignore" => Insert
       case "upsert" => Upsert
