@@ -1,19 +1,19 @@
-/*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
+// Licensed to the Apache Software Foundation (ASF) under one
+// or more contributor license agreements.  See the NOTICE file
+// distributed with this work for additional information
+// regarding copyright ownership.  The ASF licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
+//
+//   http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
 
 package org.apache.kudu.spark.kudu
 
@@ -45,6 +45,7 @@ import org.slf4j.Logger
 import org.slf4j.LoggerFactory
 import org.apache.kudu.client.SessionConfiguration.FlushMode
 import org.apache.kudu.client._
+import org.apache.kudu.spark.kudu.SparkUtil.kuduSchema
 import org.apache.kudu.spark.kudu.SparkUtil._
 import org.apache.kudu.Schema
 import org.apache.kudu.Type
@@ -58,6 +59,7 @@ import org.apache.kudu.Type
  */
 @InterfaceAudience.Public
 @InterfaceStability.Evolving
+@SerialVersionUID(1L)
 class KuduContext(val kuduMaster: String, sc: SparkContext, val socketReadTimeoutMs: Option[Long])
     extends Serializable {
   val log: Logger = LoggerFactory.getLogger(getClass)
@@ -129,7 +131,7 @@ class KuduContext(val kuduMaster: String, sc: SparkContext, val socketReadTimeou
   val durationHistogram = new HdrHistogramAccumulator()
   sc.register(durationHistogram, "kudu.write_duration")
 
-  @Deprecated()
+  @deprecated("Use KuduContext constructor", "1.4.0")
   def this(kuduMaster: String) {
     this(kuduMaster, new SparkContext())
   }
@@ -268,7 +270,8 @@ class KuduContext(val kuduMaster: String, sc: SparkContext, val socketReadTimeou
    * @param tableName the Kudu table to insert into
    */
   @deprecated(
-    "Use KuduContext.insertRows(data, tableName, new KuduWriteOptions(ignoreDuplicateRowErrors = true))")
+    "Use KuduContext.insertRows(data, tableName, new KuduWriteOptions(ignoreDuplicateRowErrors = true))",
+    "1.8.0")
   def insertIgnoreRows(data: DataFrame, tableName: String): Unit = {
     val writeOptions = KuduWriteOptions(ignoreDuplicateRowErrors = true)
     log.info(s"inserting into table '$tableName'")
@@ -334,6 +337,31 @@ class KuduContext(val kuduMaster: String, sc: SparkContext, val socketReadTimeou
     // Get the client's last propagated timestamp on the driver.
     val lastPropagatedTimestamp = syncClient.getLastPropagatedTimestamp
 
+    if (writeOptions.handleSchemaDrift) {
+      val kuduSchema = syncClient.openTable(tableName).getSchema
+      val newColumns = schema.fields.filter(f => !kuduSchema.hasColumn(f.name))
+      if (!newColumns.isEmpty) {
+        log.info(
+          s"adding ${newColumns.length} columns to table '$tableName' to handle schema drift")
+        val alter = new AlterTableOptions()
+        newColumns.foreach { col =>
+          alter.addNullableColumn(col.name, sparkTypeToKuduType(col.dataType))
+        }
+        try {
+          syncClient.alterTable(tableName, alter)
+        } catch {
+          case e: KuduException =>
+            // Ignore the exception if the column already exists due to concurrent
+            // applications attempting to handle schema drift.
+            if (e.getStatus.isAlreadyPresent) {
+              log.info(s"column already exists in table '$tableName' while handling schema drift")
+            } else {
+              throw e
+            }
+        }
+      }
+    }
+
     // Convert to an RDD and map the InternalRows to Rows.
     // This avoids any corruption as reported in SPARK-26880.
     var rdd = data.queryExecution.toRdd.mapPartitions { rows =>
@@ -453,7 +481,7 @@ class KuduContext(val kuduMaster: String, sc: SparkContext, val socketReadTimeou
       // timestamp on each executor.
       timestampAccumulator.add(syncClient.getLastPropagatedTimestamp)
       addForOperation(numRows, opType)
-      val elapsedTime = System.currentTimeMillis() - startTime
+      val elapsedTime = (System.currentTimeMillis() - startTime).toInt
       durationHistogram.add(elapsedTime)
       log.info(s"applied $numRows ${opType}s to table '$tableName' in ${elapsedTime}ms")
     }
@@ -468,7 +496,7 @@ class KuduContext(val kuduMaster: String, sc: SparkContext, val socketReadTimeou
 }
 
 private object KuduContext {
-  val Log: Logger = LoggerFactory.getLogger(classOf[KuduContext])
+  val log: Logger = LoggerFactory.getLogger(classOf[KuduContext])
 
   /**
    * Returns a new Kerberos-authenticated [[Subject]] if the Spark context contains
@@ -494,7 +522,7 @@ private object KuduContext {
     val keytab =
       sc.getConf.getOption("spark.yarn.keytab").getOrElse(return subject)
 
-    Log.info(s"Logging in as principal $principal with keytab $keytab")
+    log.info(s"Logging in as principal $principal with keytab $keytab")
 
     val conf = new Configuration {
       override def getAppConfigurationEntry(name: String): Array[AppConfigurationEntry] = {
@@ -522,7 +550,7 @@ private object KuduContext {
 }
 
 private object KuduClientCache {
-  val Log: Logger = LoggerFactory.getLogger(KuduClientCache.getClass)
+  val log: Logger = LoggerFactory.getLogger(KuduClientCache.getClass)
 
   private case class CacheValue(kuduClient: AsyncKuduClient, shutdownHookHandle: Runnable)
 
@@ -544,7 +572,7 @@ private object KuduClientCache {
         try {
           cacheValue.kuduClient.close()
         } catch {
-          case e: Exception => Log.warn("Error while shutting down the test client", e);
+          case e: Exception => log.warn("Error while shutting down the test client", e);
         }
 
         // A client may only be closed once, so once we've close this client,
