@@ -1,25 +1,30 @@
-/*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
+// Licensed to the Apache Software Foundation (ASF) under one
+// or more contributor license agreements.  See the NOTICE file
+// distributed with this work for additional information
+// regarding copyright ownership.  The ASF licenses this file
+// to you under the Apache License, Version 2.0 (the
+// "License"); you may not use this file except in compliance
+// with the License.  You may obtain a copy of the License at
+//
+//   http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing,
+// software distributed under the License is distributed on an
+// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+// KIND, either express or implied.  See the License for the
+// specific language governing permissions and limitations
+// under the License.
+
 package org.apache.kudu.spark.kudu
 
+import java.nio.charset.StandardCharsets
+
 import scala.collection.JavaConverters._
 import scala.collection.immutable.IndexedSeq
+import org.apache.spark.SparkException
 import org.apache.spark.sql.Row
 import org.apache.spark.sql.SQLContext
+import org.apache.spark.sql.SaveMode
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.types.DataTypes
 import org.apache.spark.sql.types.StructField
@@ -62,7 +67,7 @@ class DefaultSourceTest extends KuduTestSuite with Matchers {
    * syntax still works. This should be removed when the
    * deprecated `kudu` methods are removed.
    */
-  @Test
+  @Test @deprecated("Marked as deprecated to suppress warning", "")
   def testPackageReaderAndWriter(): Unit = {
     val df = sqlContext.read.options(kuduOptions).kudu
     val baseDF = df.limit(1) // filter down to just the first row
@@ -276,7 +281,7 @@ class DefaultSourceTest extends KuduTestSuite with Matchers {
     deleteRow(100)
   }
 
-  @Test
+  @Test @deprecated("Marked as deprecated to suppress warning", "")
   def testInsertIgnoreRowsMethod() {
     val df = sqlContext.read.options(kuduOptions).format("kudu").load
     val baseDF = df.limit(1) // filter down to just the first row
@@ -337,6 +342,42 @@ class DefaultSourceTest extends KuduTestSuite with Matchers {
     deleteRow(100)
   }
 
+  @Test
+  def testWriteWithSink() {
+    val df = sqlContext.read.options(kuduOptions).format("kudu").load
+    val baseDF = df.limit(1) // Filter down to just the first row.
+
+    // Change the c2 string to abc and upsert.
+    val upsertDF = baseDF.withColumn("c2_s", lit("abc"))
+    upsertDF.write
+      .format("kudu")
+      .option("kudu.master", harness.getMasterAddressesAsString)
+      .option("kudu.table", tableName)
+      // Default kudu.operation is upsert.
+      .mode(SaveMode.Append)
+      .save()
+
+    // Change the key and insert.
+    val insertDF = df
+      .limit(1)
+      .withColumn("key", df("key").plus(100))
+      .withColumn("c2_s", lit("def"))
+    insertDF.write
+      .format("kudu")
+      .option("kudu.master", harness.getMasterAddressesAsString)
+      .option("kudu.table", tableName)
+      .option("kudu.operation", "insert")
+      .mode(SaveMode.Append)
+      .save()
+
+    // Read the data back.
+    val newDF = sqlContext.read.options(kuduOptions).format("kudu").load
+    val collectedUpdate = newDF.filter("key = 0").collect()
+    assertEquals("abc", collectedUpdate(0).getAs[String]("c2_s"))
+    val collectedInsert = newDF.filter("key = 100").collect()
+    assertEquals("def", collectedInsert(0).getAs[String]("c2_s"))
+  }
+
   @Test
   def testUpsertRowsIgnoreNulls() {
     val nonNullDF =
@@ -519,6 +560,62 @@ class DefaultSourceTest extends KuduTestSuite with Matchers {
     assert(checkDf.count == 10)
   }
 
+  @Test
+  def testSchemaDrift() {
+    val nonNullDF =
+      sqlContext.createDataFrame(Seq((0, "foo"))).toDF("key", "val")
+    kuduContext.insertRows(nonNullDF, simpleTableName)
+
+    val tableOptions = Map(
+      "kudu.master" -> harness.getMasterAddressesAsString,
+      "kudu.table" -> simpleTableName
+    )
+    val df = sqlContext.read.options(tableOptions).format("kudu").load
+    assertEquals(2, df.schema.fields.length)
+
+    // Add a column not in the table schema by duplicating the val column.
+    val newDf = df.withColumn("val2", col("val"))
+
+    // Insert with handleSchemaDrift = false. Note that a new column was not created.
+    kuduContext.upsertRows(newDf, simpleTableName, KuduWriteOptions(handleSchemaDrift = false))
+    assertEquals(2, harness.getClient.openTable(simpleTableName).getSchema.getColumns.size())
+
+    // Insert with handleSchemaDrift = true. Note that a new column was created.
+    kuduContext.upsertRows(newDf, simpleTableName, KuduWriteOptions(handleSchemaDrift = true))
+    assertEquals(3, harness.getClient.openTable(simpleTableName).getSchema.getColumns.size())
+
+    val afterDf = sqlContext.read.options(tableOptions).format("kudu").load
+    assertEquals(3, afterDf.schema.fields.length)
+    assertEquals("val2", afterDf.schema.fieldNames.last)
+    assertTrue(afterDf.collect().forall(r => r.getString(1) == r.getString(2)))
+  }
+
+  @Test
+  def testInsertWrongType() {
+    val nonNullDF =
+      sqlContext.createDataFrame(Seq((0, "foo"))).toDF("key", "val")
+    kuduContext.insertRows(nonNullDF, simpleTableName)
+
+    val tableOptions = Map(
+      "kudu.master" -> harness.getMasterAddressesAsString,
+      "kudu.table" -> simpleTableName
+    )
+    val df = sqlContext.read.options(tableOptions).format("kudu").load
+    // Convert the val column to a bytes instead of string.
+    val toBytes = udf[Array[Byte], String](_.getBytes(StandardCharsets.UTF_8))
+    val newDf = df
+      .withColumn("valTmp", toBytes(col("val")))
+      .drop("val")
+      .withColumnRenamed("valTmp", "val")
+
+    try {
+      kuduContext.insertRows(newDf, simpleTableName, KuduWriteOptions())
+    } catch {
+      case e: SparkException =>
+        assertTrue(e.getMessage.contains("val isn't [Type: binary], it's string"))
+    }
+  }
+
   @Test
   def testCreateRelationWithSchema() {
     // user-supplied schema that is compatible with actual schema, but with the key at the end
@@ -529,7 +626,7 @@ class DefaultSourceTest extends KuduTestSuite with Matchers {
       ))
 
     val dfDefaultSchema = sqlContext.read.options(kuduOptions).format("kudu").load
-    assertEquals(14, dfDefaultSchema.schema.fields.length)
+    assertEquals(16, dfDefaultSchema.schema.fields.length)
 
     val dfWithUserSchema =
       sqlContext.read.options(kuduOptions).schema(userSchema).format("kudu").load
