@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+
 package org.apache.kudu.test.cluster;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
@@ -26,6 +27,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.nio.file.Paths;
+import java.security.Security;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
@@ -35,6 +37,10 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.apache.yetus.audience.InterfaceStability;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import org.apache.kudu.Common;
 import org.apache.kudu.client.HostAndPort;
@@ -43,8 +49,8 @@ import org.apache.kudu.test.KuduTestHarness;
 import org.apache.kudu.test.TempDirUtils;
 import org.apache.kudu.tools.Tool.ControlShellRequestPB;
 import org.apache.kudu.tools.Tool.ControlShellResponsePB;
-import org.apache.kudu.tools.Tool.CreateClusterRequestPB.MiniKdcOptionsPB;
 import org.apache.kudu.tools.Tool.CreateClusterRequestPB;
+import org.apache.kudu.tools.Tool.CreateClusterRequestPB.MiniKdcOptionsPB;
 import org.apache.kudu.tools.Tool.DaemonIdentifierPB;
 import org.apache.kudu.tools.Tool.DaemonInfoPB;
 import org.apache.kudu.tools.Tool.GetKDCEnvVarsRequestPB;
@@ -57,10 +63,6 @@ import org.apache.kudu.tools.Tool.StartClusterRequestPB;
 import org.apache.kudu.tools.Tool.StartDaemonRequestPB;
 import org.apache.kudu.tools.Tool.StopDaemonRequestPB;
 import org.apache.kudu.util.SecurityUtil;
-import org.apache.yetus.audience.InterfaceAudience;
-import org.apache.yetus.audience.InterfaceStability;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
 /**
  * Utility class to start and manipulate Kudu clusters. Depends on precompiled
@@ -69,7 +71,7 @@ import org.slf4j.LoggerFactory;
  */
 @InterfaceAudience.Private
 @InterfaceStability.Unstable
-public class MiniKuduCluster implements AutoCloseable {
+public final class MiniKuduCluster implements AutoCloseable {
 
   private static final Logger LOG = LoggerFactory.getLogger(MiniKuduCluster.class);
 
@@ -139,6 +141,14 @@ public class MiniKuduCluster implements AutoCloseable {
     } else {
       this.clusterRoot = clusterRoot;
     }
+
+    // Default Java security settings are restrictive with regard to RSA key
+    // length. Since Kudu masters and tablet servers in MiniKuduCluster use
+    // smaller RSA keys to shorten runtime of tests, it's necessary to override
+    // those default security settings to allow for using relaxed cryptography,
+    // particularly smaller RSA keys.
+    Security.setProperty("jdk.certpath.disabledAlgorithms", "MD2, RC4, MD5");
+    Security.setProperty("jdk.tls.disabledAlgorithms", "SSLv3, RC4, MD5");
   }
 
   /**
@@ -176,7 +186,7 @@ public class MiniKuduCluster implements AutoCloseable {
    * Starts this Kudu cluster.
    * @throws IOException if something went wrong in transit
    */
-  private void start() throws IOException {
+  private synchronized void start() throws IOException {
     Preconditions.checkArgument(numMasters > 0, "Need at least one master");
 
     // Start the control shell and the communication channel to it.
@@ -287,14 +297,14 @@ public class MiniKuduCluster implements AutoCloseable {
    * @return the list of master servers
    */
   public List<HostAndPort> getMasterServers() {
-    return new ArrayList(masterServers.keySet());
+    return new ArrayList<>(masterServers.keySet());
   }
 
   /**
    * @return the list of tablet servers
    */
   public List<HostAndPort> getTabletServers() {
-    return new ArrayList(tabletServers.keySet());
+    return new ArrayList<>(tabletServers.keySet());
   }
 
   /**
@@ -481,8 +491,6 @@ public class MiniKuduCluster implements AutoCloseable {
         .build());
   }
 
-
-  /** {@override} */
   @Override
   public void close() {
     shutdown();
@@ -491,7 +499,7 @@ public class MiniKuduCluster implements AutoCloseable {
   /**
    * Shuts down a Kudu cluster.
    */
-  public void shutdown() {
+  public synchronized void shutdown() {
     // Closing stdin should cause the control shell process to terminate.
     if (miniClusterStdin != null) {
       try {
