@@ -47,7 +47,6 @@
 #include "kudu/fs/error_manager.h"
 #include "kudu/fs/io_context.h"
 #include "kudu/gutil/basictypes.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/stringprintf.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/util/array_view.h"
@@ -536,12 +535,6 @@ Status CFileReader::ReadBlock(const IOContext* io_context, const BlockPointer &p
 
     // Set the result block to our decompressed data.
     block = Slice(buf, uncompressed_size);
-  } else {
-    // Some of the File implementations from LevelDB attempt to be tricky
-    // and just return a Slice into an mmapped region (or in-memory region).
-    // But, this is hard to program against in terms of cache management, etc,
-    // so we memcpy into our scratch buffer if necessary.
-    block.relocate(scratch.get());
   }
 
   // It's possible that one of the TryAllocateFromCache() calls above
@@ -926,18 +919,18 @@ string CFileIterator::PreparedBlock::ToString() const {
 }
 
 // Decode the null header in the beginning of the data block
-Status DecodeNullInfo(Slice *data_block, uint32_t *num_rows_in_block, Slice *null_bitmap) {
+Status DecodeNullInfo(Slice *data_block, uint32_t *num_rows_in_block, Slice *non_null_bitmap) {
   if (!GetVarint32(data_block, num_rows_in_block)) {
     return Status::Corruption("bad null header, num elements in block");
   }
 
-  uint32_t null_bitmap_size;
-  if (!GetVarint32(data_block, &null_bitmap_size)) {
+  uint32_t non_null_bitmap_size;
+  if (!GetVarint32(data_block, &non_null_bitmap_size)) {
     return Status::Corruption("bad null header, bitmap size");
   }
 
-  *null_bitmap = Slice(data_block->data(), null_bitmap_size);
-  data_block->remove_prefix(null_bitmap_size);
+  *non_null_bitmap = Slice(data_block->data(), non_null_bitmap_size);
+  data_block->remove_prefix(non_null_bitmap_size);
   return Status::OK();
 }
 
