@@ -18,6 +18,8 @@
 #include "kudu/client/meta_cache.h"
 
 #include <cstdint>
+#include <functional>
+#include <memory>
 #include <mutex>
 #include <ostream>
 #include <set>
@@ -25,7 +27,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/bind.hpp> // IWYU pragma: keep
 #include <glog/logging.h>
 #include <google/protobuf/repeated_field.h> // IWYU pragma: keep
 
@@ -37,10 +38,6 @@
 #include "kudu/common/wire_protocol.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/gutil/basictypes.h"
-#include "kudu/gutil/bind.h"
-#include "kudu/gutil/bind_helpers.h"
-#include "kudu/gutil/callback.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/stl_util.h"
@@ -57,28 +54,25 @@
 #include "kudu/util/net/sockaddr.h"
 #include "kudu/util/pb_util.h"
 
-using std::map;
+using kudu::consensus::RaftPeerPB;
+using kudu::master::ANY_REPLICA;
+using kudu::master::GetTableLocationsRequestPB;
+using kudu::master::GetTableLocationsResponsePB;
+using kudu::master::MasterServiceProxy;
+using kudu::master::TabletLocationsPB;
+using kudu::master::TSInfoPB;
+using kudu::rpc::BackoffType;
+using kudu::rpc::CredentialsPolicy;
+using kudu::tserver::TabletServerServiceProxy;
 using std::set;
 using std::shared_ptr;
 using std::string;
+using std::unique_ptr;
 using std::vector;
 using strings::Substitute;
 
 namespace kudu {
-
-using consensus::RaftPeerPB;
-using master::ANY_REPLICA;
-using master::GetTableLocationsRequestPB;
-using master::GetTableLocationsResponsePB;
-using master::MasterServiceProxy;
-using master::TabletLocationsPB;
-using master::TSInfoPB;
-using rpc::BackoffType;
-using rpc::CredentialsPolicy;
-using tserver::TabletServerServiceProxy;
-
 namespace client {
-
 namespace internal {
 
 RemoteTabletServer::RemoteTabletServer(const master::TSInfoPB& pb)
@@ -91,7 +85,7 @@ void RemoteTabletServer::DnsResolutionFinished(const HostPort& hp,
                                                KuduClient* client,
                                                const StatusCallback& user_callback,
                                                const Status &result_status) {
-  gscoped_ptr<vector<Sockaddr> > scoped_addrs(addrs);
+  unique_ptr<vector<Sockaddr>> scoped_addrs(addrs);
 
   Status s = result_status;
 
@@ -101,7 +95,7 @@ void RemoteTabletServer::DnsResolutionFinished(const HostPort& hp,
 
   if (!s.ok()) {
     s = s.CloneAndPrepend("Failed to resolve address for TS " + uuid_);
-    user_callback.Run(s);
+    user_callback(s);
     return;
   }
 
@@ -113,7 +107,7 @@ void RemoteTabletServer::DnsResolutionFinished(const HostPort& hp,
     proxy_.reset(new TabletServerServiceProxy(client->data_->messenger_, (*addrs)[0], hp.host()));
     proxy_->set_user_credentials(client->data_->user_credentials_);
   }
-  user_callback.Run(s);
+  user_callback(s);
 }
 
 void RemoteTabletServer::InitProxy(KuduClient* client, const StatusCallback& cb) {
@@ -124,7 +118,7 @@ void RemoteTabletServer::InitProxy(KuduClient* client, const StatusCallback& cb)
     if (proxy_) {
       // Already have a proxy created.
       l.unlock();
-      cb.Run(Status::OK());
+      cb(Status::OK());
       return;
     }
 
@@ -136,8 +130,9 @@ void RemoteTabletServer::InitProxy(KuduClient* client, const StatusCallback& cb)
 
   auto addrs = new vector<Sockaddr>();
   client->data_->dns_resolver_->ResolveAddressesAsync(
-      hp, addrs, Bind(&RemoteTabletServer::DnsResolutionFinished,
-                      Unretained(this), hp, addrs, client, cb));
+      hp, addrs, [=](const Status& s) {
+        this->DnsResolutionFinished(hp, addrs, client, cb, s);
+      });
 }
 
 void RemoteTabletServer::Update(const master::TSInfoPB& pb) {
@@ -459,17 +454,17 @@ void MetaCacheServerPicker::PickLeader(const ServerPickedCallback& callback,
         deadline,
         MetaCache::LookupType::kPoint,
         nullptr,
-        Bind(&MetaCacheServerPicker::LookUpTabletCb, Unretained(this), callback, deadline));
+        [this, callback, deadline](const Status& s) {
+          this->LookUpTabletCb(callback, deadline, s);
+        });
     return;
   }
 
   // If we have a current TS initialize the proxy.
   // Make sure we have a working proxy before sending out the RPC.
-  leader->InitProxy(client_,
-                    Bind(&MetaCacheServerPicker::InitProxyCb,
-                         Unretained(this),
-                         callback,
-                         leader));
+  leader->InitProxy(client_, [this, callback, leader](const Status& s) {
+    this->InitProxyCb(callback, leader, s);
+  });
 }
 
 void MetaCacheServerPicker::MarkServerFailed(RemoteTabletServer* replica, const Status& status) {
@@ -499,7 +494,7 @@ void MetaCacheServerPicker::LookUpTabletCb(const ServerPickedCallback& callback,
 
   // If we couldn't lookup the tablet call the user callback immediately.
   if (!status.ok()) {
-    callback.Run(status, nullptr);
+    callback(status, nullptr);
     return;
   }
 
@@ -513,7 +508,7 @@ void MetaCacheServerPicker::LookUpTabletCb(const ServerPickedCallback& callback,
 void MetaCacheServerPicker::InitProxyCb(const ServerPickedCallback& callback,
                                         RemoteTabletServer* replica,
                                         const Status& status) {
-  callback.Run(status, replica);
+  callback(status, replica);
 }
 
 
@@ -674,7 +669,7 @@ void LookupRpc::SendRpc() {
   Status fastpath_status = meta_cache_->DoFastPathLookup(
       table_, &partition_key_, lookup_type_, remote_tablet_);
   if (!fastpath_status.IsIncomplete()) {
-    user_cb_.Run(fastpath_status);
+    user_cb_(fastpath_status);
     delete this;
     return;
   }
@@ -722,14 +717,14 @@ void LookupRpc::ResetMasterLeaderAndRetry(CredentialsPolicy creds_policy) {
   table_->client()->data_->ConnectToClusterAsync(
       table_->client(),
       retrier().deadline(),
-      Bind(&LookupRpc::NewLeaderMasterDeterminedCb, Unretained(this), creds_policy),
+      [=](const Status& s) { this->NewLeaderMasterDeterminedCb(creds_policy, s); },
       creds_policy);
 }
 
 void LookupRpc::SendRpcCb(const Status& status) {
   // If we exit and haven't scheduled a retry, this object should delete
   // itself.
-  gscoped_ptr<LookupRpc> delete_me(this);
+  unique_ptr<LookupRpc> delete_me(this);
 
   // Check for generic errors.
   Status new_status = status;
@@ -766,7 +761,7 @@ void LookupRpc::SendRpcCb(const Status& status) {
     new_status = new_status.CloneAndPrepend(Substitute("$0 failed", ToString()));
     KLOG_EVERY_N_SECS(WARNING, 1) << new_status.ToString();
   }
-  user_cb_.Run(new_status);
+  user_cb_(new_status);
 }
 
 Status MetaCache::ProcessLookupResponse(const LookupRpc& rpc,
@@ -1018,7 +1013,7 @@ void MetaCache::LookupTabletByKey(const KuduTable* table,
   Status fastpath_status = DoFastPathLookup(
       table, &partition_key, lookup_type, remote_tablet);
   if (!fastpath_status.IsIncomplete()) {
-    callback.Run(fastpath_status);
+    callback(fastpath_status);
     return;
   }
 
