@@ -19,6 +19,7 @@
 
 #include <stdint.h>
 
+// IWYU pragma: no_include <memory>
 #include <string>
 
 #include "kudu/client/shared_ptr.h" // IWYU pragma: keep
@@ -69,7 +70,8 @@ class KUDU_EXPORT KuduWriteOperation {
     INSERT = 1,
     UPDATE = 2,
     DELETE = 3,
-    UPSERT = 4
+    UPSERT = 4,
+    INSERT_IGNORE = 5
   };
   virtual ~KuduWriteOperation();
 
@@ -161,6 +163,34 @@ class KUDU_EXPORT KuduInsert : public KuduWriteOperation {
   explicit KuduInsert(const sp::shared_ptr<KuduTable>& table);
 };
 
+
+/// @brief A single row insert ignore to be sent to the cluster, duplicate row errors are ignored.
+///
+/// @pre An insert ignore requires all key columns to be set, as well as all
+///   columns which do not have default values.
+class KUDU_EXPORT KuduInsertIgnore : public KuduWriteOperation {
+ public:
+  virtual ~KuduInsertIgnore();
+
+  /// @copydoc KuduWriteOperation::ToString()
+  virtual std::string ToString() const OVERRIDE { return "INSERT IGNORE " + row_.ToString(); }
+
+ protected:
+  /// @cond PROTECTED_MEMBERS_DOCUMENTED
+
+  /// @copydoc KuduWriteOperation::type()
+  virtual Type type() const OVERRIDE {
+    return INSERT_IGNORE;
+  }
+
+  /// @endcond
+
+ private:
+  friend class KuduTable;
+  explicit KuduInsertIgnore(const sp::shared_ptr<KuduTable>& table);
+};
+
+
 /// @brief A single row upsert to be sent to the cluster.
 ///
 /// See KuduInsert for more details.
