@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+#pragma once
 
 #include <functional>
 #include <map>
@@ -24,7 +25,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/bind.hpp>
 #include <gmock/gmock.h>
 
 #include "kudu/clock/clock.h"
@@ -62,11 +62,11 @@
 namespace kudu {
 namespace consensus {
 
-inline gscoped_ptr<ReplicateMsg> CreateDummyReplicate(int64_t term,
-                                                      int64_t index,
-                                                      const Timestamp& timestamp,
-                                                      int64_t payload_size) {
-    gscoped_ptr<ReplicateMsg> msg(new ReplicateMsg);
+inline std::unique_ptr<ReplicateMsg> CreateDummyReplicate(int64_t term,
+                                                          int64_t index,
+                                                          const Timestamp& timestamp,
+                                                          int64_t payload_size) {
+    std::unique_ptr<ReplicateMsg> msg(new ReplicateMsg);
     OpId* id = msg->mutable_id();
     id->set_term(term);
     id->set_index(index);
@@ -74,7 +74,7 @@ inline gscoped_ptr<ReplicateMsg> CreateDummyReplicate(int64_t term,
     msg->set_op_type(NO_OP);
     msg->mutable_noop_request()->mutable_payload_for_tests()->resize(payload_size);
     msg->set_timestamp(timestamp.ToUint64());
-    return std::move(msg);
+    return msg;
 }
 
 // Returns RaftPeerPB with given UUID and obviously-fake hostname / port combo.
@@ -95,7 +95,7 @@ inline RaftPeerPB FakeRaftPeerPB(const std::string& uuid) {
 // TestOperationStatus::AckPeer().
 inline void AppendReplicateMessagesToQueue(
     PeerMessageQueue* queue,
-    const scoped_refptr<clock::Clock>& clock,
+    clock::Clock* clock,
     int64_t first,
     int64_t count,
     int64_t payload_size = 0) {
@@ -150,7 +150,7 @@ class TestPeerProxy : public PeerProxy {
  protected:
   // Register the RPC callback in order to call later.
   // We currently only support one request of each method being in flight at a time.
-  virtual void RegisterCallback(Method method, const rpc::ResponseCallback& callback) {
+  void RegisterCallback(Method method, const rpc::ResponseCallback& callback) {
     std::lock_guard<simple_spinlock> lock(lock_);
     InsertOrDie(&callbacks_, method, callback);
   }
@@ -167,10 +167,10 @@ class TestPeerProxy : public PeerProxy {
     }
     // If the peer has been closed while a response was in-flight, this can
     // return a bad Status, but that's fine.
-    ignore_result(pool_->SubmitFunc(callback));
+    ignore_result(pool_->Submit(std::move(callback)));
   }
 
-  virtual void RegisterCallbackAndRespond(Method method, const rpc::ResponseCallback& callback) {
+  void RegisterCallbackAndRespond(Method method, const rpc::ResponseCallback& callback) {
     RegisterCallback(method, callback);
     Respond(method);
   }
@@ -200,7 +200,7 @@ class DelayablePeerProxy : public TestPeerProxy {
     latch_.Reset(1); // Reset for the next time.
   }
 
-  virtual void RespondUnlessDelayed(Method method) {
+  void RespondUnlessDelayed(Method method) {
     {
       std::lock_guard<simple_spinlock> l(lock_);
       if (delay_response_) {
@@ -212,35 +212,36 @@ class DelayablePeerProxy : public TestPeerProxy {
     TestPeerProxy::Respond(method);
   }
 
-  virtual void Respond(Method method) OVERRIDE {
+  void Respond(Method method) override {
     latch_.Wait();   // Wait until strictly after peer would have responded.
     return TestPeerProxy::Respond(method);
   }
 
-  virtual void UpdateAsync(const ConsensusRequestPB* request,
-                           ConsensusResponsePB* response,
-                           rpc::RpcController* controller,
-                           const rpc::ResponseCallback& callback) OVERRIDE {
+  void UpdateAsync(const ConsensusRequestPB& request,
+                   ConsensusResponsePB* response,
+                   rpc::RpcController* controller,
+                   const rpc::ResponseCallback& callback) override {
     RegisterCallback(kUpdate, callback);
-    return proxy_->UpdateAsync(request, response, controller,
-                               boost::bind(&DelayablePeerProxy::RespondUnlessDelayed,
-                                           this, kUpdate));
+    return proxy_->UpdateAsync(
+        request, response, controller,
+        [this]() { this->RespondUnlessDelayed(kUpdate); });
   }
 
-  virtual Status StartElection(const RunLeaderElectionRequestPB* /*request*/,
-                               RunLeaderElectionResponsePB* /*response*/,
-                               rpc::RpcController* /*controller*/) override {
-    return Status::OK();
+  void StartElectionAsync(const RunLeaderElectionRequestPB& /*request*/,
+                          RunLeaderElectionResponsePB* /*response*/,
+                          rpc::RpcController* /*controller*/,
+                          const rpc::ResponseCallback& callback) override {
+    callback();
   }
 
-  virtual void RequestConsensusVoteAsync(const VoteRequestPB* request,
-                                         VoteResponsePB* response,
-                                         rpc::RpcController* controller,
-                                         const rpc::ResponseCallback& callback) OVERRIDE {
+  void RequestConsensusVoteAsync(const VoteRequestPB& request,
+                                 VoteResponsePB* response,
+                                 rpc::RpcController* controller,
+                                 const rpc::ResponseCallback& callback) override {
     RegisterCallback(kRequestVote, callback);
-    return proxy_->RequestConsensusVoteAsync(request, response, controller,
-                                             boost::bind(&DelayablePeerProxy::RespondUnlessDelayed,
-                                                         this, kRequestVote));
+    return proxy_->RequestConsensusVoteAsync(
+        request, response, controller,
+        [this]() { this->RespondUnlessDelayed(kRequestVote); });
   }
 
   ProxyType* proxy() const {
@@ -248,7 +249,7 @@ class DelayablePeerProxy : public TestPeerProxy {
   }
 
  protected:
-  gscoped_ptr<ProxyType> const proxy_;
+  std::unique_ptr<ProxyType> const proxy_;
   bool delay_response_; // Protected by lock_.
   CountDownLatch latch_;
 };
@@ -262,25 +263,21 @@ class MockedPeerProxy : public TestPeerProxy {
     update_count_(0) {
   }
 
-  virtual void set_update_response(const ConsensusResponsePB& update_response) {
+  void set_update_response(const ConsensusResponsePB& update_response) {
     CHECK(update_response.IsInitialized()) << pb_util::SecureShortDebugString(update_response);
-    {
-      std::lock_guard<simple_spinlock> l(lock_);
-      update_response_ = update_response;
-    }
+    std::lock_guard<simple_spinlock> l(lock_);
+    update_response_ = update_response;
   }
 
-  virtual void set_vote_response(const VoteResponsePB& vote_response) {
-    {
-      std::lock_guard<simple_spinlock> l(lock_);
-      vote_response_ = vote_response;
-    }
+  void set_vote_response(const VoteResponsePB& vote_response) {
+    std::lock_guard<simple_spinlock> l(lock_);
+    vote_response_ = vote_response;
   }
 
-  virtual void UpdateAsync(const ConsensusRequestPB* request,
-                           ConsensusResponsePB* response,
-                           rpc::RpcController* controller,
-                           const rpc::ResponseCallback& callback) OVERRIDE {
+  void UpdateAsync(const ConsensusRequestPB& /*request*/,
+                   ConsensusResponsePB* response,
+                   rpc::RpcController* /*controller*/,
+                   const rpc::ResponseCallback& callback) override {
     {
       std::lock_guard<simple_spinlock> l(lock_);
       update_count_++;
@@ -289,18 +286,19 @@ class MockedPeerProxy : public TestPeerProxy {
     return RegisterCallbackAndRespond(kUpdate, callback);
   }
 
-  virtual void RequestConsensusVoteAsync(const VoteRequestPB* request,
-                                         VoteResponsePB* response,
-                                         rpc::RpcController* controller,
-                                         const rpc::ResponseCallback& callback) OVERRIDE {
+  void RequestConsensusVoteAsync(const VoteRequestPB& /*request*/,
+                                 VoteResponsePB* response,
+                                 rpc::RpcController* /*controller*/,
+                                 const rpc::ResponseCallback& callback) override {
     *response = vote_response_;
     return RegisterCallbackAndRespond(kRequestVote, callback);
   }
 
-  Status StartElection(const RunLeaderElectionRequestPB* /*request*/,
-                     RunLeaderElectionResponsePB* /*response*/,
-                     rpc::RpcController* /*controller*/) override {
-    return Status::OK();
+  void StartElectionAsync(const RunLeaderElectionRequestPB& /*request*/,
+                          RunLeaderElectionResponsePB* /*response*/,
+                          rpc::RpcController* /*controller*/,
+                          const rpc::ResponseCallback& callback) override {
+    callback();
   }
 
   // Return the number of times that UpdateAsync() has been called.
@@ -326,24 +324,24 @@ class NoOpTestPeerProxy : public TestPeerProxy {
     last_received_.CopyFrom(MinimumOpId());
   }
 
-  virtual void UpdateAsync(const ConsensusRequestPB* request,
-                           ConsensusResponsePB* response,
-                           rpc::RpcController* controller,
-                           const rpc::ResponseCallback& callback) OVERRIDE {
+  void UpdateAsync(const ConsensusRequestPB& request,
+                   ConsensusResponsePB* response,
+                   rpc::RpcController* /*controller*/,
+                   const rpc::ResponseCallback& callback) override {
 
     response->Clear();
     {
       std::lock_guard<simple_spinlock> lock(lock_);
-      if (OpIdLessThan(last_received_, request->preceding_id())) {
+      if (OpIdLessThan(last_received_, request.preceding_id())) {
         ConsensusErrorPB* error = response->mutable_status()->mutable_error();
         error->set_code(ConsensusErrorPB::PRECEDING_ENTRY_DIDNT_MATCH);
         StatusToPB(Status::IllegalState(""), error->mutable_status());
-      } else if (request->ops_size() > 0) {
-        last_received_.CopyFrom(request->ops(request->ops_size() - 1).id());
+      } else if (request.ops_size() > 0) {
+        last_received_.CopyFrom(request.ops(request.ops_size() - 1).id());
       }
 
       response->set_responder_uuid(peer_pb_.permanent_uuid());
-      response->set_responder_term(request->caller_term());
+      response->set_responder_term(request.caller_term());
       response->mutable_status()->mutable_last_received()->CopyFrom(last_received_);
       response->mutable_status()->mutable_last_received_current_leader()->CopyFrom(last_received_);
       // We set the last committed index to be the same index as the last received. While
@@ -354,20 +352,21 @@ class NoOpTestPeerProxy : public TestPeerProxy {
     return RegisterCallbackAndRespond(kUpdate, callback);
   }
 
-  virtual Status StartElection(const RunLeaderElectionRequestPB* /*request*/,
-                             RunLeaderElectionResponsePB* /*response*/,
-                             rpc::RpcController* /*controller*/) override {
-    return Status::OK();
+  void StartElectionAsync(const RunLeaderElectionRequestPB& /*request*/,
+                          RunLeaderElectionResponsePB* /*response*/,
+                          rpc::RpcController* /*controller*/,
+                          const rpc::ResponseCallback& callback) override {
+    callback();
   }
 
-  virtual void RequestConsensusVoteAsync(const VoteRequestPB* request,
-                                         VoteResponsePB* response,
-                                         rpc::RpcController* /*controller*/,
-                                         const rpc::ResponseCallback& callback) OVERRIDE {
+  void RequestConsensusVoteAsync(const VoteRequestPB& request,
+                                 VoteResponsePB* response,
+                                 rpc::RpcController* /*controller*/,
+                                 const rpc::ResponseCallback& callback) override {
     {
       std::lock_guard<simple_spinlock> lock(lock_);
       response->set_responder_uuid(peer_pb_.permanent_uuid());
-      response->set_responder_term(request->candidate_term());
+      response->set_responder_term(request.candidate_term());
       response->set_vote_granted(true);
     }
     return RegisterCallbackAndRespond(kRequestVote, callback);
@@ -392,7 +391,7 @@ class NoOpTestPeerProxyFactory : public PeerProxyFactory {
   }
 
   Status NewProxy(const consensus::RaftPeerPB& peer_pb,
-                  gscoped_ptr<PeerProxy>* proxy) override {
+                  std::unique_ptr<PeerProxy>* proxy) override {
     proxy->reset(new NoOpTestPeerProxy(pool_.get(), peer_pb));
     return Status::OK();
   }
@@ -473,28 +472,27 @@ class LocalTestPeerProxy : public TestPeerProxy {
         peers_(peers),
         miss_comm_(false) {}
 
-  virtual void UpdateAsync(const ConsensusRequestPB* request,
-                           ConsensusResponsePB* response,
-                           rpc::RpcController* controller,
-                           const rpc::ResponseCallback& callback) OVERRIDE {
+  void UpdateAsync(const ConsensusRequestPB& request,
+                   ConsensusResponsePB* response,
+                   rpc::RpcController* /*controller*/,
+                   const rpc::ResponseCallback& callback) override {
     RegisterCallback(kUpdate, callback);
-    CHECK_OK(pool_->SubmitFunc(boost::bind(&LocalTestPeerProxy::SendUpdateRequest,
-                                           this, request, response)));
+    CHECK_OK(pool_->Submit([=]() { this->SendUpdateRequest(request, response); }));
   }
 
-  Status StartElection(const RunLeaderElectionRequestPB* /*request*/,
-                       RunLeaderElectionResponsePB* /*response*/,
-                       rpc::RpcController* /*controller*/) override {
-    return Status::OK();
+  void StartElectionAsync(const RunLeaderElectionRequestPB& /*request*/,
+                          RunLeaderElectionResponsePB* /*response*/,
+                          rpc::RpcController* /*controller*/,
+                          const rpc::ResponseCallback& callback) override {
+    callback();
   }
 
-  virtual void RequestConsensusVoteAsync(const VoteRequestPB* request,
-                                         VoteResponsePB* response,
-                                         rpc::RpcController* /*controller*/,
-                                         const rpc::ResponseCallback& callback) OVERRIDE {
+  void RequestConsensusVoteAsync(const VoteRequestPB& request,
+                                 VoteResponsePB* response,
+                                 rpc::RpcController* /*controller*/,
+                                 const rpc::ResponseCallback& callback) override {
     RegisterCallback(kRequestVote, callback);
-    CHECK_OK(pool_->SubmitFunc(boost::bind(&LocalTestPeerProxy::SendVoteRequest,
-                                           this, request, response)));
+    CHECK_OK(pool_->Submit([=]() { this->SendVoteRequest(request, response); }));
   }
 
   template<class Response>
@@ -505,7 +503,7 @@ class LocalTestPeerProxy : public TestPeerProxy {
   }
 
   template<class Request, class Response>
-  void RespondOrMissResponse(Request* request,
+  void RespondOrMissResponse(const Request& request,
                              const Response& response_temp,
                              Response* final_response,
                              Method method) {
@@ -517,7 +515,7 @@ class LocalTestPeerProxy : public TestPeerProxy {
       miss_comm_ = false;
     }
     if (PREDICT_FALSE(miss_comm_copy)) {
-      VLOG(2) << this << ": injecting fault on " << pb_util::SecureShortDebugString(*request);
+      VLOG(2) << this << ": injecting fault on " << pb_util::SecureShortDebugString(request);
       SetResponseError(Status::IOError("Artificial error caused by communication "
           "failure injection."), final_response);
     } else {
@@ -526,12 +524,12 @@ class LocalTestPeerProxy : public TestPeerProxy {
     Respond(method);
   }
 
-  void SendUpdateRequest(const ConsensusRequestPB* request,
+  void SendUpdateRequest(const ConsensusRequestPB& request,
                          ConsensusResponsePB* response) {
     // Copy the request and the response for the other peer so that ownership
     // remains as close to the dist. impl. as possible.
     ConsensusRequestPB other_peer_req;
-    other_peer_req.CopyFrom(*request);
+    other_peer_req.CopyFrom(request);
 
     // Give the other peer a clean response object to write to.
     ConsensusResponsePB other_peer_resp;
@@ -558,13 +556,13 @@ class LocalTestPeerProxy : public TestPeerProxy {
 
 
 
-  void SendVoteRequest(const VoteRequestPB* request,
+  void SendVoteRequest(const VoteRequestPB& request,
                        VoteResponsePB* response) {
 
     // Copy the request and the response for the other peer so that ownership
     // remains as close to the dist. impl. as possible.
     VoteRequestPB other_peer_req;
-    other_peer_req.CopyFrom(*request);
+    other_peer_req.CopyFrom(request);
     VoteResponsePB other_peer_resp;
     other_peer_resp.CopyFrom(*response);
 
@@ -612,7 +610,7 @@ class LocalTestPeerProxyFactory : public PeerProxyFactory {
   }
 
   Status NewProxy(const consensus::RaftPeerPB& peer_pb,
-                  gscoped_ptr<PeerProxy>* proxy) override {
+                  std::unique_ptr<PeerProxy>* proxy) override {
     LocalTestPeerProxy* new_proxy = new LocalTestPeerProxy(peer_pb.permanent_uuid(),
                                                            pool_.get(),
                                                            peers_);
@@ -660,7 +658,7 @@ class TestDriver {
       return;
     }
     CHECK_OK(status);
-    CHECK_OK(pool_->SubmitFunc(boost::bind(&TestDriver::Apply, this)));
+    CHECK_OK(pool_->Submit([this]() { this->Apply(); }));
   }
 
   // Called in all modes to delete the transaction and, transitively, the consensus
@@ -675,11 +673,11 @@ class TestDriver {
   // The commit message has the exact same type of the replicate message, but
   // no content.
   void Apply() {
-    gscoped_ptr<CommitMsg> msg(new CommitMsg);
+    std::unique_ptr<CommitMsg> msg(new CommitMsg);
     msg->set_op_type(round_->replicate_msg()->op_type());
     msg->mutable_commited_op_id()->CopyFrom(round_->id());
-    CHECK_OK(log_->AsyncAppendCommit(std::move(msg),
-                                     Bind(&TestDriver::CommitCallback, Unretained(this))));
+    CHECK_OK(log_->AsyncAppendCommit(
+        std::move(msg), [this](const Status& s) { this->CommitCallback(s); }));
   }
 
   void CommitCallback(const Status& s) {
@@ -706,11 +704,10 @@ class TestTransactionFactory : public ConsensusRoundHandler {
   }
 
   Status StartFollowerTransaction(const scoped_refptr<ConsensusRound>& round) override {
-    auto txn = new TestDriver(pool_.get(), log_, round);
-    txn->round_->SetConsensusReplicatedCallback(std::bind(
-        &TestDriver::ReplicationFinished,
-        txn,
-        std::placeholders::_1));
+    // 'txn' is deleted when it completes.
+    auto* txn = new TestDriver(pool_.get(), log_, round);
+    txn->round_->SetConsensusReplicatedCallback(
+        [txn](const Status& s) { txn->ReplicationFinished(s); });
     return Status::OK();
   }
 
@@ -741,4 +738,3 @@ class TestTransactionFactory : public ConsensusRoundHandler {
 
 }  // namespace consensus
 }  // namespace kudu
-
