@@ -19,15 +19,14 @@
 
 #include <algorithm>
 #include <cstdlib>
+#include <functional>
 #include <memory>
 #include <mutex>
 #include <ostream>
 #include <string>
-#include <type_traits>
 #include <vector>
 
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 
 #include "kudu/common/common.pb.h"
@@ -38,7 +37,6 @@
 #include "kudu/consensus/consensus_queue.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/consensus/opid_util.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
@@ -93,6 +91,7 @@ using kudu::rpc::RpcController;
 using kudu::tserver::TabletServerErrorPB;
 using std::shared_ptr;
 using std::string;
+using std::unique_ptr;
 using std::vector;
 using std::weak_ptr;
 using strings::Substitute;
@@ -109,7 +108,7 @@ Status Peer::NewRemotePeer(RaftPeerPB peer_pb,
                            string leader_uuid,
                            PeerMessageQueue* queue,
                            ThreadPoolToken* raft_pool_token,
-                           gscoped_ptr<PeerProxy> proxy,
+                           unique_ptr<PeerProxy> proxy,
                            shared_ptr<Messenger> messenger,
                            shared_ptr<Peer>* peer) {
 
@@ -130,7 +129,7 @@ Peer::Peer(RaftPeerPB peer_pb,
            string leader_uuid,
            PeerMessageQueue* queue,
            ThreadPoolToken* raft_pool_token,
-           gscoped_ptr<PeerProxy> proxy,
+           unique_ptr<PeerProxy> proxy,
            shared_ptr<Messenger> messenger)
     : tablet_id_(std::move(tablet_id)),
       leader_uuid_(std::move(leader_uuid)),
@@ -179,7 +178,7 @@ Status Peer::SignalRequest(bool even_if_queue_empty) {
   // Capture a weak_ptr reference into the submitted functor so that we can
   // safely handle the functor outliving its peer.
   weak_ptr<Peer> w_this = shared_from_this();
-  RETURN_NOT_OK(raft_pool_token_->SubmitFunc([even_if_queue_empty, w_this]() {
+  RETURN_NOT_OK(raft_pool_token_->Submit([even_if_queue_empty, w_this]() {
     if (auto p = w_this.lock()) {
       p->SendNextRequest(even_if_queue_empty);
     }
@@ -243,7 +242,7 @@ void Peer::SendNextRequest(bool even_if_queue_empty) {
       // Capture a shared_ptr reference into the RPC callback so that we're guaranteed
       // that this object outlives the RPC.
       shared_ptr<Peer> s_this = shared_from_this();
-      proxy_->StartTabletCopyAsync(&tc_request_, &tc_response_, &controller_,
+      proxy_->StartTabletCopyAsync(tc_request_, &tc_response_, &controller_,
                                    [s_this]() {
                                      s_this->ProcessTabletCopyResponse();
                                    });
@@ -282,24 +281,38 @@ void Peer::SendNextRequest(bool even_if_queue_empty) {
   // Capture a shared_ptr reference into the RPC callback so that we're guaranteed
   // that this object outlives the RPC.
   shared_ptr<Peer> s_this = shared_from_this();
-  proxy_->UpdateAsync(&request_, &response_, &controller_,
+  proxy_->UpdateAsync(request_, &response_, &controller_,
                       [s_this]() {
                         s_this->ProcessResponse();
                       });
 }
 
-Status Peer::StartElection() {
+void Peer::StartElection() {
+  // The async proxy contract is such that the response and RPC controller must
+  // stay in scope until the callback is invoked. Unlike other Peer methods, we
+  // can't guarantee that there's only one outstanding StartElection call at a
+  // time, so we can't store the response and controller as a class member.
+  // Instead, we have to pass them into the callback and free them there.
   RunLeaderElectionRequestPB req;
-  RunLeaderElectionResponsePB resp;
-  RpcController controller;
-  req.set_dest_uuid(peer_pb().permanent_uuid());
+  unique_ptr<RunLeaderElectionResponsePB> resp_uniq(new RunLeaderElectionResponsePB());
+  unique_ptr<RpcController> controller_uniq(new RpcController());
+  string peer_uuid = peer_pb().permanent_uuid();
+  req.set_dest_uuid(peer_uuid);
   req.set_tablet_id(tablet_id_);
-  RETURN_NOT_OK(proxy_->StartElection(&req, &resp, &controller));
-  RETURN_NOT_OK(controller.status());
-  if (resp.has_error()) {
-    return StatusFromPB(response_.error().status());
-  }
-  return Status::OK();
+
+  // TODO(adar): lack of C++14 move capture makes for ugly code.
+  RunLeaderElectionResponsePB* resp = resp_uniq.release();
+  RpcController* controller = controller_uniq.release();
+  proxy_->StartElectionAsync(req, resp, controller, [resp, controller, peer_uuid]() {
+      unique_ptr<RunLeaderElectionResponsePB> r(resp);
+      unique_ptr<RpcController> c(controller);
+      string error_msg = Substitute("unable to start election on peer $0", peer_uuid);
+      if (!c->status().ok()) {
+        WARN_NOT_OK(c->status(), error_msg);
+      } else if (r->has_error()) {
+        WARN_NOT_OK(StatusFromPB(r->error().status()), error_msg);
+      }
+    });
 }
 
 void Peer::ProcessResponse() {
@@ -365,7 +378,7 @@ void Peer::ProcessResponse() {
   // Capture a weak_ptr reference into the submitted functor so that we can
   // safely handle the functor outliving its peer.
   weak_ptr<Peer> w_this = shared_from_this();
-  Status s = raft_pool_token_->SubmitFunc([w_this]() {
+  Status s = raft_pool_token_->Submit([w_this]() {
     if (auto p = w_this.lock()) {
       p->DoProcessResponse();
 
@@ -495,44 +508,45 @@ Peer::~Peer() {
   request_.mutable_ops()->ExtractSubrange(0, request_.ops_size(), nullptr);
 }
 
-RpcPeerProxy::RpcPeerProxy(gscoped_ptr<HostPort> hostport,
-                           gscoped_ptr<ConsensusServiceProxy> consensus_proxy)
-    : hostport_(std::move(DCHECK_NOTNULL(hostport))),
+RpcPeerProxy::RpcPeerProxy(HostPort hostport,
+                           unique_ptr<ConsensusServiceProxy> consensus_proxy)
+    : hostport_(std::move(hostport)),
       consensus_proxy_(std::move(DCHECK_NOTNULL(consensus_proxy))) {
 }
 
-void RpcPeerProxy::UpdateAsync(const ConsensusRequestPB* request,
+void RpcPeerProxy::UpdateAsync(const ConsensusRequestPB& request,
                                ConsensusResponsePB* response,
                                rpc::RpcController* controller,
                                const rpc::ResponseCallback& callback) {
   controller->set_timeout(MonoDelta::FromMilliseconds(FLAGS_consensus_rpc_timeout_ms));
-  consensus_proxy_->UpdateConsensusAsync(*request, response, controller, callback);
+  consensus_proxy_->UpdateConsensusAsync(request, response, controller, callback);
 }
 
-Status RpcPeerProxy::StartElection(const RunLeaderElectionRequestPB* request,
-                                 RunLeaderElectionResponsePB* response,
-                                 rpc::RpcController* controller) {
+void RpcPeerProxy::StartElectionAsync(const RunLeaderElectionRequestPB& request,
+                                      RunLeaderElectionResponsePB* response,
+                                      rpc::RpcController* controller,
+                                      const rpc::ResponseCallback& callback) {
   controller->set_timeout(MonoDelta::FromMilliseconds(FLAGS_consensus_rpc_timeout_ms));
-  return consensus_proxy_->RunLeaderElection(*request, response, controller);
+  consensus_proxy_->RunLeaderElectionAsync(request, response, controller, callback);
 }
 
-void RpcPeerProxy::RequestConsensusVoteAsync(const VoteRequestPB* request,
+void RpcPeerProxy::RequestConsensusVoteAsync(const VoteRequestPB& request,
                                              VoteResponsePB* response,
                                              rpc::RpcController* controller,
                                              const rpc::ResponseCallback& callback) {
-  consensus_proxy_->RequestConsensusVoteAsync(*request, response, controller, callback);
+  consensus_proxy_->RequestConsensusVoteAsync(request, response, controller, callback);
 }
 
-void RpcPeerProxy::StartTabletCopyAsync(const StartTabletCopyRequestPB* request,
+void RpcPeerProxy::StartTabletCopyAsync(const StartTabletCopyRequestPB& request,
                                         StartTabletCopyResponsePB* response,
                                         rpc::RpcController* controller,
                                         const rpc::ResponseCallback& callback) {
   controller->set_timeout(MonoDelta::FromMilliseconds(FLAGS_consensus_rpc_timeout_ms));
-  consensus_proxy_->StartTabletCopyAsync(*request, response, controller, callback);
+  consensus_proxy_->StartTabletCopyAsync(request, response, controller, callback);
 }
 
 string RpcPeerProxy::PeerName() const {
-  return hostport_->ToString();
+  return hostport_.ToString();
 }
 
 namespace {
@@ -541,7 +555,7 @@ Status CreateConsensusServiceProxyForHost(
     const HostPort& hostport,
     const shared_ptr<Messenger>& messenger,
     DnsResolver* dns_resolver,
-    gscoped_ptr<ConsensusServiceProxy>* new_proxy) {
+    unique_ptr<ConsensusServiceProxy>* new_proxy) {
   vector<Sockaddr> addrs;
   RETURN_NOT_OK(dns_resolver->ResolveAddresses(hostport, &addrs));
   if (addrs.size() > 1) {
@@ -562,12 +576,12 @@ RpcPeerProxyFactory::RpcPeerProxyFactory(shared_ptr<Messenger> messenger,
 }
 
 Status RpcPeerProxyFactory::NewProxy(const RaftPeerPB& peer_pb,
-                                     gscoped_ptr<PeerProxy>* proxy) {
-  gscoped_ptr<HostPort> hostport(new HostPort);
-  RETURN_NOT_OK(HostPortFromPB(peer_pb.last_known_addr(), hostport.get()));
-  gscoped_ptr<ConsensusServiceProxy> new_proxy;
+                                     unique_ptr<PeerProxy>* proxy) {
+  HostPort hostport;
+  RETURN_NOT_OK(HostPortFromPB(peer_pb.last_known_addr(), &hostport));
+  unique_ptr<ConsensusServiceProxy> new_proxy;
   RETURN_NOT_OK(CreateConsensusServiceProxyForHost(
-      *hostport, messenger_, dns_resolver_, &new_proxy));
+      hostport, messenger_, dns_resolver_, &new_proxy));
   proxy->reset(new RpcPeerProxy(std::move(hostport), std::move(new_proxy)));
   return Status::OK();
 }
@@ -579,7 +593,7 @@ Status SetPermanentUuidForRemotePeer(
   DCHECK(!remote_peer->has_permanent_uuid());
   HostPort hostport;
   RETURN_NOT_OK(HostPortFromPB(remote_peer->last_known_addr(), &hostport));
-  gscoped_ptr<ConsensusServiceProxy> proxy;
+  unique_ptr<ConsensusServiceProxy> proxy;
   RETURN_NOT_OK(CreateConsensusServiceProxyForHost(
       hostport, messenger, resolver, &proxy));
   GetNodeInstanceRequestPB req;
