@@ -14,9 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
-
-#ifndef KUDU_CONSENSUS_CONSENSUS_PEERS_H_
-#define KUDU_CONSENSUS_CONSENSUS_PEERS_H_
+#pragma once
 
 #include <cstdint>
 #include <memory>
@@ -30,7 +28,6 @@
 #include "kudu/consensus/consensus.proxy.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/consensus/ref_counted_replicate.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/util/locks.h"
@@ -79,12 +76,14 @@ class Peer : public std::enable_shared_from_this<Peer> {
   // status-only requests.
   Status SignalRequest(bool even_if_queue_empty = false);
 
-  // Synchronously starts a leader election on this peer.
+  // Starts a leader election on this peer.
+  //
   // This method is ad hoc, using this instance's PeerProxy to send the
   // StartElection request.
+  //
   // The StartElection RPC does not count as the single outstanding request
   // that this class tracks.
-  Status StartElection();
+  void StartElection();
 
   const RaftPeerPB& peer_pb() const { return peer_pb_; }
 
@@ -112,7 +111,7 @@ class Peer : public std::enable_shared_from_this<Peer> {
                               std::string leader_uuid,
                               PeerMessageQueue* queue,
                               ThreadPoolToken* raft_pool_token,
-                              gscoped_ptr<PeerProxy> proxy,
+                              std::unique_ptr<PeerProxy> proxy,
                               std::shared_ptr<rpc::Messenger> messenger,
                               std::shared_ptr<Peer>* peer);
 
@@ -122,7 +121,7 @@ class Peer : public std::enable_shared_from_this<Peer> {
        std::string leader_uuid,
        PeerMessageQueue* queue,
        ThreadPoolToken* raft_pool_token,
-       gscoped_ptr<PeerProxy> proxy,
+       std::unique_ptr<PeerProxy> proxy,
        std::shared_ptr<rpc::Messenger> messenger);
 
   void SendNextRequest(bool even_if_queue_empty);
@@ -159,7 +158,7 @@ class Peer : public std::enable_shared_from_this<Peer> {
 
   RaftPeerPB peer_pb_;
 
-  gscoped_ptr<PeerProxy> proxy_;
+  std::unique_ptr<PeerProxy> proxy_;
 
   PeerMessageQueue* queue_;
   uint64_t failed_attempts_;
@@ -204,23 +203,25 @@ class PeerProxy {
   virtual ~PeerProxy() {}
 
   // Sends a request, asynchronously, to a remote peer.
-  virtual void UpdateAsync(const ConsensusRequestPB* request,
+  virtual void UpdateAsync(const ConsensusRequestPB& request,
                            ConsensusResponsePB* response,
                            rpc::RpcController* controller,
                            const rpc::ResponseCallback& callback) = 0;
 
-  // Sends a RequestConsensusVote to a remote peer.
-  virtual void RequestConsensusVoteAsync(const VoteRequestPB* request,
+  // Asks a peer to vote for a candidate.
+  virtual void RequestConsensusVoteAsync(const VoteRequestPB& request,
                                          VoteResponsePB* response,
                                          rpc::RpcController* controller,
                                          const rpc::ResponseCallback& callback) = 0;
 
-  virtual Status StartElection(const RunLeaderElectionRequestPB* request,
-                               RunLeaderElectionResponsePB* response,
-                               rpc::RpcController* controller) = 0;
+  // Instructs a peer to kick off an election to elect itself leader.
+  virtual void StartElectionAsync(const RunLeaderElectionRequestPB& request,
+                                  RunLeaderElectionResponsePB* response,
+                                  rpc::RpcController* controller,
+                                  const rpc::ResponseCallback& callback) = 0;
 
   // Instructs a peer to begin a tablet copy session.
-  virtual void StartTabletCopyAsync(const StartTabletCopyRequestPB* /*request*/,
+  virtual void StartTabletCopyAsync(const StartTabletCopyRequestPB& /*request*/,
                                     StartTabletCopyResponsePB* /*response*/,
                                     rpc::RpcController* /*controller*/,
                                     const rpc::ResponseCallback& /*callback*/) {
@@ -237,7 +238,7 @@ class PeerProxyFactory {
  public:
 
   virtual Status NewProxy(const RaftPeerPB& peer_pb,
-                          gscoped_ptr<PeerProxy>* proxy) = 0;
+                          std::unique_ptr<PeerProxy>* proxy) = 0;
 
   virtual ~PeerProxyFactory() {}
 
@@ -247,24 +248,25 @@ class PeerProxyFactory {
 // PeerProxy implementation that does RPC calls
 class RpcPeerProxy : public PeerProxy {
  public:
-  RpcPeerProxy(gscoped_ptr<HostPort> hostport,
-               gscoped_ptr<ConsensusServiceProxy> consensus_proxy);
+  RpcPeerProxy(HostPort hostport,
+               std::unique_ptr<ConsensusServiceProxy> consensus_proxy);
 
-  void UpdateAsync(const ConsensusRequestPB* request,
+  void UpdateAsync(const ConsensusRequestPB& request,
                    ConsensusResponsePB* response,
                    rpc::RpcController* controller,
                    const rpc::ResponseCallback& callback) override;
 
-  void RequestConsensusVoteAsync(const VoteRequestPB* request,
+  void RequestConsensusVoteAsync(const VoteRequestPB& request,
                                  VoteResponsePB* response,
                                  rpc::RpcController* controller,
                                  const rpc::ResponseCallback& callback) override;
 
-  Status StartElection(const RunLeaderElectionRequestPB* request,
-                       RunLeaderElectionResponsePB* response,
-                       rpc::RpcController* controller) override;
+  void StartElectionAsync(const RunLeaderElectionRequestPB& request,
+                          RunLeaderElectionResponsePB* response,
+                          rpc::RpcController* controller,
+                          const rpc::ResponseCallback& callback) override;
 
-  void StartTabletCopyAsync(const StartTabletCopyRequestPB* request,
+  void StartTabletCopyAsync(const StartTabletCopyRequestPB& request,
                             StartTabletCopyResponsePB* response,
                             rpc::RpcController* controller,
                             const rpc::ResponseCallback& callback) override;
@@ -272,8 +274,8 @@ class RpcPeerProxy : public PeerProxy {
   std::string PeerName() const override;
 
  private:
-  gscoped_ptr<HostPort> hostport_;
-  gscoped_ptr<ConsensusServiceProxy> consensus_proxy_;
+  const HostPort hostport_;
+  std::unique_ptr<ConsensusServiceProxy> consensus_proxy_;
 };
 
 // PeerProxyFactory implementation that generates RPCPeerProxies
@@ -284,7 +286,7 @@ class RpcPeerProxyFactory : public PeerProxyFactory {
   ~RpcPeerProxyFactory() = default;
 
   Status NewProxy(const RaftPeerPB& peer_pb,
-                  gscoped_ptr<PeerProxy>* proxy) override;
+                  std::unique_ptr<PeerProxy>* proxy) override;
 
   const std::shared_ptr<rpc::Messenger>& messenger() const override {
     return messenger_;
@@ -305,5 +307,3 @@ Status SetPermanentUuidForRemotePeer(
 
 }  // namespace consensus
 }  // namespace kudu
-
-#endif /* KUDU_CONSENSUS_CONSENSUS_PEERS_H_ */
