@@ -15,17 +15,21 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#include <cstddef>
+#include "kudu/consensus/consensus_queue.h"
+
+#include <atomic>
 #include <cstdint>
+#include <deque>
 #include <memory>
 #include <ostream>
 #include <string>
 #include <vector>
 
+#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
+#include <gtest/gtest_prod.h>
 
 #include "kudu/clock/clock.h"
 #include "kudu/clock/hybrid_clock.h"
@@ -36,7 +40,6 @@
 #include "kudu/common/wire_protocol.h"
 #include "kudu/consensus/consensus-test-util.h"
 #include "kudu/consensus/consensus.pb.h"
-#include "kudu/consensus/consensus_queue.h"
 #include "kudu/consensus/log-test-base.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log_anchor_registry.h"
@@ -47,7 +50,6 @@
 #include "kudu/consensus/ref_counted_replicate.h"
 #include "kudu/consensus/time_manager.h"
 #include "kudu/fs/fs_manager.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/util/async_util.h"
@@ -63,6 +65,9 @@ DECLARE_int32(consensus_max_batch_size_bytes);
 DECLARE_int32(follower_unavailable_considered_failed_sec);
 
 using kudu::consensus::HealthReportPB;
+using std::atomic;
+using std::deque;
+using std::string;
 using std::unique_ptr;
 using std::vector;
 
@@ -77,40 +82,44 @@ class ConsensusQueueTest : public KuduTest {
  public:
   ConsensusQueueTest()
       : schema_(GetSimpleTestSchema()),
-        metric_entity_(METRIC_ENTITY_tablet.Instantiate(&metric_registry_, "queue-test")),
-        registry_(new log::LogAnchorRegistry) {
+        metric_entity_server_(METRIC_ENTITY_server.Instantiate(
+            &metric_registry_, "consensus-queue-test::server")),
+        metric_entity_tablet_(METRIC_ENTITY_tablet.Instantiate(
+            &metric_registry_, "consensus-queue-test::tablet")),
+        registry_(new log::LogAnchorRegistry),
+        quiescing_(false) {
   }
 
   virtual void SetUp() OVERRIDE {
     KuduTest::SetUp();
-    fs_manager_.reset(new FsManager(env_, GetTestPath("fs_root")));
+    fs_manager_.reset(new FsManager(env_, FsManagerOpts(GetTestPath("fs_root"))));
     ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout());
     ASSERT_OK(fs_manager_->Open());
     CHECK_OK(log::Log::Open(log::LogOptions(),
                             fs_manager_.get(),
+                            /*file_cache*/nullptr,
                             kTestTablet,
                             schema_,
-                            0, // schema_version
-                            NULL,
+                            /*schema_version*/0,
+                            /*metric_entity*/nullptr,
                             &log_));
-    clock_.reset(new clock::HybridClock());
+    clock_.reset(new clock::HybridClock(metric_entity_server_));
     ASSERT_OK(clock_->Init());
 
     ASSERT_OK(ThreadPoolBuilder("raft").Build(&raft_pool_));
+    time_manager_.reset(new TimeManager(clock_.get(), Timestamp::kMin));
     CloseAndReopenQueue(MinimumOpId(), MinimumOpId());
   }
 
   void CloseAndReopenQueue(const OpId& replicated_opid, const OpId& committed_opid) {
-    scoped_refptr<clock::Clock> clock(new clock::HybridClock());
-    ASSERT_OK(clock->Init());
-    scoped_refptr<TimeManager> time_manager(new TimeManager(clock, Timestamp::kMin));
     queue_.reset(new PeerMessageQueue(
-        metric_entity_,
+        metric_entity_tablet_,
         log_.get(),
-        time_manager,
+        time_manager_.get(),
         FakeRaftPeerPB(kLeaderUuid),
         kTestTablet,
         raft_pool_->NewToken(ThreadPool::ExecutionMode::SERIAL),
+        &quiescing_,
         replicated_opid,
         committed_opid));
   }
@@ -226,23 +235,139 @@ class ConsensusQueueTest : public KuduTest {
 
  protected:
   const Schema schema_;
-  gscoped_ptr<FsManager> fs_manager_;
   MetricRegistry metric_registry_;
-  scoped_refptr<MetricEntity> metric_entity_;
+  scoped_refptr<MetricEntity> metric_entity_server_;
+  scoped_refptr<MetricEntity> metric_entity_tablet_;
+  unique_ptr<FsManager> fs_manager_;
   scoped_refptr<log::Log> log_;
   unique_ptr<ThreadPool> raft_pool_;
-  gscoped_ptr<PeerMessageQueue> queue_;
+  unique_ptr<TimeManager> time_manager_;
+  unique_ptr<PeerMessageQueue> queue_;
   scoped_refptr<log::LogAnchorRegistry> registry_;
-  scoped_refptr<clock::Clock> clock_;
+  unique_ptr<clock::Clock> clock_;
+  atomic<bool> quiescing_;
 };
 
+// Observer of a PeerMessageQueue that tracks the notifications sent to
+// observers.
+class SimpleObserver : public PeerMessageQueueObserver {
+ public:
+  SimpleObserver() = default;
+
+  void NotifyPeerToStartElection(const string& peer_uuid) override {
+    peers_to_start_election_.emplace_back(peer_uuid);
+  }
+
+  // Other notifications aren't implemented. Just no-op.
+  void NotifyCommitIndex(int64_t /*commit_index*/) override {}
+  void NotifyTermChange(int64_t /*term*/) override {}
+  void NotifyFailedFollower(const string& /*peer_uuid*/, int64_t /*term*/,
+                            const string& /*reason*/) override {}
+  void NotifyPeerToPromote(const string& /*peer_uuid*/) override {}
+  void NotifyPeerHealthChange() override {}
+
+ private:
+  FRIEND_TEST(ConsensusQueueTest, TestTransferLeadershipWhenAppropriate);
+
+  // The following track the notifications sent in chronological order.
+  deque<string> peers_to_start_election_;
+};
+
+// Test that the leader consensus queue will only attempt to trigger elections
+// when appropriate.
+TEST_F(ConsensusQueueTest, TestTransferLeadershipWhenAppropriate) {
+  SimpleObserver observer;
+  queue_->RegisterObserver(&observer);
+  RaftConfigPB config = BuildRaftConfigPBForTests(/*num_voters*/2);
+  queue_->SetLeaderMode(kMinimumOpIdIndex, kMinimumTerm, config);
+  RaftPeerPB follower = MakePeer(kPeerUuid, RaftPeerPB::VOTER);
+  queue_->TrackPeer(follower);
+
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 10);
+  WaitForLocalPeerToAckIndex(10);
+
+  ConsensusResponsePB peer_response;
+  peer_response.set_responder_term(1);
+  peer_response.set_responder_uuid(kPeerUuid);
+  SetLastReceivedAndLastCommitted(&peer_response, MakeOpId(1, 9), MinimumOpId().index());
+
+  int elections_so_far = 0;
+  // Simulates receiving the peer's response and checks that, upon receiving
+  // it, the PeerMessageQueue either did or didn't notify that the peer should
+  // start an election.
+  auto verify_elections = [&] (bool election_happened) {
+    ASSERT_TRUE(queue_->ResponseFromPeer(kPeerUuid, peer_response));
+    // Notifications are communicated via the Raft threadpool, so wait for any
+    // such notifying tasks to finish.
+    raft_pool_->Wait();
+    if (election_happened) {
+      elections_so_far++;
+    }
+    ASSERT_EQ(elections_so_far, observer.peers_to_start_election_.size());
+  };
+  // We haven't begun watching for a successor yet and our conditions aren't
+  // met for this peer to become a leader.
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // Even after waiting for a successor, this peer isn't ready yet.
+  queue_->BeginWatchForSuccessor(boost::none);
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // Once the peer says it's gotten the last-appended op, we should be good to
+  // transfer leadership to it.
+  SetLastReceivedAndLastCommitted(&peer_response, MakeOpId(1, 10), MinimumOpId().index());
+  NO_FATALS(verify_elections(/*election_happened*/true));
+
+  // After we've triggered our election, we shouldn't trigger another.
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // And if we try to step down but specify a different peer, we also won't try
+  // electing the peer in-hand.
+  queue_->BeginWatchForSuccessor(boost::make_optional<string>("different-peer"));
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // Even if we begin quiescing, because we're looking for a specific
+  // successor, we shouldn't see an election.
+  quiescing_ = true;
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // If we stop watching for that successor and we're quiescing, we'll trigger
+  // elections.
+  queue_->EndWatchForSuccessor();
+  for (int i = 0; i < 3; i++) {
+    NO_FATALS(verify_elections(/*election_happened*/true));
+  }
+
+  // If the peer weren't a voter, we would also not trigger elections.
+  config.mutable_peers(1)->set_member_type(RaftPeerPB::NON_VOTER);
+  queue_->SetLeaderMode(10, 1, config);
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // Now undo that.
+  config.mutable_peers(1)->set_member_type(RaftPeerPB::VOTER);
+  queue_->SetLeaderMode(10, 1, config);
+  NO_FATALS(verify_elections(/*election_happened*/true));
+
+  // If the peer reported itself as quiescing, we also wouldn't trigger an
+  // election.
+  peer_response.set_server_quiescing(true);
+  NO_FATALS(verify_elections(/*election_happened*/false));
+
+  // If the peer stops reporting its server as quiescing, elections will start
+  // up again.
+  peer_response.clear_server_quiescing();
+  NO_FATALS(verify_elections(/*election_happened*/true));
+  peer_response.set_server_quiescing(false);
+  NO_FATALS(verify_elections(/*election_happened*/true));
+}
+
 // Tests that the queue is able to track a peer when it starts tracking a peer
 // after the initial message in the queue. In particular this creates a queue
 // with several messages and then starts to track a peer whose watermark
 // falls in the middle of the current messages in the queue.
 TEST_F(ConsensusQueueTest, TestStartTrackingAfterStart) {
   queue_->SetLeaderMode(kMinimumOpIdIndex, kMinimumTerm, BuildRaftConfigPBForTests(2));
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 100);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 100);
 
   ConsensusRequestPB request;
   ConsensusResponsePB response;
@@ -318,7 +443,7 @@ TEST_F(ConsensusQueueTest, TestGetPagedMessages) {
   // Append the messages after the queue is tracked. Otherwise the ops might
   // get evicted from the cache immediately and the requests below would
   // result in async log reads instead of cache hits.
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 100);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 100);
 
   OpId last;
   for (int i = 0; i < 11; i++) {
@@ -350,7 +475,7 @@ TEST_F(ConsensusQueueTest, TestGetPagedMessages) {
 
 TEST_F(ConsensusQueueTest, TestPeersDontAckBeyondWatermarks) {
   queue_->SetLeaderMode(kMinimumOpIdIndex, kMinimumTerm, BuildRaftConfigPBForTests(3));
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 100);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 100);
 
   // Wait for the local peer to append all messages
   WaitForLocalPeerToAckIndex(100);
@@ -382,7 +507,7 @@ TEST_F(ConsensusQueueTest, TestPeersDontAckBeyondWatermarks) {
   ASSERT_FALSE(needs_tablet_copy);
   ASSERT_EQ(50, request.ops_size());
 
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 101, 100);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 101, 100);
 
   SetLastReceivedAndLastCommitted(&response, request.ops(49).id());
   response.set_responder_term(28);
@@ -424,7 +549,7 @@ TEST_F(ConsensusQueueTest, TestQueueAdvancesCommittedIndex) {
 
   // Append 10 messages to the queue.
   // This should add messages 0.1 -> 0.7, 1.8 -> 1.10 to the queue.
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 10);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 10);
   WaitForLocalPeerToAckIndex(10);
 
   // Since only the local log has ACKed at this point,
@@ -509,7 +634,7 @@ TEST_F(ConsensusQueueTest, TestNonVoterAcksDontCountTowardMajority) {
   // Append 10 messages to the queue.
   // This should add messages 0.1 -> 0.7, 1.8 -> 1.10 to the queue.
   const int kNumMessages = 10;
-  AppendReplicateMessagesToQueue(queue_.get(), clock_,
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(),
                                  /*first=*/ 1, /*count=*/ kNumMessages);
   WaitForLocalPeerToAckIndex(kNumMessages);
 
@@ -560,10 +685,10 @@ TEST_F(ConsensusQueueTest, TestQueueLoadsOperationsForPeer) {
 
   const int kOpsToAppend = 100;
   for (int i = 1; i <= kOpsToAppend; i++) {
-    ASSERT_OK(log::AppendNoOpToLogSync(clock_, log_.get(), &opid));
+    ASSERT_OK(log::AppendNoOpToLogSync(clock_.get(), log_.get(), &opid));
     // Roll the log every 10 ops
     if (i % 10 == 0) {
-      ASSERT_OK(log_->AllocateSegmentAndRollOver());
+      ASSERT_OK(log_->AllocateSegmentAndRollOverForTests());
     }
   }
   ASSERT_OK(log_->WaitUntilAllFlushed());
@@ -623,20 +748,20 @@ TEST_F(ConsensusQueueTest, TestQueueHandlesOperationOverwriting) {
   OpId opid = MakeOpId(1, 1);
   // Append 10 messages in term 1 to the log.
   for (int i = 1; i <= 10; i++) {
-    ASSERT_OK(log::AppendNoOpToLogSync(clock_, log_.get(), &opid));
+    ASSERT_OK(log::AppendNoOpToLogSync(clock_.get(), log_.get(), &opid));
     // Roll the log every 3 ops
     if (i % 3 == 0) {
-      ASSERT_OK(log_->AllocateSegmentAndRollOver());
+      ASSERT_OK(log_->AllocateSegmentAndRollOverForTests());
     }
   }
 
   opid = MakeOpId(2, 11);
   // Now append 10 more messages in term 2.
   for (int i = 11; i <= 20; i++) {
-    ASSERT_OK(log::AppendNoOpToLogSync(clock_, log_.get(), &opid));
+    ASSERT_OK(log::AppendNoOpToLogSync(clock_.get(), log_.get(), &opid));
     // Roll the log every 3 ops
     if (i % 3 == 0) {
-      ASSERT_OK(log_->AllocateSegmentAndRollOver());
+      ASSERT_OK(log_->AllocateSegmentAndRollOverForTests());
     }
   }
 
@@ -729,7 +854,7 @@ TEST_F(ConsensusQueueTest, TestQueueMovesWatermarksBackward) {
   queue_->SetNonLeaderMode(BuildRaftConfigPBForTests(3));
   // Append a bunch of messages and update as if they were also appeneded to the leader.
   queue_->UpdateLastIndexAppendedToLeader(10);
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 10);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 10);
   log_->WaitUntilAllFlushed();
 
   // Now rewrite some of the operations and wait for the log to append.
@@ -889,7 +1014,7 @@ TEST_F(ConsensusQueueTest, TestOnlyAdvancesWatermarkWhenPeerHasAPrefixOfOurLog)
 // Test that Tablet Copy is triggered when a "tablet not found" error occurs.
 TEST_F(ConsensusQueueTest, TestTriggerTabletCopyIfTabletNotFound) {
   queue_->SetLeaderMode(kMinimumOpIdIndex, kMinimumTerm, BuildRaftConfigPBForTests(3));
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 100);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 100);
 
   ConsensusRequestPB request;
   ConsensusResponsePB response;
@@ -926,10 +1051,10 @@ TEST_F(ConsensusQueueTest, TestFollowerCommittedIndexAndMetrics) {
 
   // Emulate a follower sending a request to replicate 10 messages.
   queue_->UpdateLastIndexAppendedToLeader(10);
-  AppendReplicateMessagesToQueue(queue_.get(), clock_, 1, 10);
+  AppendReplicateMessagesToQueue(queue_.get(), clock_.get(), 1, 10);
   WaitForLocalPeerToAckIndex(10);
 
-  // The committed_index should be MinimumOpId() since UpdateFollowerCommittedIndex
+  // The committed_index should be MinimumOpId() since UpdateFollowerWatermarks
   // has not been called.
   ASSERT_EQ(0, queue_->GetCommittedIndex());
 
