@@ -27,21 +27,20 @@
 
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/opid_util.h"
-#include "kudu/consensus/ref_counted_replicate.h"
 #include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/strings/util.h"
 #include "kudu/util/array_view.h" // IWYU pragma: keep
-#include "kudu/util/coding-inl.h"
 #include "kudu/util/coding.h"
+#include "kudu/util/coding-inl.h"
 #include "kudu/util/compression/compression.pb.h"
 #include "kudu/util/compression/compression_codec.h"
 #include "kudu/util/crc.h"
 #include "kudu/util/debug/trace_event.h"
-#include "kudu/util/env_util.h"
 #include "kudu/util/fault_injection.h"
+#include "kudu/util/file_cache.h"
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/pb_util.h"
@@ -66,6 +65,8 @@ DEFINE_double(fault_crash_before_write_log_segment_header, 0.0,
               "Fraction of the time we will crash just before writing the log segment header");
 TAG_FLAG(fault_crash_before_write_log_segment_header, unsafe);
 
+DECLARE_bool(fs_wal_use_file_cache);
+
 using kudu::consensus::OpId;
 using std::string;
 using std::shared_ptr;
@@ -109,7 +110,7 @@ LogOptions::LogOptions()
 // LogEntryReader
 ////////////////////////////////////////////////////////////
 
-LogEntryReader::LogEntryReader(ReadableLogSegment* seg)
+LogEntryReader::LogEntryReader(const ReadableLogSegment* seg)
     : seg_(seg),
       num_batches_read_(0),
       num_entries_read_(0),
@@ -258,24 +259,34 @@ Status LogEntryReader::MakeCorruptionStatus(const Status& status) const {
 ////////////////////////////////////////////////////////////
 
 Status ReadableLogSegment::Open(Env* env,
+                                FileCache* file_cache,
                                 const string& path,
                                 scoped_refptr<ReadableLogSegment>* segment) {
   VLOG(1) << "Parsing wal segment: " << path;
-  shared_ptr<RandomAccessFile> readable_file;
-  RETURN_NOT_OK_PREPEND(env_util::OpenFileForRandom(env, path, &readable_file),
-                        "Unable to open file for reading");
+  shared_ptr<RWFile> file;
+  if (PREDICT_TRUE(file_cache && FLAGS_fs_wal_use_file_cache)) {
+    RETURN_NOT_OK_PREPEND(file_cache->OpenFile<Env::MUST_EXIST>(path, &file),
+                          "unable to open file for reading");
+  } else {
+    unique_ptr<RWFile> f;
+    RWFileOptions opts;
+    opts.mode = Env::MUST_EXIST;
+    RETURN_NOT_OK_PREPEND(env->NewRWFile(opts, path, &f),
+                          "Unable to open file for reading");
+    file.reset(f.release());
+  }
 
-  segment->reset(new ReadableLogSegment(path, readable_file));
+  segment->reset(new ReadableLogSegment(path, std::move(file)));
   RETURN_NOT_OK_PREPEND((*segment)->Init(), "Unable to initialize segment");
   return Status::OK();
 }
 
 ReadableLogSegment::ReadableLogSegment(
-    std::string path, shared_ptr<RandomAccessFile> readable_file)
+    string path, shared_ptr<RWFile> file)
     : path_(std::move(path)),
       file_size_(0),
       readable_to_offset_(0),
-      readable_file_(std::move(readable_file)),
+      file_(std::move(file)),
       codec_(nullptr),
       is_initialized_(false),
       footer_was_rebuilt_(false) {}
@@ -354,7 +365,7 @@ Status ReadableLogSegment::InitCompressionCodec() {
   return Status::OK();
 }
 
-const int64_t ReadableLogSegment::readable_up_to() const {
+int64_t ReadableLogSegment::readable_up_to() const {
   return readable_to_offset_.Load();
 }
 
@@ -402,7 +413,7 @@ Status ReadableLogSegment::ReadFileSize() {
   // Env uses uint here, even though we generally prefer signed ints to avoid
   // underflow bugs. Use a local to convert.
   uint64_t size;
-  RETURN_NOT_OK_PREPEND(readable_file_->Size(&size), "Unable to read file size");
+  RETURN_NOT_OK_PREPEND(file_->Size(&size), "Unable to read file size");
   file_size_.Store(size);
   if (size == 0) {
     VLOG(1) << "Log segment file $0 is zero-length: " << path();
@@ -427,8 +438,8 @@ Status ReadableLogSegment::ReadHeader() {
   LogSegmentHeaderPB header;
 
   // Read and parse the log segment header.
-  RETURN_NOT_OK_PREPEND(readable_file_->Read(kLogSegmentHeaderMagicAndHeaderLength,
-                                             header_slice),
+  RETURN_NOT_OK_PREPEND(file_->Read(kLogSegmentHeaderMagicAndHeaderLength,
+                                    header_slice),
                         "Unable to read fully");
 
   RETURN_NOT_OK_PREPEND(pb_util::ParseFromArray(&header,
@@ -441,23 +452,23 @@ Status ReadableLogSegment::ReadHeader() {
                                 "of Kudu");
   }
 
-  header_.Swap(&header);
+  header_ = std::move(header);
   first_entry_offset_ = header_size + kLogSegmentHeaderMagicAndHeaderLength;
 
   return Status::OK();
 }
 
 
-Status ReadableLogSegment::ReadHeaderMagicAndHeaderLength(uint32_t *len) {
+Status ReadableLogSegment::ReadHeaderMagicAndHeaderLength(uint32_t *len) const {
   uint8_t scratch[kLogSegmentHeaderMagicAndHeaderLength];
   Slice slice(scratch, kLogSegmentHeaderMagicAndHeaderLength);
-  RETURN_NOT_OK(readable_file_->Read(0, slice));
+  RETURN_NOT_OK(file_->Read(0, slice));
   RETURN_NOT_OK(ParseHeaderMagicAndHeaderLength(slice, len));
   return Status::OK();
 }
 
-Status ReadableLogSegment::ParseHeaderMagicAndHeaderLength(const Slice &data,
-                                                           uint32_t *parsed_len) {
+Status ReadableLogSegment::ParseHeaderMagicAndHeaderLength(
+    const Slice &data, uint32_t *parsed_len) const {
   RETURN_NOT_OK_PREPEND(data.check_size(kLogSegmentHeaderMagicAndHeaderLength),
                         "Log segment file is too small to contain initial magic number");
 
@@ -509,7 +520,7 @@ Status ReadableLogSegment::ReadFooter() {
   LogSegmentFooterPB footer;
 
   // Read and parse the log segment footer.
-  RETURN_NOT_OK_PREPEND(readable_file_->Read(footer_offset, footer_slice),
+  RETURN_NOT_OK_PREPEND(file_->Read(footer_offset, footer_slice),
                         "Footer not found. Could not read fully.");
 
   RETURN_NOT_OK_PREPEND(pb_util::ParseFromArray(&footer,
@@ -517,24 +528,24 @@ Status ReadableLogSegment::ReadFooter() {
                                                 footer_size),
                         "Unable to parse protobuf");
 
-  footer_.Swap(&footer);
+  footer_ = std::move(footer);
   return Status::OK();
 }
 
-Status ReadableLogSegment::ReadFooterMagicAndFooterLength(uint32_t *len) {
+Status ReadableLogSegment::ReadFooterMagicAndFooterLength(uint32_t *len) const {
   uint8_t scratch[kLogSegmentFooterMagicAndFooterLength];
   Slice slice(scratch, kLogSegmentFooterMagicAndFooterLength);
 
   CHECK_GT(file_size(), kLogSegmentFooterMagicAndFooterLength);
-  RETURN_NOT_OK(readable_file_->Read(file_size() - kLogSegmentFooterMagicAndFooterLength,
-                                     slice));
+  RETURN_NOT_OK(file_->Read(file_size() - kLogSegmentFooterMagicAndFooterLength,
+                            slice));
 
   RETURN_NOT_OK(ParseFooterMagicAndFooterLength(slice, len));
   return Status::OK();
 }
 
-Status ReadableLogSegment::ParseFooterMagicAndFooterLength(const Slice &data,
-                                                           uint32_t *parsed_len) {
+Status ReadableLogSegment::ParseFooterMagicAndFooterLength(
+    const Slice &data, uint32_t *parsed_len) {
   RETURN_NOT_OK_PREPEND(data.check_size(kLogSegmentFooterMagicAndFooterLength),
                         "Slice is too small to contain final magic number");
 
@@ -547,7 +558,7 @@ Status ReadableLogSegment::ParseFooterMagicAndFooterLength(const Slice &data,
   return Status::OK();
 }
 
-Status ReadableLogSegment::ReadEntries(LogEntries* entries) {
+Status ReadableLogSegment::ReadEntries(LogEntries* entries) const {
   TRACE_EVENT1("log", "ReadableLogSegment::ReadEntries",
                "path", path_);
   LogEntryReader reader(this);
@@ -571,7 +582,8 @@ size_t ReadableLogSegment::entry_header_size() const {
   return header_.has_deprecated_major_version() ? kEntryHeaderSizeV1 : kEntryHeaderSizeV2;
 }
 
-Status ReadableLogSegment::ScanForValidEntryHeaders(int64_t offset, bool* has_valid_entries) {
+Status ReadableLogSegment::ScanForValidEntryHeaders(
+    int64_t offset, bool* has_valid_entries) const {
   TRACE_EVENT1("log", "ReadableLogSegment::ScanForValidEntryHeaders",
                "path", path_);
   VLOG(1) << "Scanning " << path_ << " for valid entry headers "
@@ -588,7 +600,7 @@ Status ReadableLogSegment::ScanForValidEntryHeaders(int64_t offset, bool* has_va
        offset += kChunkSize - entry_header_size()) {
     int rem = std::min<int64_t>(file_size() - offset, kChunkSize);
     Slice chunk(buf.get(), rem);
-    RETURN_NOT_OK(readable_file()->Read(offset, chunk));
+    RETURN_NOT_OK(file_->Read(offset, chunk));
 
     // Optimization for the case where a chunk is all zeros -- this is common in the
     // case of pre-allocated files. This avoids a lot of redundant CRC calculation.
@@ -617,7 +629,7 @@ Status ReadableLogSegment::ScanForValidEntryHeaders(int64_t offset, bool* has_va
 
 Status ReadableLogSegment::ReadEntryHeaderAndBatch(int64_t* offset, faststring* tmp_buf,
                                                    unique_ptr<LogEntryBatchPB>* batch,
-                                                   EntryHeaderStatus* status_detail) {
+                                                   EntryHeaderStatus* status_detail) const {
   int64_t cur_offset = *offset;
   EntryHeader header;
   RETURN_NOT_OK(ReadEntryHeader(&cur_offset, &header, status_detail));
@@ -633,11 +645,11 @@ Status ReadableLogSegment::ReadEntryHeaderAndBatch(int64_t* offset, faststring*
 }
 
 Status ReadableLogSegment::ReadEntryHeader(int64_t *offset, EntryHeader* header,
-                                           EntryHeaderStatus* status_detail) {
+                                           EntryHeaderStatus* status_detail) const {
   const size_t header_size = entry_header_size();
   uint8_t scratch[header_size];
   Slice slice(scratch, header_size);
-  RETURN_NOT_OK_PREPEND(readable_file()->Read(*offset, slice),
+  RETURN_NOT_OK_PREPEND(file_->Read(*offset, slice),
                         "Could not read log entry header");
 
   *status_detail = DecodeEntryHeader(slice, header);
@@ -658,7 +670,7 @@ Status ReadableLogSegment::ReadEntryHeader(int64_t *offset, EntryHeader* header,
 }
 
 EntryHeaderStatus ReadableLogSegment::DecodeEntryHeader(
-    const Slice& data, EntryHeader* header) {
+    const Slice& data, EntryHeader* header) const {
   uint32_t computed_header_crc;
   if (entry_header_size() == kEntryHeaderSizeV2) {
     header->msg_length_compressed = DecodeFixed32(&data[0]);
@@ -689,7 +701,7 @@ EntryHeaderStatus ReadableLogSegment::DecodeEntryHeader(
 Status ReadableLogSegment::ReadEntryBatch(int64_t* offset,
                                           const EntryHeader& header,
                                           faststring* tmp_buf,
-                                          unique_ptr<LogEntryBatchPB>* entry_batch) {
+                                          unique_ptr<LogEntryBatchPB>* entry_batch) const {
   TRACE_EVENT2("log", "ReadableLogSegment::ReadEntryBatch",
                "path", path_,
                "range", Substitute("offset=$0 entry_len=$1",
@@ -715,7 +727,7 @@ Status ReadableLogSegment::ReadEntryBatch(int64_t* offset,
   }
   tmp_buf->resize(buf_len);
   Slice entry_batch_slice(tmp_buf->data(), header.msg_length_compressed);
-  Status s =  readable_file()->Read(*offset, entry_batch_slice);
+  Status s =  file_->Read(*offset, entry_batch_slice);
 
   if (!s.ok()) return Status::IOError(Substitute("Could not read entry. Cause: $0",
                                                  s.ToString()));
@@ -753,14 +765,14 @@ Status ReadableLogSegment::ReadEntryBatch(int64_t* offset,
 }
 
 WritableLogSegment::WritableLogSegment(string path,
-                                       shared_ptr<WritableFile> writable_file)
+                                       shared_ptr<RWFile> file)
     : path_(std::move(path)),
-      writable_file_(std::move(writable_file)),
+      file_(std::move(file)),
       is_header_written_(false),
       is_footer_written_(false),
       written_offset_(0) {}
 
-Status WritableLogSegment::WriteHeaderAndOpen(const LogSegmentHeaderPB& new_header) {
+Status WritableLogSegment::WriteHeader(const LogSegmentHeaderPB& new_header) {
   MAYBE_FAULT(FLAGS_fault_crash_before_write_log_segment_header);
 
   DCHECK(!IsHeaderWritten()) << "Can only call WriteHeader() once";
@@ -774,7 +786,7 @@ Status WritableLogSegment::WriteHeaderAndOpen(const LogSegmentHeaderPB& new_head
   PutFixed32(&buf, new_header.ByteSize());
   // Then Serialize the PB.
   pb_util::AppendToString(new_header, &buf);
-  RETURN_NOT_OK(writable_file()->Append(Slice(buf)));
+  RETURN_NOT_OK(file_->Write(0, Slice(buf)));
 
   header_.CopyFrom(new_header);
   first_entry_offset_ = buf.size();
@@ -784,9 +796,8 @@ Status WritableLogSegment::WriteHeaderAndOpen(const LogSegmentHeaderPB& new_head
   return Status::OK();
 }
 
-Status WritableLogSegment::WriteFooterAndClose(const LogSegmentFooterPB& footer) {
-  TRACE_EVENT1("log", "WritableLogSegment::WriteFooterAndClose",
-               "path", path_);
+Status WritableLogSegment::WriteFooter(const LogSegmentFooterPB& footer) {
+  TRACE_EVENT1("log", "WritableLogSegment::WriteFooter", "path", path_);
   DCHECK(IsHeaderWritten());
   DCHECK(!IsFooterWritten());
   DCHECK(footer.IsInitialized()) << footer.InitializationErrorString();
@@ -796,14 +807,12 @@ Status WritableLogSegment::WriteFooterAndClose(const LogSegmentFooterPB& footer)
   buf.append(kLogSegmentFooterMagicString);
   PutFixed32(&buf, footer.ByteSize());
 
-  RETURN_NOT_OK_PREPEND(writable_file()->Append(Slice(buf)), "Could not write the footer");
+  RETURN_NOT_OK_PREPEND(file_->Write(written_offset_, Slice(buf)),
+                        "Could not write the footer");
 
   footer_.CopyFrom(footer);
-  is_footer_written_ = true;
-
-  RETURN_NOT_OK(writable_file_->Close());
-
   written_offset_ += buf.size();
+  is_footer_written_ = true;
 
   return Status::OK();
 }
@@ -839,22 +848,14 @@ Status WritableLogSegment::WriteEntryBatch(const Slice& data,
   Slice slices[2] = {
     Slice(header_buf, arraysize(header_buf)),
     data_to_write };
-  RETURN_NOT_OK(writable_file_->AppendV(slices));
+  RETURN_NOT_OK(file_->WriteV(written_offset_, slices));
   written_offset_ += arraysize(header_buf) + data_to_write.size();
   return Status::OK();
 }
 
-
-unique_ptr<LogEntryBatchPB> CreateBatchFromAllocatedOperations(
-    const vector<consensus::ReplicateRefPtr>& msgs) {
-  unique_ptr<LogEntryBatchPB> entry_batch(new LogEntryBatchPB);
-  entry_batch->mutable_entry()->Reserve(msgs.size());
-  for (const auto& msg : msgs) {
-    LogEntryPB* entry_pb = entry_batch->add_entry();
-    entry_pb->set_type(log::REPLICATE);
-    entry_pb->set_allocated_replicate(msg->get());
-  }
-  return entry_batch;
+void WritableLogSegment::GoIdle() {
+  compress_buf_.clear();
+  compress_buf_.shrink_to_fit();
 }
 
 bool IsLogFileName(const string& fname) {
