@@ -17,10 +17,10 @@
 
 #include <algorithm>
 #include <cstdint>
+#include <functional>
 #include <memory>
 #include <ostream>
 #include <string>
-#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
@@ -30,7 +30,6 @@
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
-#include "kudu/clock/clock.h"
 #include "kudu/clock/logical_clock.h"
 #include "kudu/common/common.pb.h"
 #include "kudu/common/schema.h"
@@ -55,12 +54,9 @@
 #include "kudu/consensus/raft_consensus.h"
 #include "kudu/consensus/time_manager.h"
 #include "kudu/fs/fs_manager.h"
-#include "kudu/gutil/bind.h"
 #include "kudu/gutil/casts.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/strcat.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/metadata.pb.h"
@@ -119,9 +115,9 @@ class RaftConsensusQuorumTest : public KuduTest {
   typedef vector<unique_ptr<LogEntryPB>> LogEntries;
 
   RaftConsensusQuorumTest()
-    : clock_(clock::LogicalClock::CreateStartingAt(Timestamp(1))),
-      metric_entity_(METRIC_ENTITY_tablet.Instantiate(&metric_registry_, "raft-test")),
-      schema_(GetSimpleTestSchema()) {
+      : clock_(Timestamp(1)),
+        metric_entity_(METRIC_ENTITY_tablet.Instantiate(&metric_registry_, "raft-test")),
+        schema_(GetSimpleTestSchema()) {
     options_.tablet_id = kTestTablet;
     FLAGS_enable_leader_failure_detection = false;
     CHECK_OK(ThreadPoolBuilder("raft").Build(&raft_pool_));
@@ -147,24 +143,21 @@ class RaftConsensusQuorumTest : public KuduTest {
       opts.parent_mem_tracker = parent_mem_tracker;
       opts.wal_root = test_path;
       opts.data_roots = { test_path };
-      gscoped_ptr<FsManager> fs_manager(new FsManager(env_, opts));
+      unique_ptr<FsManager> fs_manager(new FsManager(env_, opts));
       RETURN_NOT_OK(fs_manager->CreateInitialFileSystemLayout());
       RETURN_NOT_OK(fs_manager->Open());
 
-      scoped_refptr<ConsensusMetadataManager> cmeta_manager(
-          new ConsensusMetadataManager(fs_manager.get()));
-      cmeta_managers_.push_back(cmeta_manager);
-
       scoped_refptr<Log> log;
       RETURN_NOT_OK(Log::Open(LogOptions(),
                               fs_manager.get(),
+                              /*file_cache*/nullptr,
                               kTestTablet,
                               schema_,
                               0, // schema_version
-                              nullptr,
+                              /*metric_entity*/nullptr,
                               &log));
       logs_.emplace_back(std::move(log));
-      fs_managers_.push_back(fs_manager.release());
+      fs_managers_.push_back(std::move(fs_manager));
     }
     return Status::OK();
   }
@@ -184,19 +177,25 @@ class RaftConsensusQuorumTest : public KuduTest {
   }
 
   Status BuildPeers() {
-    CHECK_EQ(config_.peers_size(), cmeta_managers_.size());
     CHECK_EQ(config_.peers_size(), fs_managers_.size());
     for (int i = 0; i < config_.peers_size(); i++) {
-      RETURN_NOT_OK(cmeta_managers_[i]->Create(kTestTablet, config_, kMinimumTerm));
+      FsManager* fs = fs_managers_[i].get();
+      scoped_refptr<ConsensusMetadataManager> cmeta_manager(
+          new ConsensusMetadataManager(fs));
+
+      RETURN_NOT_OK(cmeta_manager->Create(kTestTablet, config_, kMinimumTerm));
 
       RaftPeerPB* local_peer_pb;
-      RETURN_NOT_OK(GetRaftConfigMember(&config_, fs_managers_[i]->uuid(), &local_peer_pb));
+      RETURN_NOT_OK(GetRaftConfigMember(&config_, fs->uuid(), &local_peer_pb));
 
       shared_ptr<RaftConsensus> peer;
+      ServerContext ctx({ /*quiescing*/nullptr,
+                          /*num_leaders*/nullptr,
+                          raft_pool_.get() });
       RETURN_NOT_OK(RaftConsensus::Create(options_,
                                           config_.peers(i),
-                                          cmeta_managers_[i],
-                                          raft_pool_.get(),
+                                          std::move(cmeta_manager),
+                                          std::move(ctx),
                                           &peer));
       peers_->AddPeer(config_.peers(i).permanent_uuid(), peer);
     }
@@ -211,20 +210,23 @@ class RaftConsensusQuorumTest : public KuduTest {
       shared_ptr<RaftConsensus> peer;
       RETURN_NOT_OK(peers_->GetPeerByIdx(i, &peer));
 
-      gscoped_ptr<PeerProxyFactory> proxy_factory(new LocalTestPeerProxyFactory(peers_.get()));
-      scoped_refptr<TimeManager> time_manager(new TimeManager(clock_, Timestamp::kMin));
-      auto txn_factory = new TestTransactionFactory(logs_[i].get());
+      unique_ptr<PeerProxyFactory> proxy_factory(
+          new LocalTestPeerProxyFactory(peers_.get()));
+      unique_ptr<TimeManager> time_manager(
+          new TimeManager(&clock_, Timestamp::kMin));
+      unique_ptr<TestTransactionFactory> txn_factory(
+          new TestTransactionFactory(logs_[i].get()));
       txn_factory->SetConsensus(peer.get());
-      txn_factories_.push_back(txn_factory);
+      txn_factories_.emplace_back(std::move(txn_factory));
 
       RETURN_NOT_OK(peer->Start(
           boot_info,
           std::move(proxy_factory),
           logs_[i],
-          time_manager,
-          txn_factory,
+          std::move(time_manager),
+          txn_factories_.back().get(),
           metric_entity_,
-          Bind(&DoNothing)));
+          &DoNothing));
     }
     return Status::OK();
   }
@@ -244,7 +246,7 @@ class RaftConsensusQuorumTest : public KuduTest {
     const int kLeaderIdx = num - 1;
     shared_ptr<RaftConsensus> leader;
     RETURN_NOT_OK(peers_->GetPeerByIdx(kLeaderIdx, &leader));
-    RETURN_NOT_OK(leader->EmulateElection());
+    RETURN_NOT_OK(leader->EmulateElectionForTests());
     return Status::OK();
   }
 
@@ -265,18 +267,18 @@ class RaftConsensusQuorumTest : public KuduTest {
 
   Status AppendDummyMessage(int peer_idx,
                             scoped_refptr<ConsensusRound>* round) {
-    gscoped_ptr<ReplicateMsg> msg(new ReplicateMsg());
+    unique_ptr<ReplicateMsg> msg(new ReplicateMsg());
     msg->set_op_type(NO_OP);
     msg->mutable_noop_request();
-    msg->set_timestamp(clock_->Now().ToUint64());
+    msg->set_timestamp(clock_.Now().ToUint64());
 
     shared_ptr<RaftConsensus> peer;
     CHECK_OK(peers_->GetPeerByIdx(peer_idx, &peer));
 
     // Use a latch in place of a Transaction callback.
-    gscoped_ptr<Synchronizer> sync(new Synchronizer());
-    *round = peer->NewRound(std::move(msg), sync->AsStdStatusCallback());
-    InsertOrDie(&syncs_, round->get(), sync.release());
+    unique_ptr<Synchronizer> sync(new Synchronizer());
+    *round = peer->NewRound(std::move(msg), sync->AsStatusCallback());
+    EmplaceOrDie(&syncs_, round->get(), std::move(sync));
     RETURN_NOT_OK_PREPEND(peer->Replicate(round->get()),
                           Substitute("Unable to replicate to peer $0", peer_idx));
     return Status::OK();
@@ -291,13 +293,14 @@ class RaftConsensusQuorumTest : public KuduTest {
                             shared_ptr<Synchronizer>* commit_sync = nullptr) {
     StatusCallback commit_callback;
     if (commit_sync != nullptr) {
-      commit_sync->reset(new Synchronizer());
-      commit_callback = Bind(&FireSharedSynchronizer, *commit_sync);
+      shared_ptr<Synchronizer> sync(std::make_shared<Synchronizer>());
+      commit_callback = [sync](const Status& s) { FireSharedSynchronizer(sync, s); };
+      *commit_sync = std::move(sync);
     } else {
-      commit_callback = Bind(&DoNothingStatusCB);
+      commit_callback = &DoNothingStatusCB;
     }
 
-    gscoped_ptr<CommitMsg> msg(new CommitMsg());
+    unique_ptr<CommitMsg> msg(new CommitMsg());
     msg->set_op_type(NO_OP);
     msg->mutable_commited_op_id()->CopyFrom(round->id());
     CHECK_OK(logs_[peer_idx]->AsyncAppendCommit(std::move(msg), commit_callback));
@@ -429,13 +432,14 @@ class RaftConsensusQuorumTest : public KuduTest {
     ASSERT_OK(log->WaitUntilAllFlushed());
     log->Close();
     shared_ptr<LogReader> log_reader;
-    ASSERT_OK(log::LogReader::Open(fs_managers_[idx],
-                                   scoped_refptr<log::LogIndex>(),
+    ASSERT_OK(log::LogReader::Open(fs_managers_[idx].get(),
+                                   /*index*/nullptr,
                                    kTestTablet,
                                    metric_entity_.get(),
+                                   /*file_cache*/nullptr,
                                    &log_reader));
     log::SegmentSequence segments;
-    ASSERT_OK(log_reader->GetSegmentsSnapshot(&segments));
+    log_reader->GetSegmentsSnapshot(&segments);
 
     LogEntries ret;
     for (const log::SegmentSequence::value_type& entry : segments) {
@@ -450,7 +454,7 @@ class RaftConsensusQuorumTest : public KuduTest {
   void VerifyLogs(int leader_idx, int first_replica_idx, int last_replica_idx) {
     // Wait for in-flight transactions to be done. We're destroying the
     // peers next and leader transactions won't be able to commit anymore.
-    for (TestTransactionFactory* factory : txn_factories_) {
+    for (const auto& factory : txn_factories_) {
       factory->WaitDone();
     }
 
@@ -543,8 +547,9 @@ class RaftConsensusQuorumTest : public KuduTest {
 
   // Read the ConsensusMetadata for the given peer from disk.
   scoped_refptr<ConsensusMetadata> ReadConsensusMetadataFromDisk(int peer_index) {
+    FsManager* fs = fs_managers_[peer_index].get();
     scoped_refptr<ConsensusMetadata> cmeta;
-    CHECK_OK(cmeta_managers_[peer_index]->Load(kTestTablet, &cmeta));
+    CHECK_OK(ConsensusMetadata::Load(fs, kTestTablet, fs->uuid(), &cmeta));
     return cmeta;
   }
 
@@ -564,12 +569,6 @@ class RaftConsensusQuorumTest : public KuduTest {
 
   ~RaftConsensusQuorumTest() {
     peers_->Clear();
-    STLDeleteElements(&txn_factories_);
-    // We need to clear the logs before deleting the fs_managers_ or we'll
-    // get a SIGSEGV when closing the logs.
-    logs_.clear();
-    STLDeleteElements(&fs_managers_);
-    STLDeleteValues(&syncs_);
   }
 
  protected:
@@ -577,17 +576,16 @@ class RaftConsensusQuorumTest : public KuduTest {
   RaftConfigPB config_;
   OpId initial_id_;
   vector<shared_ptr<MemTracker>> parent_mem_trackers_;
-  vector<FsManager*> fs_managers_;
+  vector<unique_ptr<FsManager>> fs_managers_;
   vector<scoped_refptr<Log> > logs_;
   unique_ptr<ThreadPool> raft_pool_;
-  vector<scoped_refptr<ConsensusMetadataManager>> cmeta_managers_;
-  gscoped_ptr<TestPeerMapManager> peers_;
-  vector<TestTransactionFactory*> txn_factories_;
-  scoped_refptr<clock::Clock> clock_;
+  unique_ptr<TestPeerMapManager> peers_;
+  vector<unique_ptr<TestTransactionFactory>> txn_factories_;
+  clock::LogicalClock clock_;
   MetricRegistry metric_registry_;
   scoped_refptr<MetricEntity> metric_entity_;
   const Schema schema_;
-  std::unordered_map<ConsensusRound*, Synchronizer*> syncs_;
+  std::unordered_map<ConsensusRound*, unique_ptr<Synchronizer>> syncs_;
 };
 
 // Tests Replicate/Commit a single message through the leader.
@@ -828,7 +826,7 @@ TEST_F(RaftConsensusQuorumTest, TestLeaderHeartbeats) {
 
   shared_ptr<RaftConsensus> leader;
   CHECK_OK(peers_->GetPeerByIdx(kLeaderIdx, &leader));
-  ASSERT_OK(leader->EmulateElection());
+  ASSERT_OK(leader->EmulateElectionForTests());
 
   // Wait for the config round to get committed and count the number
   // of update calls, calls after that will be heartbeats.
@@ -961,7 +959,7 @@ TEST_F(RaftConsensusQuorumTest, TestReplicasEnforceTheLogMatchingProperty) {
 
   // Send a request with the next index.
   ReplicateMsg* replicate = req.add_ops();
-  replicate->set_timestamp(clock_->Now().ToUint64());
+  replicate->set_timestamp(clock_.Now().ToUint64());
   OpId* id = replicate->mutable_id();
   id->set_term(last_op_id.term());
   id->set_index(last_op_id.index() + 1);
