@@ -23,12 +23,12 @@
 #include <mutex>
 #include <ostream>
 #include <string>
+#include <thread>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
@@ -41,7 +41,6 @@
 #include "kudu/fs/fs_report.h"
 #include "kudu/fs/log_block_manager-test-util.h"
 #include "kudu/fs/log_block_manager.h"  // IWYU pragma: keep
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/split.h"
@@ -51,19 +50,19 @@
 #include "kudu/util/env.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/file_cache-test-util.h"
+#include "kudu/util/file_cache.h"
 #include "kudu/util/locks.h"
+#include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/random.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
-#include "kudu/util/thread.h"
 
 DECLARE_bool(cache_force_single_shard);
 DECLARE_double(log_container_excess_space_before_cleanup_fraction);
 DECLARE_double(log_container_live_metadata_before_compact_ratio);
-DECLARE_int64(block_manager_max_open_files);
 DECLARE_uint64(log_container_max_size);
 DECLARE_uint64(log_container_preallocate_bytes);
 
@@ -86,9 +85,11 @@ DEFINE_int32(num_inconsistencies, 16,
 DEFINE_string(block_manager_paths, "", "Comma-separated list of paths to "
               "use for block storage. If empty, will use the default unit "
               "test path");
+DEFINE_int32(max_open_files, 32, "Maximum size of the test's file cache");
 
 using std::string;
 using std::shared_ptr;
+using std::thread;
 using std::unique_ptr;
 using std::unordered_map;
 using std::vector;
@@ -119,23 +120,21 @@ template <typename T>
 class BlockManagerStressTest : public KuduTest {
  public:
   BlockManagerStressTest() :
-    rand_seed_(SeedRandom()),
-    stop_latch_(1),
-    test_error_manager_(new FsErrorManager()),
-    test_tablet_name_("test_tablet"),
-    total_blocks_written_(0),
-    total_bytes_written_(0),
-    total_blocks_read_(0),
-    total_bytes_read_(0),
-    total_blocks_deleted_(0) {
+      rand_seed_(SeedRandom()),
+      stop_latch_(1),
+      file_cache_("test_cache", env_, FLAGS_max_open_files,
+                  scoped_refptr<MetricEntity>()),
+      test_tablet_name_("test_tablet"),
+      total_blocks_written_(0),
+      total_bytes_written_(0),
+      total_blocks_read_(0),
+      total_bytes_read_(0),
+      total_blocks_deleted_(0) {
 
     // Increase the number of containers created.
     FLAGS_log_container_max_size = 1 * 1024 * 1024;
     FLAGS_log_container_preallocate_bytes = 1 * 1024 * 1024;
 
-    // Ensure the file cache is under stress too.
-    FLAGS_block_manager_max_open_files = 32;
-
     // Maximize the amount of cleanup triggered by the extra space heuristic.
     FLAGS_log_container_excess_space_before_cleanup_fraction = 0.0;
 
@@ -148,25 +147,21 @@ class BlockManagerStressTest : public KuduTest {
 
     // Defer block manager creation until after the above flags are set.
     bm_.reset(CreateBlockManager());
-    bm_->Open(nullptr);
-    dd_manager_->CreateDataDirGroup(test_tablet_name_);
+    CHECK_OK(file_cache_.Init());
+    CHECK_OK(bm_->Open(nullptr));
+    CHECK_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
     CHECK_OK(dd_manager_->GetDataDirGroupPB(test_tablet_name_, &test_group_pb_));
   }
 
   virtual void TearDown() override {
-    // Ensure the proper destructor order. The directory manager must outlive
-    // the block manager.
-    bm_.reset();
-
     // If non-standard paths were provided we need to delete them in between
     // test runs.
     if (!FLAGS_block_manager_paths.empty()) {
-      for (const auto& dd : dd_manager_->GetDataRoots()) {
+      for (const auto& dd : dd_manager_->GetRoots()) {
         WARN_NOT_OK(env_->DeleteRecursively(dd),
                     Substitute("Couldn't recursively delete $0", dd));
       }
     }
-    dd_manager_.reset();
   }
 
   BlockManager* CreateBlockManager() {
@@ -190,8 +185,8 @@ class BlockManagerStressTest : public KuduTest {
       CHECK_OK(DataDirManager::OpenExistingForTests(env_, data_dirs,
           DataDirManagerOptions(), &dd_manager_));
     }
-    return new T(env_, dd_manager_.get(), test_error_manager_.get(),
-                 BlockManagerOptions());
+    return new T(env_, dd_manager_.get(), &error_manager_,
+                 &file_cache_, BlockManagerOptions());
   }
 
   void RunTest(double secs) {
@@ -200,43 +195,33 @@ class BlockManagerStressTest : public KuduTest {
     SleepFor(MonoDelta::FromSeconds(secs));
     LOG(INFO) << "Stopping all threads";
     this->StopThreads();
-    this->JoinThreads();
     this->stop_latch_.Reset(1);
+    this->threads_.clear();
   }
 
   void StartThreads() {
-    scoped_refptr<Thread> new_thread;
     for (int i = 0; i < FLAGS_num_writer_threads; i++) {
-      CHECK_OK(Thread::Create("BlockManagerStressTest", Substitute("writer-$0", i),
-                              &BlockManagerStressTest::WriterThread, this, &new_thread));
-      threads_.push_back(new_thread);
+      threads_.emplace_back([this]() { this->WriterThread(); });
     }
     for (int i = 0; i < FLAGS_num_reader_threads; i++) {
-      CHECK_OK(Thread::Create("BlockManagerStressTest", Substitute("reader-$0", i),
-                              &BlockManagerStressTest::ReaderThread, this, &new_thread));
-      threads_.push_back(new_thread);
+      threads_.emplace_back([this]() { this->ReaderThread(); });
     }
     for (int i = 0; i < FLAGS_num_deleter_threads; i++) {
-      CHECK_OK(Thread::Create("BlockManagerStressTest", Substitute("deleter-$0", i),
-                              &BlockManagerStressTest::DeleterThread, this, &new_thread));
-      threads_.push_back(new_thread);
+      threads_.emplace_back([this]() { this->DeleterThread(); });
     }
   }
 
   void StopThreads() {
     stop_latch_.CountDown();
+    for (auto& t : threads_) {
+      t.join();
+    }
   }
 
   bool ShouldStop(const MonoDelta& wait_time) {
     return stop_latch_.WaitFor(wait_time);
   }
 
-  void JoinThreads() {
-    for (const scoped_refptr<kudu::Thread>& thr : threads_) {
-     CHECK_OK(ThreadJoiner(thr.get()).Join());
-    }
-  }
-
   void WriterThread();
   void ReaderThread();
   void DeleterThread();
@@ -268,14 +253,13 @@ class BlockManagerStressTest : public KuduTest {
   // Protects written_blocks_.
   simple_spinlock lock_;
 
-  // The block manager.
-  gscoped_ptr<BlockManager> bm_;
-
-  // The directory manager.
   unique_ptr<DataDirManager> dd_manager_;
 
-  // The error manager.
-  unique_ptr<FsErrorManager> test_error_manager_;
+  FsErrorManager error_manager_;
+
+  FileCache file_cache_;
+
+  unique_ptr<BlockManager> bm_;
 
   // Test group of disk to spread data across.
   DataDirGroupPB test_group_pb_;
@@ -284,7 +268,7 @@ class BlockManagerStressTest : public KuduTest {
   string test_tablet_name_;
 
   // The running threads.
-  vector<scoped_refptr<Thread> > threads_;
+  vector<thread> threads_;
 
   // Some performance counters.
 
@@ -408,7 +392,7 @@ void BlockManagerStressTest<T>::ReaderThread() {
     // Read it fully into memory.
     uint64_t block_size;
     CHECK_OK(block->Size(&block_size));
-    gscoped_ptr<uint8_t[]> scratch(new uint8_t[block_size]);
+    unique_ptr<uint8_t[]> scratch(new uint8_t[block_size]);
     Slice data(scratch.get(), block_size);
     CHECK_OK(block->Read(0, data));
 
@@ -486,7 +470,7 @@ void BlockManagerStressTest<T>::DeleterThread() {
 
 template <>
 int BlockManagerStressTest<FileBlockManager>::GetMaxFdCount() const {
-  return FLAGS_block_manager_max_open_files +
+  return FLAGS_max_open_files +
       // Each open block exists outside the file cache.
       (FLAGS_num_writer_threads * FLAGS_block_group_size * FLAGS_block_group_number) +
       // Each reader thread can open a file outside the cache if its lookup
@@ -497,7 +481,7 @@ int BlockManagerStressTest<FileBlockManager>::GetMaxFdCount() const {
 
 template <>
 int BlockManagerStressTest<LogBlockManager>::GetMaxFdCount() const {
-  return FLAGS_block_manager_max_open_files +
+  return FLAGS_max_open_files +
       // If all containers are full, each open block could theoretically
       // result in a new container, which is two files briefly outside the
       // cache (before they are inserted and evict other cached files).
@@ -511,7 +495,7 @@ void BlockManagerStressTest<FileBlockManager>::InjectNonFatalInconsistencies() {
 
 template <>
 void BlockManagerStressTest<LogBlockManager>::InjectNonFatalInconsistencies() {
-  LBMCorruptor corruptor(env_, dd_manager_->GetDataDirs(), rand_seed_);
+  LBMCorruptor corruptor(env_, dd_manager_->GetDirs(), rand_seed_);
   ASSERT_OK(corruptor.Init());
 
   for (int i = 0; i < FLAGS_num_inconsistencies; i++) {
@@ -559,7 +543,7 @@ TYPED_TEST(BlockManagerStressTest, StressTest) {
     FsReport report;
     ASSERT_OK(this->bm_->Open(&report));
     ASSERT_OK(this->dd_manager_->LoadDataDirGroupFromPB(this->test_tablet_name_,
-                                                              this->test_group_pb_));
+                                                        this->test_group_pb_));
     ASSERT_OK(report.LogAndCheckForFatalErrors());
     this->RunTest(FLAGS_test_duration_secs / kNumStarts);
   }
