@@ -33,10 +33,10 @@
 
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
+#include "kudu/fs/fs.pb.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/util/atomic.h"
-#include "kudu/util/file_cache.h"
 #include "kudu/util/locks.h"
 #include "kudu/util/mem_tracker.h"
 #include "kudu/util/oid_generator.h"
@@ -44,13 +44,12 @@
 
 namespace kudu {
 
-class BlockRecordPB;
 class Env;
-class RWFile;
+class FileCache;
 
 namespace fs {
-class DataDir;
 class DataDirManager;
+class Dir;
 class FsErrorManager;
 struct FsReport;
 
@@ -59,7 +58,7 @@ class LogBlock;
 class LogBlockContainer;
 class LogBlockDeletionTransaction;
 class LogWritableBlock;
-
+struct LogBlockContainerLoadResult;
 struct LogBlockManagerMetrics;
 } // namespace internal
 
@@ -190,6 +189,7 @@ class LogBlockManager : public BlockManager {
   LogBlockManager(Env* env,
                   DataDirManager* dd_manager,
                   FsErrorManager* error_manager,
+                  FileCache* file_cache,
                   BlockManagerOptions opts);
 
   virtual ~LogBlockManager();
@@ -296,21 +296,20 @@ class LogBlockManager : public BlockManager {
   // use), false otherwise.
   bool TryUseBlockId(const BlockId& block_id);
 
-  // Adds a LogBlock to in-memory data structures.
+  // Creates and adds a LogBlock to in-memory data structures.
   //
   // Returns the created LogBlock if it was successfully added or nullptr if a
   // block with that ID was already present.
-  LogBlockRefPtr AddLogBlock(
+  LogBlockRefPtr CreateAndAddLogBlock(
       LogBlockContainerRefPtr container,
       const BlockId& block_id,
       int64_t offset,
       int64_t length);
 
-  // Unlocked variant of AddLogBlock() for an already-constructed LogBlock object.
-  // Must hold 'lock_'.
+  // Adds a LogBlock for an already-constructed LogBlock object.
   //
   // Returns true if the LogBlock was successfully added, false if it was already present.
-  bool AddLogBlockUnlocked(LogBlockRefPtr lb);
+  bool AddLogBlock(LogBlockRefPtr lb);
 
   // Removes the given set of LogBlocks from in-memory data structures, and
   // appends the block deletion metadata to record the on-disk deletion.
@@ -323,12 +322,15 @@ class LogBlockManager : public BlockManager {
                          std::vector<LogBlockRefPtr>* log_blocks,
                          std::vector<BlockId>* deleted);
 
-  // Removes a LogBlock from in-memory data structures. Must hold 'lock_'.
+  // Removes a LogBlock from in-memory data structures.
   // The 'lb' out parameter will be set with the successfully deleted LogBlock.
   //
   // Returns an error of LogBlock cannot be successfully removed.
-  Status RemoveLogBlockUnlocked(const BlockId& block_id,
-                                LogBlockRefPtr* lb);
+  Status RemoveLogBlock(const BlockId& block_id,
+                        LogBlockRefPtr* lb);
+
+  // Simple wrapper of Repair(), used as a runnable function in thread.
+  void RepairTask(Dir* dir, internal::LogBlockContainerLoadResult* result);
 
   // Repairs any inconsistencies for 'dir' described in 'report'.
   //
@@ -339,7 +341,7 @@ class LogBlockManager : public BlockManager {
   //    files compacted.
   //
   // Returns an error if repairing a fatal inconsistency failed.
-  Status Repair(DataDir* dir,
+  Status Repair(Dir* dir,
                 FsReport* report,
                 std::vector<LogBlockRefPtr> need_repunching,
                 std::vector<LogBlockContainerRefPtr> dead_containers,
@@ -360,14 +362,19 @@ class LogBlockManager : public BlockManager {
                              int64_t* file_bytes_delta);
 
   // Opens a particular data directory belonging to the block manager. The
-  // results of consistency checking (and repair, if applicable) are written to
-  // 'report'.
+  // results of consistency checking are written to 'results'.
   //
   // Success or failure is set in 'result_status'.
-  void OpenDataDir(DataDir* dir,
-                   FsReport* report,
+  void OpenDataDir(Dir* dir,
+                   std::vector<std::unique_ptr<internal::LogBlockContainerLoadResult>>* results,
                    Status* result_status);
 
+  // Reads records from one log block container in the data directory.
+  // The result details will be collected into 'result'.
+  void LoadContainer(Dir* dir,
+                     LogBlockContainerRefPtr container,
+                     internal::LogBlockContainerLoadResult* result);
+
   // Perform basic initialization.
   Status Init();
 
@@ -408,28 +415,34 @@ class LogBlockManager : public BlockManager {
   // interesting (e.g. LogBlocks).
   std::shared_ptr<MemTracker> mem_tracker_;
 
-  // Protects the block map, container structures, and 'dirty_dirs'.
+  // Block IDs container used to prevent collisions when creating new anonymous blocks.
+  struct ManagedBlockShard {
+    // Protects 'blocks_by_block_id' and 'open_block_ids'.
+    std::unique_ptr<simple_spinlock> lock;
+
+    // Maps block IDs to blocks that are now readable, either because they
+    // already existed on disk when the block manager was opened, or because
+    // they're WritableBlocks that were closed.
+    std::unique_ptr<BlockMap> blocks_by_block_id;
+
+    // Contains block IDs for WritableBlocks that are still open for writing.
+    // When a WritableBlock is closed, its ID is moved to 'blocks_by_block_id'.
+    BlockIdSet open_block_ids;
+  };
+
+  // Sharding block IDs containers.
+  std::vector<ManagedBlockShard> managed_block_shards_;
+
+  // Protects 'all_containers_by_name_', 'available_containers_by_data_dir_' and 'dirty_dirs'.
   mutable simple_spinlock lock_;
 
   // Maps a data directory to an upper bound on the number of blocks that a
   // container residing in that directory should observe, if one is necessary.
-  std::unordered_map<const DataDir*,
+  std::unordered_map<const Dir*,
                      boost::optional<int64_t>> block_limits_by_data_dir_;
 
   // Manages files opened for reading.
-  FileCache<RWFile> file_cache_;
-
-  // Maps block IDs to blocks that are now readable, either because they
-  // already existed on disk when the block manager was opened, or because
-  // they're WritableBlocks that were closed.
-  BlockMap blocks_by_block_id_;
-
-  // Contains block IDs for WritableBlocks that are still open for writing.
-  // When a WritableBlock is closed, its ID is moved to blocks_by_block_id.
-  //
-  // Together with blocks_by_block_id's keys, used to prevent collisions
-  // when creating new anonymous blocks.
-  BlockIdSet open_block_ids_;
+  FileCache* file_cache_;
 
   // Holds (and owns) all containers loaded from disk.
   std::unordered_map<std::string,
@@ -439,7 +452,7 @@ class LogBlockManager : public BlockManager {
   // excluding containers that are either in use or full.
   //
   // Does not own the containers.
-  std::unordered_map<const DataDir*,
+  std::unordered_map<const Dir*,
                      std::deque<LogBlockContainerRefPtr>> available_containers_by_data_dir_;
 
   // Tracks dirty container directories.
