@@ -17,10 +17,10 @@
 
 #include "kudu/hms/hms_catalog.h"
 
+#include <functional>
 #include <map>
 #include <memory>
 #include <string>
-#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -195,6 +195,7 @@ class HmsCatalogTest : public KuduTest {
     b.AddColumn("int32_val", DataType::INT32);
     b.AddColumn("int64_val", DataType::INT64);
     b.AddColumn("timestamp_val", DataType::UNIXTIME_MICROS);
+    b.AddColumn("date_val", DataType::DATE);
     b.AddColumn("string_val", DataType::STRING);
     b.AddColumn("bool_val", DataType::BOOL);
     b.AddColumn("float_val", DataType::FLOAT);
@@ -203,6 +204,7 @@ class HmsCatalogTest : public KuduTest {
     b.AddColumn("decimal32_val", DataType::DECIMAL32);
     b.AddColumn("decimal64_val", DataType::DECIMAL64);
     b.AddColumn("decimal128_val", DataType::DECIMAL128);
+    b.AddColumn("varchar_val", DataType::VARCHAR);
     return b.Build();
   }
 
