@@ -30,7 +30,6 @@
 #include <boost/optional/optional.hpp> // IWYU pragma: keep
 #include <boost/optional/optional_io.hpp> // IWYU pragma: keep
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <glog/stl_logging.h>
 #include <gtest/gtest.h>
@@ -40,7 +39,7 @@
 #include "kudu/client/scan_batch.h"
 #include "kudu/client/scan_predicate.h"
 #include "kudu/client/schema.h"
-#include "kudu/client/shared_ptr.h"
+#include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/client/value.h"
 #include "kudu/client/write_op.h"
 #include "kudu/clock/clock.h"
@@ -49,7 +48,6 @@
 #include "kudu/common/partial_row.h"
 #include "kudu/common/schema.h"
 #include "kudu/gutil/casts.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/join.h"
@@ -105,6 +103,8 @@ namespace tablet {
 enum TestOpType {
   TEST_INSERT,
   TEST_INSERT_PK_ONLY,
+  TEST_INSERT_IGNORE,
+  TEST_INSERT_IGNORE_PK_ONLY,
   TEST_UPSERT,
   TEST_UPSERT_PK_ONLY,
   TEST_UPDATE,
@@ -125,6 +125,8 @@ const char* kTableName = "table";
 const char* TestOpType_names[] = {
   "TEST_INSERT",
   "TEST_INSERT_PK_ONLY",
+  "TEST_INSERT_IGNORE",
+  "TEST_INSERT_IGNORE_PK_ONLY",
   "TEST_UPSERT",
   "TEST_UPSERT_PK_ONLY",
   "TEST_UPDATE",
@@ -173,6 +175,8 @@ struct TestOp {
         return strings::Substitute("{$0}", TestOpType_names[type]);
       case TEST_INSERT:
       case TEST_INSERT_PK_ONLY:
+      case TEST_INSERT_IGNORE:
+      case TEST_INSERT_IGNORE_PK_ONLY:
       case TEST_UPSERT:
       case TEST_UPSERT_PK_ONLY:
       case TEST_UPDATE:
@@ -215,6 +219,8 @@ struct Redo {
 
 const vector<TestOpType> kAllOps {TEST_INSERT,
                                   TEST_INSERT_PK_ONLY,
+                                  TEST_INSERT_IGNORE,
+                                  TEST_INSERT_IGNORE_PK_ONLY,
                                   TEST_UPSERT,
                                   TEST_UPSERT_PK_ONLY,
                                   TEST_UPDATE,
@@ -230,6 +236,7 @@ const vector<TestOpType> kAllOps {TEST_INSERT,
                                   TEST_DIFF_SCAN};
 
 const vector<TestOpType> kPkOnlyOps {TEST_INSERT_PK_ONLY,
+                                     TEST_INSERT_IGNORE_PK_ONLY,
                                      TEST_UPSERT_PK_ONLY,
                                      TEST_DELETE,
                                      TEST_FLUSH_OPS,
@@ -268,7 +275,7 @@ class FuzzTest : public KuduTest {
              .default_admin_operation_timeout(MonoDelta::FromSeconds(60))
              .Build(&client_));
     // Add a table, make sure it reports itself.
-    gscoped_ptr<KuduTableCreator> table_creator(client_->NewTableCreator());
+    unique_ptr<KuduTableCreator> table_creator(client_->NewTableCreator());
     CHECK_OK(table_creator->table_name(kTableName)
              .schema(&schema_)
              .set_range_partition_columns({ "key" })
@@ -324,6 +331,8 @@ class FuzzTest : public KuduTest {
     unique_ptr<KuduWriteOperation> op;
     if (type == TEST_INSERT || type == TEST_INSERT_PK_ONLY) {
       op.reset(table_->NewInsert());
+    } else if (type == TEST_INSERT_IGNORE || type == TEST_INSERT_IGNORE_PK_ONLY) {
+      op.reset(table_->NewInsertIgnore());
     } else {
       op.reset(table_->NewUpsert());
     }
@@ -332,6 +341,7 @@ class FuzzTest : public KuduTest {
     ret.key = key;
     switch (type) {
       case TEST_INSERT:
+      case TEST_INSERT_IGNORE:
       case TEST_UPSERT: {
         if (val & 1) {
           CHECK_OK(row->SetNull(1));
@@ -339,13 +349,18 @@ class FuzzTest : public KuduTest {
           CHECK_OK(row->SetInt32(1, val));
           ret.val = val;
         }
+        if (type == TEST_INSERT_IGNORE && old_row) {
+          // insert ignore when the row already exists results in old value
+          ret.val = old_row->val;
+        }
         break;
       }
       case TEST_INSERT_PK_ONLY:
         break;
+      case TEST_INSERT_IGNORE_PK_ONLY:
       case TEST_UPSERT_PK_ONLY: {
-        // For "upsert PK only", we expect the row to keep its old value if
-        // the row existed, or NULL if there was no old row.
+        // For "upsert PK only" and "insert ignore PK only", we expect the row
+        // to keep its old value if the row existed, or NULL if there was no old row.
         ret.val = old_row ? old_row->val : boost::none;
         break;
       }
@@ -651,7 +666,7 @@ class FuzzTest : public KuduTest {
                    int update_multiplier);
 
   KuduSchema schema_;
-  gscoped_ptr<InternalMiniCluster> cluster_;
+  unique_ptr<InternalMiniCluster> cluster_;
   shared_ptr<KuduClient> client_;
   shared_ptr<KuduSession> session_;
   shared_ptr<KuduTable> table_;
@@ -688,6 +703,8 @@ bool IsMutation(const TestOpType& op) {
   switch (op) {
     case TEST_INSERT:
     case TEST_INSERT_PK_ONLY:
+    case TEST_INSERT_IGNORE:
+    case TEST_INSERT_IGNORE_PK_ONLY:
     case TEST_UPSERT:
     case TEST_UPSERT_PK_ONLY:
     case TEST_UPDATE:
@@ -727,10 +744,20 @@ void GenerateTestCase(vector<TestOp>* ops, int len, TestOpSets sets = ALL) {
         ops_pending = true;
         data_in_mrs = true;
         break;
+      case TEST_INSERT_IGNORE:
+      case TEST_INSERT_IGNORE_PK_ONLY:
+        ops->emplace_back(r, row_key);
+        ops_pending = true;
+        // If the row doesn't currently exist, this will act like an insert
+        // and put it into MRS.
+        if (!exists[row_key]) {
+          data_in_mrs = true;
+        }
+        exists[row_key] = true;
+        break;
       case TEST_UPSERT:
       case TEST_UPSERT_PK_ONLY:
         ops->emplace_back(r, row_key);
-        exists[row_key] = true;
         ops_pending = true;
         // If the row doesn't currently exist, this will act like an insert
         // and put it into MRS.
@@ -741,6 +768,7 @@ void GenerateTestCase(vector<TestOp>* ops, int len, TestOpSets sets = ALL) {
           // a DMS.
           data_in_dms = true;
         }
+        exists[row_key] = true;
         break;
       case TEST_UPDATE:
         if (!exists[row_key]) continue;
@@ -850,6 +878,8 @@ void FuzzTest::ValidateFuzzCase(const vector<TestOp>& test_ops) {
         CHECK(!exists[test_op.val]) << "invalid case: inserting already-existing row";
         exists[test_op.val] = true;
         break;
+      case TEST_INSERT_IGNORE:
+      case TEST_INSERT_IGNORE_PK_ONLY:
       case TEST_UPSERT:
       case TEST_UPSERT_PK_ONLY:
         exists[test_op.val] = true;
@@ -889,17 +919,28 @@ void FuzzTest::RunFuzzCase(const vector<TestOp>& test_ops,
     switch (test_op.type) {
       case TEST_INSERT:
       case TEST_INSERT_PK_ONLY:
+      case TEST_INSERT_IGNORE:
+      case TEST_INSERT_IGNORE_PK_ONLY:
       case TEST_UPSERT:
       case TEST_UPSERT_PK_ONLY: {
         RedoType rtype = pending_val[test_op.val] ? UPDATE : INSERT;
         pending_val[test_op.val] = InsertOrUpsertRow(
             test_op.val, i++, pending_val[test_op.val], test_op.type);
 
-        // A PK-only UPSERT that is converted into an UPDATE will be dropped
-        // server-side. We must do the same.
-        if (test_op.type != TEST_UPSERT_PK_ONLY || rtype != UPDATE) {
-          pending_redos.emplace_back(rtype, test_op.val, pending_val[test_op.val]->val);
+        // An insert ignore on a row that already exists will be dropped server-side.
+        // We must do the same.
+        if ((test_op.type == TEST_INSERT_IGNORE || test_op.type == TEST_INSERT_IGNORE_PK_ONLY) &&
+            rtype == UPDATE) {
+          break;
         }
+
+        // An "upsert PK-only" that is converted into an update will be dropped server-side.
+        // We must do the same.
+        if (test_op.type == TEST_UPSERT_PK_ONLY && rtype == UPDATE) {
+          break;
+        }
+
+        pending_redos.emplace_back(rtype, test_op.val, pending_val[test_op.val]->val);
         break;
       }
       case TEST_UPDATE:
@@ -916,7 +957,7 @@ void FuzzTest::RunFuzzCase(const vector<TestOp>& test_ops,
         FlushSessionOrDie(session_);
         cur_val = pending_val;
         int current_time = down_cast<kudu::clock::LogicalClock*>(
-            tablet()->clock().get())->GetCurrentTime();
+            tablet()->clock())->GetCurrentTime();
         VLOG(1) << "Current time: " << current_time;
         saved_values_[current_time] = cur_val;
         saved_redos_[current_time] = pending_redos;
