@@ -16,6 +16,7 @@
 // under the License.
 
 #include <cstdint> // IWYU pragma: keep
+#include <functional>
 #include <memory>
 #include <ostream> // IWYU pragma: keep
 #include <set>
@@ -29,7 +30,7 @@
 #include "kudu/client/client-test-util.h" // IWYU pragma: keep
 #include "kudu/client/client.h"
 #include "kudu/client/schema.h"
-#include "kudu/client/shared_ptr.h"
+#include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/common/common.pb.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/split.h"
