@@ -97,7 +97,7 @@ class TokenSigningPublicKeyPB;
 
 namespace tserver {
 class TsTabletManagerITest_TestTableStats_Test;
-}
+} // namespace tserver
 
 namespace tablet {
 class TabletReplica;
@@ -106,6 +106,7 @@ class TabletReplica;
 namespace master {
 
 class AuthzProvider;
+class AutoRebalancerTask;
 class CatalogManagerBgTasks;
 class HmsNotificationLogListenerTask;
 class Master;
@@ -145,8 +146,8 @@ struct PersistentTabletInfo {
 // of data are in PersistentTabletInfo above, and typically accessed using
 // TabletMetadataLock. For example:
 //
-//   TabletInfo* table = ...;
-//   TabletMetadataLock l(tablet, TableMetadataLock::READ);
+//   TabletInfo* tablet = ...;
+//   TabletMetadataLock l(tablet, TabletMetadataLock::READ);
 //   if (l.data().is_running()) { ... }
 //
 // The non-persistent information about the tablet is protected by an internal
@@ -263,6 +264,7 @@ class TableInfo : public RefCountedThreadSafe<TableInfo> {
   explicit TableInfo(std::string table_id);
 
   std::string ToString() const;
+  uint32_t schema_version() const;
 
   // Return the table's ID. Does not require synchronization.
   const std::string& id() const { return table_id_; }
@@ -292,11 +294,15 @@ class TableInfo : public RefCountedThreadSafe<TableInfo> {
   // Returns true if an "Alter" operation is in-progress
   bool IsAlterInProgress(uint32_t version) const;
 
-  void AddTask(MonitoredTask *task);
-  void RemoveTask(MonitoredTask *task);
+  void AddTask(const std::string& tablet_id, const scoped_refptr<MonitoredTask>& task);
+  void RemoveTask(const std::string& tablet_id, MonitoredTask* task);
   void AbortTasks();
   void WaitTasksCompletion();
 
+  // Returns true if pending_tasks_ contains a task whose description
+  // is the same as task_description.
+  bool ContainsTask(const std::string& tablet_id, const std::string& task_description);
+
   // Allow for showing outstanding tasks in the master UI.
   void GetTaskList(std::vector<scoped_refptr<MonitoredTask> > *tasks);
 
@@ -327,6 +333,9 @@ class TableInfo : public RefCountedThreadSafe<TableInfo> {
                      const tablet::ReportedTabletStatsPB& old_stats,
                      const tablet::ReportedTabletStatsPB& new_stats);
 
+  // Invalidate stats belonging to 'tablet_id' in the table's metrics.
+  void InvalidateMetrics(const std::string& tablet_id);
+
   // Remove stats belonging to 'tablet_id' from table metrics.
   void RemoveMetrics(const std::string& tablet_id,
                      const tablet::ReportedTabletStatsPB& old_stats);
@@ -365,8 +374,8 @@ class TableInfo : public RefCountedThreadSafe<TableInfo> {
 
   CowObject<PersistentTableInfo> metadata_;
 
-  // List of pending tasks (e.g. create/alter tablet requests)
-  std::unordered_set<MonitoredTask*> pending_tasks_;
+  // Map of tablet_id to pending tasks (e.g. create/alter tablet requests).
+  std::unordered_multimap<std::string, scoped_refptr<MonitoredTask>> pending_tasks_;
 
   // Map of schema version to the number of tablets that reported that version.
   //
@@ -752,7 +761,7 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // must be initialized before calling this method.
   consensus::RaftPeerPB::Role Role() const;
 
-  hms::HmsCatalog* HmsCatalog() const {
+  hms::HmsCatalog* hms_catalog() const {
     return hms_catalog_.get();
   }
 
@@ -760,6 +769,10 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
     return authz_provider_.get();
   }
 
+  master::AutoRebalancerTask* auto_rebalancer() const {
+    return auto_rebalancer_.get();
+  }
+
   // Returns the normalized form of the provided table name.
   //
   // If the HMS integration is configured and the table name is a valid HMS
@@ -782,6 +795,7 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // This test exclusively acquires the leader_lock_ directly.
   FRIEND_TEST(kudu::client::ServiceUnavailableRetryClientTest, CreateTable);
 
+  friend class AutoRebalancerTest;
   friend class TableLoader;
   friend class TabletLoader;
 
@@ -908,7 +922,6 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
                                              const PartitionPB& partition,
                                              const boost::optional<std::string>& dimension_label);
 
-
   // Builds the TabletLocationsPB for a tablet based on the provided TabletInfo
   // and the replica type fiter specified. Populates locs_pb and returns
   // Status::OK on success.
@@ -1101,6 +1114,8 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
 
   std::unique_ptr<master::AuthzProvider> authz_provider_;
 
+  std::unique_ptr<AutoRebalancerTask> auto_rebalancer_;
+
   enum State {
     kConstructed,
     kStarting,
