@@ -20,7 +20,7 @@
 #include <algorithm>
 #include <array>
 #include <cstdint>
-#include <iosfwd>
+#include <functional>
 #include <limits>
 #include <map>
 #include <memory>
@@ -31,7 +31,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/bind.hpp> // IWYU pragma: keep
 #include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
@@ -61,6 +60,7 @@
 #include "kudu/master/ts_manager.h"
 #include "kudu/server/monitored_task.h"
 #include "kudu/server/webui_util.h"
+#include "kudu/tablet/metadata.pb.h"
 #include "kudu/util/cow_object.h"
 #include "kudu/util/easy_json.h"
 #include "kudu/util/jsonwriter.h"
@@ -110,7 +110,7 @@ MasterPathHandlers::~MasterPathHandlers() {
 
 void MasterPathHandlers::HandleTabletServers(const Webserver::WebRequest& /*req*/,
                                              Webserver::WebResponse* resp) {
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   vector<shared_ptr<TSDescriptor>> descs;
   master_->ts_manager()->GetAllDescriptors(&descs);
 
@@ -168,7 +168,7 @@ void MasterPathHandlers::HandleTabletServers(const Webserver::WebRequest& /*req*
     desc->GetRegistration(&reg);
     ts_json["uuid"] = ts_uuid;
     if (!reg.http_addresses().empty()) {
-      string webserver = Substitute("$0://$1:$2/",
+      string webserver = Substitute("$0://$1:$2",
                                     reg.https_enabled() ? "https" : "http",
                                     reg.http_addresses(0).host(),
                                     reg.http_addresses(0).port());
@@ -233,7 +233,7 @@ int ExtractRedirectsFromRequest(const Webserver::WebRequest& req) {
 
 void MasterPathHandlers::HandleCatalogManager(const Webserver::WebRequest& req,
                                               Webserver::WebResponse* resp) {
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   CatalogManager::ScopedLeaderSharedLock l(master_->catalog_manager());
   if (!l.catalog_status().ok()) {
     (*output)["error"] = Substitute("Master is not ready: $0",  l.catalog_status().ToString());
@@ -263,16 +263,22 @@ void MasterPathHandlers::HandleCatalogManager(const Webserver::WebRequest& req,
     table_json["id"] = EscapeForHtmlToString(table->id());
     table_json["state"] = state;
     table_json["message"] = EscapeForHtmlToString(l.data().pb.state_msg());
+    table_json["tablet_count"] = HumanReadableInt::ToString(table->num_tablets());
+    const TableMetrics* table_metrics = table->GetMetrics();
+    if (table_metrics) {
+      table_json["on_disk_size"] =
+          HumanReadableNumBytes::ToString(table_metrics->on_disk_size->value());
+    }
     std::string str_create_time;
     if (l.data().pb.has_create_timestamp()) {
       str_create_time = TimestampAsString(l.data().pb.create_timestamp());
     }
-    table_json["create time"] = EscapeForHtmlToString(str_create_time);
+    table_json["create_time"] = EscapeForHtmlToString(str_create_time);
     std::string str_alter_time;
     if (l.data().pb.has_alter_timestamp()) {
       str_alter_time = TimestampAsString(l.data().pb.alter_timestamp());
     }
-    table_json["alter time"] = EscapeForHtmlToString(str_alter_time);
+    table_json["alter_time"] = EscapeForHtmlToString(str_alter_time);
   }
   (*output).Set<int64_t>("num_tables", num_running_tables);
 }
@@ -305,7 +311,7 @@ bool CompareByRole(const pair<TabletDetailPeerInfo, RaftPeerPB::Role>& a,
 
 void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
                                          Webserver::WebResponse* resp) {
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   // Parse argument.
   string table_id;
   if (!FindCopy(req.parsed_args, "id", &table_id)) {
@@ -475,6 +481,8 @@ void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
     Capitalize(&state);
     tablet_detail_json["state"] = state;
     tablet_detail_json["state_msg"] = l.data().pb.state_msg();
+    tablet_detail_json["on_disk_size"] =
+        HumanReadableNumBytes::ToString(tablet->GetStats().on_disk_size());
     EasyJson peers_json = tablet_detail_json.Set("peers", EasyJson::kArray);
     for (const auto& e : sorted_replicas) {
       EasyJson peer_json = peers_json.PushBack(EasyJson::kObject);
@@ -489,10 +497,14 @@ void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
 
   const TableMetrics* table_metrics = table->GetMetrics();
   if (table_metrics) {
-    // If the table doesn't support live row counts, the value will be negative.
-    // But the value of disk size will never be negative.
-    (*output)["table_disk_size"] =
-        HumanReadableNumBytes::ToString(table_metrics->on_disk_size->value());
+    // If the table doesn't support 'on disk size' and 'live row count',
+    // we need to show their values as "N/A".
+    if (table_metrics->TableSupportsOnDiskSize()) {
+      (*output)["table_disk_size"] =
+          HumanReadableNumBytes::ToString(table_metrics->on_disk_size->value());
+    } else {
+      (*output)["table_disk_size"] = "N/A";
+    }
     if (table_metrics->TableSupportsLiveRowCount()) {
       (*output)["table_live_row_count"] = table_metrics->live_row_count->value();
     } else {
@@ -540,7 +552,7 @@ void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
 
 void MasterPathHandlers::HandleMasters(const Webserver::WebRequest& /*req*/,
                                        Webserver::WebResponse* resp) {
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   vector<ServerEntryPB> masters;
   Status s = master_->ListMasters(&masters);
   if (!s.ok()) {
@@ -567,7 +579,7 @@ void MasterPathHandlers::HandleMasters(const Webserver::WebRequest& /*req*/,
     ServerRegistrationPB reg = master.registration();
     master_json["uuid"] = master.instance_id().permanent_uuid();
     if (!reg.http_addresses().empty()) {
-      master_json["target"] = Substitute("$0://$1:$2/",
+      master_json["target"] = Substitute("$0://$1:$2",
                                          reg.https_enabled() ? "https" : "http",
                                          reg.http_addresses(0).host(),
                                          reg.http_addresses(0).port());
@@ -686,7 +698,7 @@ void JsonError(const Status& s, ostringstream* out) {
 
 void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/,
                                             Webserver::PrerenderedWebResponse* resp) {
-  ostringstream* output = resp->output;
+  ostringstream* output = &resp->output;
   Status s = master_->catalog_manager()->CheckOnline();
   if (!s.ok()) {
     JsonError(s, output);
@@ -768,23 +780,33 @@ Status MasterPathHandlers::Register(Webserver* server) {
   bool is_on_nav_bar = true;
   server->RegisterPathHandler(
       "/tablet-servers", "Tablet Servers",
-      boost::bind(&MasterPathHandlers::HandleTabletServers, this, _1, _2),
+      [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+        this->HandleTabletServers(req, resp);
+      },
       is_styled, is_on_nav_bar);
   server->RegisterPathHandler(
       "/tables", "Tables",
-      boost::bind(&MasterPathHandlers::HandleCatalogManager, this, _1, _2),
+      [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+        this->HandleCatalogManager(req, resp);
+      },
       is_styled, is_on_nav_bar);
   server->RegisterPathHandler(
       "/table", "",
-      boost::bind(&MasterPathHandlers::HandleTablePage, this, _1, _2),
+      [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+        this->HandleTablePage(req, resp);
+      },
       is_styled, false);
   server->RegisterPathHandler(
       "/masters", "Masters",
-      boost::bind(&MasterPathHandlers::HandleMasters, this, _1, _2),
+      [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+        this->HandleMasters(req, resp);
+      },
       is_styled, is_on_nav_bar);
   server->RegisterPrerenderedPathHandler(
       "/dump-entities", "Dump Entities",
-      boost::bind(&MasterPathHandlers::HandleDumpEntities, this, _1, _2),
+      [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
+        this->HandleDumpEntities(req, resp);
+      },
       false, false);
   return Status::OK();
 }
