@@ -23,6 +23,7 @@
 #include <iterator>
 #include <limits>
 #include <map>
+#include <random>
 #include <set>
 #include <string>
 #include <unordered_map>
@@ -64,6 +65,7 @@ Rebalancer::Config::Config(
     size_t max_moves_per_server,
     size_t max_staleness_interval_sec,
     int64_t max_run_time_sec,
+    bool move_replicas_from_ignored_tservers,
     bool move_rf1_replicas,
     bool output_replica_distribution_details,
     bool run_policy_fixer,
@@ -76,6 +78,7 @@ Rebalancer::Config::Config(
       max_moves_per_server(max_moves_per_server),
       max_staleness_interval_sec(max_staleness_interval_sec),
       max_run_time_sec(max_run_time_sec),
+      move_replicas_from_ignored_tservers(move_replicas_from_ignored_tservers),
       move_rf1_replicas(move_rf1_replicas),
       output_replica_distribution_details(output_replica_distribution_details),
       run_policy_fixer(run_policy_fixer),
@@ -101,9 +104,9 @@ Rebalancer::Rebalancer(Config config)
 // replicas of affected tablets would make the client to re-resolve new leaders
 // and retry the operations. Moving leader replicas is used as last resort
 // when no other candidates are left.
-Status Rebalancer::FindReplicas(const TableReplicaMove& move,
-                                const ClusterRawInfo& raw_info,
-                                vector<string>* tablet_ids) {
+void Rebalancer::FindReplicas(const TableReplicaMove& move,
+                              const ClusterRawInfo& raw_info,
+                              vector<string>* tablet_ids) {
   const auto& table_id = move.table_id;
 
   // Tablet ids of replicas on the source tserver that are non-leaders.
@@ -162,7 +165,7 @@ Status Rebalancer::FindReplicas(const TableReplicaMove& move,
     // If there are tablets with non-leader replicas at the source server,
     // those are the best candidates for movement.
     tablet_ids->swap(tablet_uuids);
-    return Status::OK();
+    return;
   }
 
   // If no tablets with non-leader replicas were found, resort to tablets with
@@ -175,8 +178,6 @@ Status Rebalancer::FindReplicas(const TableReplicaMove& move,
       inserter(tablet_uuids, tablet_uuids.begin()));
 
   tablet_ids->swap(tablet_uuids);
-
-  return Status::OK();
 }
 
 void Rebalancer::FilterMoves(const MovesInProgress& scheduled_moves,
@@ -282,6 +283,7 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
 
   unordered_map<string, int32_t> tserver_replicas_count;
   unordered_map<string, TableReplicasAtServer> table_replicas_info;
+  unordered_set<string> unhealthy_tablet_servers;
 
   // Build a set of tables with RF=1 (single replica tables).
   unordered_set<string> rf1_tables;
@@ -311,6 +313,7 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
                               "non-HEALTHY status ($2)",
                               s.uuid, s.address,
                               ServerHealthToString(s.health));
+      unhealthy_tablet_servers.emplace(s.uuid);
       continue;
     }
     tserver_replicas_count.emplace(s.uuid, 0);
@@ -318,7 +321,7 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
 
   for (const auto& tablet : raw_info.tablet_summaries) {
     if (!config_.move_rf1_replicas) {
-      if (rf1_tables.find(tablet.table_id) != rf1_tables.end()) {
+      if (ContainsKey(rf1_tables, tablet.table_id)) {
         LOG(INFO) << Substitute("tablet $0 of table '$1' ($2) has single replica, skipping",
                                 tablet.id, tablet.table_name, tablet.table_id);
         continue;
@@ -409,6 +412,10 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
   // Populate ClusterBalanceInfo::servers_by_total_replica_count
   auto& servers_by_count = result_info.balance.servers_by_total_replica_count;
   for (const auto& elem : tserver_replicas_count) {
+    if (ContainsKey(config_.ignored_tservers, elem.first)) {
+      VLOG(1) << Substitute("ignoring tserver $0", elem.first);
+      continue;
+    }
     servers_by_count.emplace(elem.second, elem.first);
   }
 
@@ -423,18 +430,105 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
     for (const auto& e : elem.second) {
       const auto& ts_uuid = e.first;
       const auto replica_count = e.second;
+      if (ContainsKey(config_.ignored_tservers, ts_uuid)) {
+        VLOG(1) << Substitute("ignoring replicas of table $0 on tserver $1", table_id, ts_uuid);
+        continue;
+      }
       tbi.servers_by_replica_count.emplace(replica_count, ts_uuid);
       max_count = std::max(replica_count, max_count);
       min_count = std::min(replica_count, min_count);
     }
     table_info_by_skew.emplace(max_count - min_count, std::move(tbi));
   }
+
+  // Populate ClusterInfo::tservers_to_empty
+  if (config_.move_replicas_from_ignored_tservers) {
+    auto& tservers_to_empty = result_info.tservers_to_empty;
+    for (const auto& ignored_tserver : config_.ignored_tservers) {
+      if (ContainsKey(unhealthy_tablet_servers, ignored_tserver)) {
+        continue;
+      }
+      const int* replica_count = FindOrNull(tserver_replicas_count, ignored_tserver);
+      if (!replica_count) {
+        return Status::InvalidArgument(Substitute(
+            "ignored tserver $0 is not reported among known tservers", ignored_tserver));
+      }
+      tservers_to_empty.emplace(ignored_tserver, *replica_count);
+    }
+  }
+
   // TODO(aserbin): add sanity checks on the result.
   *info = std::move(result_info);
 
   return Status::OK();
 }
 
+void BuildTabletExtraInfoMap(
+    const ClusterRawInfo& raw_info,
+    std::unordered_map<std::string, TabletExtraInfo>* extra_info_by_tablet_id) {
+  unordered_map<string, int> replication_factors_by_table;
+  for (const auto& s : raw_info.table_summaries) {
+    EmplaceOrDie(&replication_factors_by_table, s.id, s.replication_factor);
+  }
+  for (const auto& s : raw_info.tablet_summaries) {
+    int num_voters = 0;
+    for (const auto& rs : s.replicas) {
+      if (rs.is_voter) {
+        ++num_voters;
+      }
+    }
+    const auto rf = FindOrDie(replication_factors_by_table, s.table_id);
+    EmplaceOrDie(extra_info_by_tablet_id,
+                 s.id, TabletExtraInfo{rf, num_voters});
+  }
+}
+
+Status SelectReplicaToMove(
+    const TableReplicaMove& move,
+    const unordered_map<string, TabletExtraInfo>& extra_info_by_tablet_id,
+    std::mt19937* random_generator,
+    vector<string> tablet_ids,
+    unordered_set<string>* tablets_in_move,
+    vector<Rebalancer::ReplicaMove>* replica_moves) {
+
+  // Shuffle the set of the tablet identifiers: that's to achieve even spread
+  // of moves across tables with the same skew.
+  std::shuffle(tablet_ids.begin(), tablet_ids.end(), *random_generator);
+  string move_tablet_id;
+  for (const auto& tablet_id : tablet_ids) {
+    if (!ContainsKey(*tablets_in_move, tablet_id)) {
+      // For now, choose the very first tablet that does not have replicas
+      // in move. Later on, additional logic might be added to find
+      // the best candidate.
+      move_tablet_id = tablet_id;
+      break;
+    }
+  }
+  if (move_tablet_id.empty()) {
+    return Status::NotFound(Substitute(
+        "table $0: could not find any suitable replica to move "
+        "from server $1 to server $2", move.table_id, move.from, move.to));
+  }
+  Rebalancer::ReplicaMove move_info;
+  move_info.tablet_uuid = move_tablet_id;
+  move_info.ts_uuid_from = move.from;
+  const auto& extra_info = FindOrDie(extra_info_by_tablet_id, move_tablet_id);
+  if (extra_info.replication_factor < extra_info.num_voters) {
+    // The number of voter replicas is greater than the target replication
+    // factor. It might happen the replica distribution would be better
+    // if just removing the source replica. Anyway, once a replica is removed,
+    // the system will automatically add a new one, if needed, where the new
+    // replica will be placed to have balanced replica distribution.
+    move_info.ts_uuid_to = "";
+  } else {
+    move_info.ts_uuid_to = move.to;
+  }
+  replica_moves->emplace_back(std::move(move_info));
+  // Mark the tablet as 'has a replica in move'.
+  tablets_in_move->emplace(std::move(move_tablet_id));
+
+  return Status::OK();
+}
 
 } // namespace rebalance
 } // namespace kudu
