@@ -20,7 +20,6 @@
 #include <memory>
 #include <string>
 
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/rpc/messenger.h"
@@ -32,6 +31,7 @@
 namespace kudu {
 
 class DnsResolver;
+class FileCache;
 class FsManager;
 class MemTracker;
 class MetricEntity;
@@ -97,14 +97,15 @@ class ServerBase {
 
   const scoped_refptr<MetricEntity>& metric_entity() const { return metric_entity_; }
 
-  MetricRegistry* metric_registry() { return metric_registry_.get(); }
+  MetricRegistry* metric_registry() const { return metric_registry_.get(); }
 
   const scoped_refptr<rpc::ResultTracker>& result_tracker() const { return result_tracker_; }
 
-  // Returns this server's clock.
-  clock::Clock* clock() { return clock_.get(); }
+  clock::Clock* clock() const { return clock_.get(); }
 
-  DnsResolver* dns_resolver() { return dns_resolver_.get(); }
+  DnsResolver* dns_resolver() const { return dns_resolver_.get(); }
+
+  FileCache* file_cache() const { return file_cache_.get(); }
 
   // Return a PB describing the status of the server (version info, bound ports, etc)
   Status GetStatusPB(ServerStatusPB* status) const;
@@ -140,11 +141,13 @@ class ServerBase {
   virtual Status Start();
 
   // Shuts down the server.
-  virtual void Shutdown();
+  virtual void Shutdown() {
+    ShutdownImpl();
+  }
 
   // Registers a new RPC service. Once Start() is called, the server will
   // process and dispatch incoming RPCs belonging to this service.
-  Status RegisterService(gscoped_ptr<rpc::ServiceIf> rpc_impl);
+  Status RegisterService(std::unique_ptr<rpc::ServiceIf> rpc_impl);
 
   // Unregisters all RPC services. After this function returns, any subsequent
   // incoming RPCs will be rejected.
@@ -175,20 +178,21 @@ class ServerBase {
 
   std::unique_ptr<MinidumpExceptionHandler> minidump_handler_;
   std::shared_ptr<MemTracker> mem_tracker_;
-  gscoped_ptr<MetricRegistry> metric_registry_;
+  std::unique_ptr<MetricRegistry> metric_registry_;
   scoped_refptr<MetricEntity> metric_entity_;
-  gscoped_ptr<FsManager> fs_manager_;
-  gscoped_ptr<RpcServer> rpc_server_;
-  gscoped_ptr<Webserver> web_server_;
+  std::unique_ptr<FileCache> file_cache_;
+  std::unique_ptr<FsManager> fs_manager_;
+  std::unique_ptr<RpcServer> rpc_server_;
+  std::unique_ptr<Webserver> web_server_;
 
   std::shared_ptr<rpc::Messenger> messenger_;
   scoped_refptr<rpc::ResultTracker> result_tracker_;
   bool is_first_run_;
 
-  scoped_refptr<clock::Clock> clock_;
+  std::unique_ptr<clock::Clock> clock_;
 
   // The instance identifier of this server.
-  gscoped_ptr<NodeInstancePB> instance_pb_;
+  std::unique_ptr<NodeInstancePB> instance_pb_;
 
   // The ACL of users who are allowed to act as superusers.
   security::SimpleAcl superuser_acl_;
@@ -215,6 +219,11 @@ class ServerBase {
   Status StartExcessLogFileDeleterThread();
   void ExcessLogFileDeleterThread();
 
+  // A method for internal use in the destructor. Some static code analyzers
+  // issue a warning if calling a virtual function from destructor even if it's
+  // safe in a particular case.
+  void ShutdownImpl();
+
 #ifdef TCMALLOC_ENABLED
   // Start thread to GC tcmalloc allocated memory.
   Status StartTcmallocMemoryGcThread();
@@ -233,7 +242,7 @@ class ServerBase {
 #endif
   CountDownLatch stop_background_threads_latch_;
 
-  gscoped_ptr<ScopedGLogMetrics> glog_metrics_;
+  std::unique_ptr<ScopedGLogMetrics> glog_metrics_;
 
   DISALLOW_COPY_AND_ASSIGN(ServerBase);
 };
