@@ -26,11 +26,9 @@
 #include <ostream>
 #include <string>
 #include <thread>
-#include <utility>
 #include <vector>
 
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
@@ -51,7 +49,6 @@
 #include "kudu/fs/fs_manager.h"
 #include "kudu/fs/log_block_manager.h"
 #include "kudu/gutil/casts.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/diskrowset.h"
@@ -110,12 +107,12 @@ static const size_t kSmallRollThreshold = 1024; // 1KB
 class TestCompaction : public KuduRowSetTest {
  public:
   TestCompaction()
-    : KuduRowSetTest(CreateSchema()),
-      op_id_(consensus::MaximumOpId()),
-      row_builder_(&schema_),
-      arena_(32*1024),
-      clock_(clock::LogicalClock::CreateStartingAt(Timestamp::kInitialTimestamp)),
-      log_anchor_registry_(new log::LogAnchorRegistry()) {
+      : KuduRowSetTest(CreateSchema()),
+        op_id_(consensus::MaximumOpId()),
+        row_builder_(&schema_),
+        arena_(32*1024),
+        clock_(Timestamp::kInitialTimestamp),
+        log_anchor_registry_(new log::LogAnchorRegistry()) {
   }
 
   static Schema CreateSchema() {
@@ -138,7 +135,7 @@ class TestCompaction : public KuduRowSetTest {
   // The 'nullable_val' column is set to either NULL (when val is odd)
   // or 'val' (when val is even).
   void InsertRow(MemRowSet *mrs, int row_key, int32_t val) {
-    ScopedTransaction tx(&mvcc_, clock_->Now());
+    ScopedTransaction tx(&mvcc_, clock_.Now());
     tx.StartApplying();
     InsertRowInTransaction(mrs, tx, row_key, val);
     tx.Commit();
@@ -189,7 +186,7 @@ class TestCompaction : public KuduRowSetTest {
   }
 
   void UpdateRow(RowSet *rowset, int row_key, int32_t new_val) {
-    ScopedTransaction tx(&mvcc_, clock_->Now());
+    ScopedTransaction tx(&mvcc_, clock_.Now());
     tx.StartApplying();
     UpdateRowInTransaction(rowset, tx, row_key, new_val);
     tx.Commit();
@@ -244,7 +241,7 @@ class TestCompaction : public KuduRowSetTest {
   }
 
   void DeleteRow(RowSet* rowset, int row_key) {
-    ScopedTransaction tx(&mvcc_, clock_->Now());
+    ScopedTransaction tx(&mvcc_, clock_.Now());
     tx.StartApplying();
     DeleteRowInTransaction(rowset, tx, row_key);
     tx.Commit();
@@ -311,13 +308,13 @@ class TestCompaction : public KuduRowSetTest {
     }
   }
 
-  Status BuildCompactionInput(const MvccSnapshot& merge_snap,
-                              const vector<shared_ptr<DiskRowSet> >& rowsets,
-                              const Schema& projection,
-                              gscoped_ptr<CompactionInput>* out) {
+  static Status BuildCompactionInput(const MvccSnapshot& merge_snap,
+                                     const vector<shared_ptr<DiskRowSet> >& rowsets,
+                                     const Schema& projection,
+                                     unique_ptr<CompactionInput>* out) {
     vector<shared_ptr<CompactionInput> > merge_inputs;
     for (const shared_ptr<DiskRowSet> &rs : rowsets) {
-      gscoped_ptr<CompactionInput> input;
+      unique_ptr<CompactionInput> input;
       RETURN_NOT_OK(CompactionInput::Create(*rs, &projection, merge_snap, nullptr, &input));
       merge_inputs.push_back(shared_ptr<CompactionInput>(input.release()));
     }
@@ -332,7 +329,7 @@ class TestCompaction : public KuduRowSetTest {
                           const Schema& projection, int64_t roll_threshold,
                           vector<shared_ptr<DiskRowSet> >* result_rowsets) {
     MvccSnapshot merge_snap(mvcc_);
-    gscoped_ptr<CompactionInput> compact_input;
+    unique_ptr<CompactionInput> compact_input;
     RETURN_NOT_OK(BuildCompactionInput(merge_snap, rowsets, projection, &compact_input));
     DoFlushAndReopen(compact_input.get(), projection, merge_snap, roll_threshold,
                      result_rowsets);
@@ -357,7 +354,7 @@ class TestCompaction : public KuduRowSetTest {
                          vector<shared_ptr<DiskRowSet> >* result_rowsets) {
     MvccSnapshot snap(mvcc_);
     vector<shared_ptr<RowSetMetadata> > rowset_metas;
-    gscoped_ptr<CompactionInput> input(CompactionInput::Create(mrs, &projection, snap));
+    unique_ptr<CompactionInput> input(CompactionInput::Create(mrs, &projection, snap));
     DoFlushAndReopen(input.get(), projection, snap, roll_threshold, result_rowsets);
   }
 
@@ -443,7 +440,7 @@ class TestCompaction : public KuduRowSetTest {
       }
     } else {
       string tablet_id = "KuduCompactionBenchTablet";
-      FsManager fs_manager(env_, FLAGS_merge_benchmark_input_dir);
+      FsManager fs_manager(env_, FsManagerOpts(FLAGS_merge_benchmark_input_dir));
       scoped_refptr<TabletMetadata> input_meta;
       ASSERT_OK(TabletMetadata::Load(&fs_manager, tablet_id, &input_meta));
 
@@ -460,7 +457,7 @@ class TestCompaction : public KuduRowSetTest {
     LOG_TIMING(INFO, "compacting " +
                std::string((OVERLAP_INPUTS ? "with overlap" : "without overlap"))) {
       MvccSnapshot merge_snap(mvcc_);
-      gscoped_ptr<CompactionInput> compact_input;
+      unique_ptr<CompactionInput> compact_input;
       ASSERT_OK(BuildCompactionInput(merge_snap, rowsets, schema_, &compact_input));
       // Use a low target row size to increase the number of resulting rowsets.
       RollingDiskRowSetWriter rdrsw(tablet()->metadata(), schema_,
@@ -485,7 +482,7 @@ class TestCompaction : public KuduRowSetTest {
   RowBuilder row_builder_;
   char key_buf_[256];
   Arena arena_;
-  scoped_refptr<clock::LogicalClock> clock_;
+  clock::LogicalClock clock_;
   MvccManager mvcc_;
 
   scoped_refptr<LogAnchorRegistry> log_anchor_registry_;
@@ -506,7 +503,7 @@ TEST_F(TestCompaction, TestMemRowSetInput) {
   // and mutations.
   vector<string> out;
   MvccSnapshot snap(mvcc_);
-  gscoped_ptr<CompactionInput> input(CompactionInput::Create(*mrs, &schema_, snap));
+  unique_ptr<CompactionInput> input(CompactionInput::Create(*mrs, &schema_, snap));
   IterateInput(input.get(), &out);
   ASSERT_EQ(10, out.size());
   EXPECT_EQ(R"(RowIdxInBlock: 0; Base: (string key="hello 00000000", int32 val=0, )"
@@ -570,7 +567,7 @@ TEST_F(TestCompaction, TestRowSetInput) {
 
   // Check compaction input
   vector<string> out;
-  gscoped_ptr<CompactionInput> input;
+  unique_ptr<CompactionInput> input;
   ASSERT_OK(CompactionInput::Create(*rs, &schema_, MvccSnapshot(mvcc_), nullptr, &input));
   IterateInput(input.get(), &out);
   ASSERT_EQ(10, out.size());
@@ -639,7 +636,7 @@ TEST_F(TestCompaction, TestDuplicatedGhostRowsMerging) {
   // Now compact all the drs and make sure we don't get duplicated keys on the output
   CompactAndReopenNoRoll(all_rss, schema_, &result);
 
-  gscoped_ptr<CompactionInput> input;
+  unique_ptr<CompactionInput> input;
   ASSERT_OK(CompactionInput::Create(*result,
                                     &schema_,
                                     MvccSnapshot::CreateSnapshotIncludingAllTransactions(),
@@ -664,7 +661,7 @@ void TestCompaction::AddExpectedDelete(Mutation** current_head, Timestamp ts) {
   faststring buf;
   RowChangeListEncoder enc(&buf);
   enc.SetToDelete();
-  if (ts == Timestamp::kInvalidTimestamp) ts = Timestamp(clock_->GetCurrentTime());
+  if (ts == Timestamp::kInvalidTimestamp) ts = Timestamp(clock_.GetCurrentTime());
   Mutation* mutation = Mutation::CreateInArena(&arena_,
                                                ts,
                                                enc.as_changelist());
@@ -683,7 +680,7 @@ void TestCompaction::AddExpectedUpdate(Mutation** current_head, int32_t val) {
     enc.AddColumnUpdate(schema_.column(2), schema_.column_id(2), nullptr);
   }
   Mutation* mutation = Mutation::CreateInArena(&arena_,
-                                               Timestamp(clock_->GetCurrentTime()),
+                                               Timestamp(clock_.GetCurrentTime()),
                                                enc.as_changelist());
   mutation->set_next(*current_head);
   *current_head = mutation;
@@ -699,7 +696,8 @@ void TestCompaction::AddExpectedReinsert(Mutation** current_head, int32_t val) {
   } else {
     enc.EncodeColumnMutation(schema_.column(2), schema_.column_id(2), nullptr);
   }
-  Mutation* mutation = Mutation::CreateInArena(&arena_, Timestamp(clock_->GetCurrentTime()),
+  Mutation* mutation = Mutation::CreateInArena(&arena_,
+                                               Timestamp(clock_.GetCurrentTime()),
                                                enc.as_changelist());
   mutation->set_next(*current_head);
   *current_head = mutation;
@@ -803,7 +801,7 @@ TEST_F(TestCompaction, TestDuplicatedRowsRandomCompaction) {
 
   vector<shared_ptr<CompactionInput>> inputs;
   for (auto& row_set : row_sets) {
-    gscoped_ptr<CompactionInput> ci;
+    unique_ptr<CompactionInput> ci;
     CHECK_OK(row_set->NewCompactionInput(&schema_, all_snap, nullptr, &ci));
     inputs.push_back(shared_ptr<CompactionInput>(ci.release()));
   }
@@ -824,7 +822,7 @@ TEST_F(TestCompaction, TestDuplicatedRowsRandomCompaction) {
   }
 
   vector<string> out;
-  gscoped_ptr<CompactionInput> ci;
+  unique_ptr<CompactionInput> ci;
   CHECK_OK(row_sets[0]->NewCompactionInput(&schema_, all_snap, nullptr, &ci));
   IterateInput(ci.get(), &out);
 
@@ -858,7 +856,7 @@ TEST_F(TestCompaction, TestMRSCompactionDoesntOutputUnobservableRows) {
     shared_ptr<MemRowSet> mrs;
     ASSERT_OK(MemRowSet::Create(1, schema_, log_anchor_registry_.get(),
                                 mem_trackers_.tablet_tracker, &mrs));
-    ScopedTransaction tx(&mvcc_, clock_->Now());
+    ScopedTransaction tx(&mvcc_, clock_.Now());
     tx.StartApplying();
     DeleteRowInTransaction(rs1.get(), tx, 1);
     InsertRowInTransaction(mrs.get(), tx, 1, 2);
@@ -873,17 +871,17 @@ TEST_F(TestCompaction, TestMRSCompactionDoesntOutputUnobservableRows) {
 
   MvccSnapshot all_snap = MvccSnapshot::CreateSnapshotIncludingAllTransactions();
 
-  gscoped_ptr<CompactionInput> rs1_input;
+  unique_ptr<CompactionInput> rs1_input;
   ASSERT_OK(CompactionInput::Create(*rs1, &schema_, all_snap, nullptr, &rs1_input));
 
-  gscoped_ptr<CompactionInput> rs2_input;
+  unique_ptr<CompactionInput> rs2_input;
   ASSERT_OK(CompactionInput::Create(*rs2, &schema_, all_snap, nullptr, &rs2_input));
 
   vector<shared_ptr<CompactionInput>> to_merge;
   to_merge.push_back(shared_ptr<CompactionInput>(rs1_input.release()));
   to_merge.push_back(shared_ptr<CompactionInput>(rs2_input.release()));
 
-  gscoped_ptr<CompactionInput> merged(CompactionInput::Merge(to_merge, &schema_));
+  unique_ptr<CompactionInput> merged(CompactionInput::Merge(to_merge, &schema_));
 
   // Make sure the unobservable version of the row that was inserted and deleted in the MRS
   // in the same transaction doesn't show up in the compaction input.
@@ -919,7 +917,7 @@ TEST_F(TestCompaction, TestOneToOne) {
 
   // Catch the updates that came in after the snapshot flush was made.
   MvccSnapshot snap2(mvcc_);
-  gscoped_ptr<CompactionInput> input(CompactionInput::Create(*mrs, &schema_, snap2));
+  unique_ptr<CompactionInput> input(CompactionInput::Create(*mrs, &schema_, snap2));
 
   // Add some more updates which come into the new rowset while the "reupdate" is happening.
   UpdateRows(rs.get(), 1000, 0, 3);
@@ -941,7 +939,7 @@ TEST_F(TestCompaction, TestOneToOne) {
 
   // And compact (1 input to 1 output)
   MvccSnapshot snap3(mvcc_);
-  gscoped_ptr<CompactionInput> compact_input;
+  unique_ptr<CompactionInput> compact_input;
   ASSERT_OK(CompactionInput::Create(*rs, &schema_, snap3, nullptr, &compact_input));
   DoFlushAndReopen(compact_input.get(), schema_, snap3, kLargeRollThreshold, nullptr);
 }
@@ -979,7 +977,7 @@ TEST_F(TestCompaction, TestKUDU102) {
         shared_ptr<CompactionInput>(CompactionInput::Create(*mrs, &schema_, snap2)));
   merge_inputs.push_back(
         shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_b, &schema_, snap2)));
-  gscoped_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
 
   string dummy_name = "";
 
@@ -1036,7 +1034,7 @@ TEST_F(TestCompaction, TestMergeMRS) {
         shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_a, &schema_, snap)));
   merge_inputs.push_back(
         shared_ptr<CompactionInput>(CompactionInput::Create(*mrs_b, &schema_, snap)));
-  gscoped_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
+  unique_ptr<CompactionInput> input(CompactionInput::Merge(merge_inputs, &schema_));
 
   vector<string> out;
   IterateInput(input.get(), &out);
