@@ -623,7 +623,7 @@ Status WriteDeltaIteratorToFile(DeltaIterator* iter,
   RETURN_NOT_OK(iter->SeekToOrdinal(0));
 
   const size_t kRowsPerBlock = 100;
-  DeltaStats stats;
+  std::unique_ptr<DeltaStats> stats(new DeltaStats);
   Arena arena(32 * 1024);
   for (size_t i = 0; iter->HasNext(); ) {
     size_t n;
@@ -646,12 +646,12 @@ Status WriteDeltaIteratorToFile(DeltaIterator* iter,
     for (const DeltaKeyAndUpdate& cell : cells) {
       RowChangeList rcl(cell.cell);
       RETURN_NOT_OK(out->AppendDelta<Type>(cell.key, rcl));
-      RETURN_NOT_OK(stats.UpdateStats(cell.key.timestamp(), rcl));
+      RETURN_NOT_OK(stats->UpdateStats(cell.key.timestamp(), rcl));
     }
 
     i += n;
   }
-  out->WriteDeltaStats(stats);
+  out->WriteDeltaStats(std::move(stats));
   return Status::OK();
 }
 
