@@ -14,14 +14,15 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
-#ifndef KUDU_TABLET_DELTAFILE_H
-#define KUDU_TABLET_DELTAFILE_H
+#pragma once
 
 #include <cstddef>
 #include <cstdint>
 #include <deque>
 #include <memory>
+#include <mutex>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <boost/optional/optional.hpp>
@@ -32,14 +33,13 @@
 #include "kudu/cfile/cfile_reader.h"
 #include "kudu/cfile/cfile_writer.h"
 #include "kudu/common/rowid.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/macros.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/delta_key.h"
 #include "kudu/tablet/delta_stats.h"
 #include "kudu/tablet/delta_store.h"
 #include "kudu/util/faststring.h"
+#include "kudu/util/locks.h"
 #include "kudu/util/once.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
@@ -102,7 +102,11 @@ class DeltaFileWriter {
   template<DeltaType Type>
   Status AppendDelta(const DeltaKey &key, const RowChangeList &delta);
 
-  void WriteDeltaStats(const DeltaStats& stats);
+  void WriteDeltaStats(std::unique_ptr<DeltaStats> stats);
+
+  std::unique_ptr<DeltaStats> release_delta_stats() {
+    return std::move(delta_stats_);
+  }
 
   size_t written_size() const {
     return writer_->written_size();
@@ -111,6 +115,7 @@ class DeltaFileWriter {
  private:
   Status DoAppendDelta(const DeltaKey &key, const RowChangeList &delta);
 
+  std::unique_ptr<DeltaStats> delta_stats_;
   std::unique_ptr<cfile::CFileWriter> writer_;
 
   // Buffer used as a temporary for storing the serialized form
@@ -149,33 +154,40 @@ class DeltaFileReader : public DeltaStore,
   static Status OpenNoInit(std::unique_ptr<fs::ReadableBlock> block,
                            DeltaType delta_type,
                            cfile::ReaderOptions options,
+                           std::unique_ptr<DeltaStats> delta_stats,
                            std::shared_ptr<DeltaFileReader>* reader_out);
 
-  virtual Status Init(const fs::IOContext* io_context) OVERRIDE;
+  Status Init(const fs::IOContext* io_context) override;
 
-  virtual bool Initted() OVERRIDE {
+  bool Initted() const override {
     return init_once_.init_succeeded();
   }
 
   // See DeltaStore::NewDeltaIterator(...)
   Status NewDeltaIterator(const RowIteratorOptions& opts,
-                          std::unique_ptr<DeltaIterator>* iterator) const OVERRIDE;
+                          std::unique_ptr<DeltaIterator>* iterator) const override;
 
   // See DeltaStore::CheckRowDeleted
-  virtual Status CheckRowDeleted(rowid_t row_idx,
-                                 const fs::IOContext* io_context,
-                                 bool *deleted) const OVERRIDE;
+  Status CheckRowDeleted(rowid_t row_idx,
+                         const fs::IOContext* io_context,
+                         bool *deleted) const override;
 
-  virtual uint64_t EstimateSize() const OVERRIDE;
+  uint64_t EstimateSize() const override;
 
   const BlockId& block_id() const { return reader_->block_id(); }
 
-  virtual const DeltaStats& delta_stats() const OVERRIDE {
-    DCHECK(init_once_.init_succeeded());
+  const DeltaStats& delta_stats() const override {
+    std::lock_guard<simple_spinlock> l(stats_lock_);
+    DCHECK(delta_stats_);
     return *delta_stats_;
   }
 
-  virtual std::string ToString() const OVERRIDE {
+  bool has_delta_stats() const override {
+    std::lock_guard<simple_spinlock> l(stats_lock_);
+    return delta_stats_ != nullptr;
+  }
+
+  std::string ToString() const override {
     if (!init_once_.init_succeeded()) return reader_->ToString();
     return strings::Substitute("$0 ($1)", reader_->ToString(), delta_stats_->ToString());
   }
@@ -203,6 +215,7 @@ class DeltaFileReader : public DeltaStore,
   }
 
   DeltaFileReader(std::unique_ptr<cfile::CFileReader> cf_reader,
+                  std::unique_ptr<DeltaStats> delta_stats,
                   DeltaType delta_type);
 
   // Callback used in 'init_once_' to initialize this delta file.
@@ -211,7 +224,11 @@ class DeltaFileReader : public DeltaStore,
   Status ReadDeltaStats();
 
   std::shared_ptr<cfile::CFileReader> reader_;
-  gscoped_ptr<DeltaStats> delta_stats_;
+
+  // TODO(awong): it'd be nice to not heap-allocate this and other usages of
+  // delta stats.
+  mutable simple_spinlock stats_lock_;
+  std::unique_ptr<DeltaStats> delta_stats_;
 
   // The type of this delta, i.e. UNDO or REDO.
   const DeltaType delta_type_;
@@ -269,7 +286,7 @@ class DeltaFileIterator : public DeltaIterator {
 
     // The block decoder, to avoid having to re-parse the block header
     // on every ApplyUpdates() call
-    gscoped_ptr<cfile::BinaryPlainBlockDecoder> decoder_;
+    std::unique_ptr<cfile::BinaryPlainBlockDecoder> decoder_;
 
     // The first row index for which there is an update in this delta block.
     rowid_t first_updated_idx_;
@@ -318,7 +335,7 @@ class DeltaFileIterator : public DeltaIterator {
 
   DeltaPreparer<DeltaFilePreparerTraits<Type>> preparer_;
 
-  gscoped_ptr<cfile::IndexTreeIterator> index_iter_;
+  std::unique_ptr<cfile::IndexTreeIterator> index_iter_;
 
   bool prepared_;
   bool exhausted_;
@@ -337,5 +354,3 @@ class DeltaFileIterator : public DeltaIterator {
 
 } // namespace tablet
 } // namespace kudu
-
-#endif
