@@ -17,13 +17,14 @@
 
 #include <cstdint>
 #include <cstdlib>
+#include <functional>
 #include <memory>
 #include <ostream>
 #include <string>
+#include <thread>
 #include <vector>
 
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
@@ -36,7 +37,6 @@
 #include "kudu/common/rowid.h"
 #include "kudu/common/schema.h"
 #include "kudu/gutil/basictypes.h"
-#include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/local_tablet_writer.h"
 #include "kudu/tablet/rowset.h"
@@ -52,7 +52,6 @@
 #include "kudu/util/stopwatch.h"
 #include "kudu/util/test_graph.h"
 #include "kudu/util/test_util.h"
-#include "kudu/util/thread.h"
 
 DECLARE_int32(tablet_history_max_age_sec);
 DECLARE_double(tablet_delta_store_major_compact_min_ratio);
@@ -64,6 +63,7 @@ DEFINE_int32(num_slowreader_threads, 1, "Number of 'slow' reader threads to laun
 DEFINE_int32(num_flush_threads, 1, "Number of flusher reader threads to launch");
 DEFINE_int32(num_compact_threads, 1, "Number of compactor threads to launch");
 DEFINE_int32(num_undo_delta_gc_threads, 1, "Number of undo delta gc threads to launch");
+DEFINE_int32(num_deleted_rowset_gc_threads, 1, "Number of deleted rowset gc threads to launch");
 DEFINE_int32(num_updater_threads, 1, "Number of updating threads to launch");
 DEFINE_int32(num_flush_delta_threads, 1, "Number of delta flusher reader threads to launch");
 DEFINE_int32(num_minor_compact_deltas_threads, 1,
@@ -78,11 +78,17 @@ DEFINE_double(flusher_backoff, 2.0f, "Ratio to backoff the flusher thread");
 DEFINE_int32(flusher_initial_frequency_ms, 30, "Number of ms to wait between flushes");
 
 using std::shared_ptr;
+using std::thread;
 using std::unique_ptr;
+using std::vector;
 
 namespace kudu {
 namespace tablet {
 
+namespace {
+const MonoDelta kBackgroundOpInterval = MonoDelta::FromMilliseconds(100);
+} // anonymous namespace
+
 template<class SETUP>
 class MultiThreadedTabletTest : public TabletTestBase<SETUP> {
   // Import some names from superclass, since C++ is stingy about
@@ -114,8 +120,7 @@ class MultiThreadedTabletTest : public TabletTestBase<SETUP> {
                                    TabletHarness::Options::ClockType::LOGICAL_CLOCK)
     : TabletTestBase<SETUP>(clock_type),
       running_insert_count_(FLAGS_num_insert_threads),
-      ts_collector_(::testing::UnitTest::GetInstance()->current_test_info()->test_case_name()) {
-  }
+      ts_collector_(::testing::UnitTest::GetInstance()->current_test_info()->test_case_name()) {}
 
   void InsertThread(int tid) {
     CountDownOnScopeExit dec_count(&running_insert_count_);
@@ -309,49 +314,45 @@ class MultiThreadedTabletTest : public TabletTestBase<SETUP> {
     }
   }
 
-  void FlushDeltasThread(int tid) {
-    int wait_time = 100;
+  void FlushDeltasThread(int /*tid*/) {
     while (running_insert_count_.count() > 0) {
       CHECK_OK(tablet()->FlushBiggestDMS());
 
       // Wait, unless the inserters are all done.
-      running_insert_count_.WaitFor(MonoDelta::FromMilliseconds(wait_time));
+      running_insert_count_.WaitFor(kBackgroundOpInterval);
     }
   }
 
-  void MinorCompactDeltasThread(int tid) {
+  void MinorCompactDeltasThread(int /*tid*/) {
     CompactDeltas(RowSet::MINOR_DELTA_COMPACTION);
   }
 
-  void MajorCompactDeltasThread(int tid) {
+  void MajorCompactDeltasThread(int /*tid*/) {
     CompactDeltas(RowSet::MAJOR_DELTA_COMPACTION);
   }
 
   void CompactDeltas(RowSet::DeltaCompactionType type) {
-    int wait_time = 100;
     while (running_insert_count_.count() > 0) {
       VLOG(1) << "Compacting worst deltas";
       CHECK_OK(tablet()->CompactWorstDeltas(type));
 
       // Wait, unless the inserters are all done.
-      running_insert_count_.WaitFor(MonoDelta::FromMilliseconds(wait_time));
+      running_insert_count_.WaitFor(kBackgroundOpInterval);
     }
   }
 
-  void CompactThread(int tid) {
-    int wait_time = 100;
+  void CompactThread(int /*tid*/) {
     while (running_insert_count_.count() > 0) {
       CHECK_OK(tablet()->Compact(Tablet::COMPACT_NO_FLAGS));
 
       // Wait, unless the inserters are all done.
-      running_insert_count_.WaitFor(MonoDelta::FromMilliseconds(wait_time));
+      running_insert_count_.WaitFor(kBackgroundOpInterval);
     }
   }
 
   void DeleteAncientUndoDeltasThread(int /*tid*/) {
-    int wait_time = 100;
     while (running_insert_count_.count() > 0) {
-      MonoDelta time_budget = MonoDelta::FromMilliseconds(wait_time);
+      MonoDelta time_budget = kBackgroundOpInterval;
       int64_t bytes_in_ancient_undos = 0;
       CHECK_OK(tablet()->InitAncientUndoDeltas(time_budget, &bytes_in_ancient_undos));
       VLOG(1) << "Found " << bytes_in_ancient_undos << " bytes of ancient delta undos";
@@ -365,10 +366,26 @@ class MultiThreadedTabletTest : public TabletTestBase<SETUP> {
       }
 
       // Wait, unless the inserters are all done.
-      running_insert_count_.WaitFor(MonoDelta::FromMilliseconds(wait_time));
+      running_insert_count_.WaitFor(kBackgroundOpInterval);
     }
   }
 
+  // Thread that looks for rowsets that are ancient and fully deleted, GCing
+  // those that are.
+  void DeleteAncientDeletedRowsetsThreads(int /*tid*/) {
+    do {
+      int64_t bytes_in_ancient_deleted_rowsets = 0;
+      CHECK_OK(tablet()->GetBytesInAncientDeletedRowsets(&bytes_in_ancient_deleted_rowsets));
+      VLOG(1) << Substitute("Found $0 bytes in ancient, fully deleted rowsets",
+                            bytes_in_ancient_deleted_rowsets);
+      if (bytes_in_ancient_deleted_rowsets > 0) {
+        CHECK_OK(tablet()->DeleteAncientDeletedRowsets());
+        LOG(INFO) << Substitute("Deleted $0 bytes found in ancient, fully deleted rowsets",
+                                bytes_in_ancient_deleted_rowsets);
+      }
+    } while (!running_insert_count_.WaitFor(kBackgroundOpInterval));
+  }
+
   // Thread which cycles between inserting and deleting a test row, each time
   // with a different value.
   void DeleteAndReinsertCycleThread(int tid) {
@@ -424,23 +441,19 @@ class MultiThreadedTabletTest : public TabletTestBase<SETUP> {
     }
   }
 
-  template<typename FunctionType>
-  void StartThreads(int n_threads, const FunctionType &function) {
+  void StartThreads(int n_threads, const std::function<void(int)>& function) {
     for (int i = 0; i < n_threads; i++) {
-      scoped_refptr<kudu::Thread> new_thread;
-      CHECK_OK(kudu::Thread::Create("test", strings::Substitute("test$0", i),
-          function, this, i, &new_thread));
-      threads_.push_back(new_thread);
+      threads_.emplace_back([=]() { function(i); });
     }
   }
 
   void JoinThreads() {
-    for (scoped_refptr<kudu::Thread> thr : threads_) {
-     CHECK_OK(ThreadJoiner(thr.get()).Join());
+    for (auto& t : threads_) {
+      t.join();
     }
   }
 
-  std::vector<scoped_refptr<kudu::Thread> > threads_;
+  vector<thread> threads_;
   CountDownLatch running_insert_count_;
 
   // Projection with only an int column.
@@ -462,20 +475,29 @@ TYPED_TEST(MultiThreadedTabletTest, DoTestAllAtOnce) {
   }
 
   // Spawn a bunch of threads, each of which will do updates.
-  this->StartThreads(1, &TestFixture::CollectStatisticsThread);
-  this->StartThreads(FLAGS_num_insert_threads, &TestFixture::InsertThread);
-  this->StartThreads(FLAGS_num_counter_threads, &TestFixture::CountThread);
-  this->StartThreads(FLAGS_num_summer_threads, &TestFixture::SummerThread);
-  this->StartThreads(FLAGS_num_flush_threads, &TestFixture::FlushThread);
-  this->StartThreads(FLAGS_num_compact_threads, &TestFixture::CompactThread);
-  this->StartThreads(FLAGS_num_undo_delta_gc_threads, &TestFixture::DeleteAncientUndoDeltasThread);
-  this->StartThreads(FLAGS_num_flush_delta_threads, &TestFixture::FlushDeltasThread);
+  this->StartThreads(1, [this](int i) { this->CollectStatisticsThread(i); });
+  this->StartThreads(FLAGS_num_insert_threads,
+                     [this](int i) { this->InsertThread(i); });
+  this->StartThreads(FLAGS_num_counter_threads,
+                     [this](int i) { this->CountThread(i); });
+  this->StartThreads(FLAGS_num_summer_threads,
+                     [this](int i) { this->SummerThread(i); });
+  this->StartThreads(FLAGS_num_flush_threads,
+                     [this](int i) { this->FlushThread(i); });
+  this->StartThreads(FLAGS_num_compact_threads,
+                     [this](int i) { this->CompactThread(i); });
+  this->StartThreads(FLAGS_num_undo_delta_gc_threads,
+                     [this](int i) { this->DeleteAncientUndoDeltasThread(i); });
+  this->StartThreads(FLAGS_num_flush_delta_threads,
+                     [this](int i) { this->FlushDeltasThread(i); });
   this->StartThreads(FLAGS_num_minor_compact_deltas_threads,
-                     &TestFixture::MinorCompactDeltasThread);
+                     [this](int i) { this->MinorCompactDeltasThread(i); });
   this->StartThreads(FLAGS_num_major_compact_deltas_threads,
-                     &TestFixture::MajorCompactDeltasThread);
-  this->StartThreads(FLAGS_num_slowreader_threads, &TestFixture::SlowReaderThread);
-  this->StartThreads(FLAGS_num_updater_threads, &TestFixture::UpdateThread);
+                     [this](int i) { this->MajorCompactDeltasThread(i); });
+  this->StartThreads(FLAGS_num_slowreader_threads,
+                     [this](int i) { this->SlowReaderThread(i); });
+  this->StartThreads(FLAGS_num_updater_threads,
+                     [this](int i) { this->UpdateThread(i); });
   this->JoinThreads();
   LOG_TIMING(INFO, "Summing int32 column") {
     uint64_t sum = this->CountSum(shared_ptr<TimeSeries>());
@@ -493,21 +515,27 @@ TYPED_TEST(MultiThreadedTabletTest, DoTestAllAtOnce) {
 // of DELETE/REINSERT during flushes.
 TYPED_TEST(MultiThreadedTabletTest, DeleteAndReinsert) {
   google::FlagSaver saver;
-  FLAGS_flusher_backoff = 1.0f;
+  FLAGS_flusher_backoff = 1.0F;
   FLAGS_flusher_initial_frequency_ms = 1;
-  FLAGS_tablet_delta_store_major_compact_min_ratio = 0.01f;
+  FLAGS_tablet_delta_store_major_compact_min_ratio = 0.01F;
   FLAGS_tablet_delta_store_minor_compact_max = 10;
-  this->StartThreads(1, &TestFixture::CollectStatisticsThread);
-  this->StartThreads(FLAGS_num_flush_threads, &TestFixture::FlushThread);
-  this->StartThreads(FLAGS_num_compact_threads, &TestFixture::CompactThread);
-  this->StartThreads(FLAGS_num_undo_delta_gc_threads, &TestFixture::DeleteAncientUndoDeltasThread);
-  this->StartThreads(FLAGS_num_flush_delta_threads, &TestFixture::FlushDeltasThread);
+  this->StartThreads(1, [this](int i) { this->CollectStatisticsThread(i); });
+  this->StartThreads(FLAGS_num_flush_threads,
+                     [this](int i) { this->FlushThread(i); });
+  this->StartThreads(FLAGS_num_compact_threads,
+                     [this](int i) { this->CompactThread(i); });
+  this->StartThreads(FLAGS_num_undo_delta_gc_threads,
+                     [this](int i) { this->DeleteAncientUndoDeltasThread(i); });
+  this->StartThreads(FLAGS_num_flush_delta_threads,
+                     [this](int i) { this->FlushDeltasThread(i); });
   this->StartThreads(FLAGS_num_minor_compact_deltas_threads,
-                     &TestFixture::MinorCompactDeltasThread);
+                     [this](int i) { this->MinorCompactDeltasThread(i); });
   this->StartThreads(FLAGS_num_major_compact_deltas_threads,
-                     &TestFixture::MajorCompactDeltasThread);
-  this->StartThreads(10, &TestFixture::DeleteAndReinsertCycleThread);
-  this->StartThreads(10, &TestFixture::StubbornlyUpdateSameRowThread);
+                     [this](int i) { this->MajorCompactDeltasThread(i); });
+  this->StartThreads(10,
+                     [this](int i) { this->DeleteAndReinsertCycleThread(i); });
+  this->StartThreads(10,
+                     [this](int i) { this->StubbornlyUpdateSameRowThread(i); });
 
   // Run very quickly in dev builds, longer in slow builds.
   float runtime_seconds = AllowSlowTests() ? 2 : 0.1;
@@ -542,17 +570,32 @@ TYPED_TEST(MultiThreadedHybridClockTabletTest, UpdateNoMergeCompaction) {
   google::FlagSaver saver;
   FLAGS_tablet_history_max_age_sec = 0; // GC data as aggressively as possible.
 
-  FLAGS_flusher_backoff = 1.0f;
+  FLAGS_flusher_backoff = 1.0F;
   FLAGS_flusher_initial_frequency_ms = 1;
-  FLAGS_tablet_delta_store_major_compact_min_ratio = 0.01f;
+  FLAGS_tablet_delta_store_major_compact_min_ratio = 0.01F;
   FLAGS_tablet_delta_store_minor_compact_max = 10;
-  this->StartThreads(FLAGS_num_flush_threads, &TestFixture::FlushThread);
-  this->StartThreads(FLAGS_num_flush_delta_threads, &TestFixture::FlushDeltasThread);
+
+  // Start up our background op threads, targeting the creation of delta files.
+  this->StartThreads(FLAGS_num_flush_threads,
+                     [this](int i) { this->FlushThread(i); });
+  this->StartThreads(FLAGS_num_flush_delta_threads,
+                     [this](int i) { this->FlushDeltasThread(i); });
   this->StartThreads(FLAGS_num_major_compact_deltas_threads,
-                     &TestFixture::MajorCompactDeltasThread);
-  this->StartThreads(10, &TestFixture::DeleteAndReinsertCycleThread);
-  this->StartThreads(10, &TestFixture::StubbornlyUpdateSameRowThread);
-  this->StartThreads(FLAGS_num_undo_delta_gc_threads, &TestFixture::DeleteAncientUndoDeltasThread);
+                     [this](int i) { this->MajorCompactDeltasThread(i); });
+  this->StartThreads(FLAGS_num_undo_delta_gc_threads,
+                     [this](int i) { this->DeleteAncientUndoDeltasThread(i); });
+  this->StartThreads(FLAGS_num_deleted_rowset_gc_threads,
+                     [this](int i) { this->DeleteAncientDeletedRowsetsThreads(i); });
+  // Start our workload threads, targeting the creation of deltas that we can
+  // eventually GC.
+  this->StartThreads(10,
+                     [this](int i) { this->DeleteAndReinsertCycleThread(i); });
+  this->StartThreads(10,
+                     [this](int i) { this->StubbornlyUpdateSameRowThread(i); });
+
+  // For good measure, we'll also start a thread that scans.
+  this->StartThreads(FLAGS_num_summer_threads,
+                     [this](int i) { this->SummerThread(i); });
 
   // Run very quickly in dev builds, longer in slow builds.
   float runtime_seconds = AllowSlowTests() ? 2 : 0.1;
