@@ -15,6 +15,9 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include "kudu/tablet/mvcc.h"
+
+#include <memory>
 #include <mutex>
 #include <ostream>
 #include <string>
@@ -24,32 +27,28 @@
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
-#include "kudu/clock/clock.h"
 #include "kudu/clock/hybrid_clock.h"
 #include "kudu/clock/logical_clock.h"
 #include "kudu/common/timestamp.h"
-#include "kudu/gutil/gscoped_ptr.h"
-#include "kudu/gutil/ref_counted.h"
-#include "kudu/tablet/mvcc.h"
 #include "kudu/util/locks.h"
+#include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
 using std::thread;
+using std::unique_ptr;
+
+METRIC_DECLARE_entity(server);
 
 namespace kudu {
 namespace tablet {
 
-using clock::Clock;
-using clock::HybridClock;
-
 class MvccTest : public KuduTest {
  public:
   MvccTest()
-      : clock_(
-          clock::LogicalClock::CreateStartingAt(Timestamp::kInitialTimestamp)) {
+      : clock_(Timestamp::kInitialTimestamp) {
   }
 
   void WaitForSnapshotAtTSThread(MvccManager* mgr, Timestamp ts) {
@@ -66,10 +65,9 @@ class MvccTest : public KuduTest {
   }
 
  protected:
-  scoped_refptr<clock::Clock> clock_;
-
-  mutable simple_spinlock lock_;
-  gscoped_ptr<MvccSnapshot> result_snapshot_;
+  clock::LogicalClock clock_;
+  simple_spinlock lock_;
+  unique_ptr<MvccSnapshot> result_snapshot_;
 };
 
 TEST_F(MvccTest, TestMvccBasic) {
@@ -77,33 +75,33 @@ TEST_F(MvccTest, TestMvccBasic) {
   MvccSnapshot snap;
 
   // Initial state should not have any committed transactions.
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed={T|T < 1}]", snap.ToString());
   ASSERT_FALSE(snap.IsCommitted(Timestamp(1)));
   ASSERT_FALSE(snap.IsCommitted(Timestamp(2)));
 
   // Start timestamp 1
-  Timestamp t = clock_->Now();
+  Timestamp t = clock_.Now();
   ASSERT_EQ(1, t.value());
-  mgr.StartTransaction(t);
+  ScopedTransaction txn(&mgr, t);
 
   // State should still have no committed transactions, since 1 is in-flight.
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed={T|T < 1}]", snap.ToString());
   ASSERT_FALSE(snap.IsCommitted(Timestamp(1)));
   ASSERT_FALSE(snap.IsCommitted(Timestamp(2)));
 
   // Mark timestamp 1 as "applying"
-  mgr.StartApplyingTransaction(t);
+  txn.StartApplying();
 
   // This should not change the set of committed transactions.
   ASSERT_FALSE(snap.IsCommitted(Timestamp(1)));
 
   // Commit timestamp 1
-  mgr.CommitTransaction(t);
+  txn.Commit();
 
   // State should show 0 as committed, 1 as uncommitted.
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed={T|T < 1 or (T in {1})}]", snap.ToString());
   ASSERT_TRUE(snap.IsCommitted(Timestamp(1)));
   ASSERT_FALSE(snap.IsCommitted(Timestamp(2)));
@@ -113,26 +111,26 @@ TEST_F(MvccTest, TestMvccMultipleInFlight) {
   MvccManager mgr;
   MvccSnapshot snap;
 
-  Timestamp t1 = clock_->Now();
+  Timestamp t1 = clock_.Now();
   ASSERT_EQ(1, t1.value());
-  mgr.StartTransaction(t1);
-  Timestamp t2 = clock_->Now();
+  ScopedTransaction txn1(&mgr, t1);
+  Timestamp t2 = clock_.Now();
   ASSERT_EQ(2, t2.value());
-  mgr.StartTransaction(t2);
+  ScopedTransaction txn2(&mgr, t2);
 
   // State should still have no committed transactions, since both are in-flight.
 
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed={T|T < 1}]", snap.ToString());
   ASSERT_FALSE(snap.IsCommitted(t1));
   ASSERT_FALSE(snap.IsCommitted(t2));
 
   // Commit timestamp 2
-  mgr.StartApplyingTransaction(t2);
-  mgr.CommitTransaction(t2);
+  txn2.StartApplying();
+  txn2.Commit();
 
   // State should show 2 as committed, 1 as uncommitted.
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed="
             "{T|T < 1 or (T in {2})}]",
             snap.ToString());
@@ -140,12 +138,12 @@ TEST_F(MvccTest, TestMvccMultipleInFlight) {
   ASSERT_TRUE(snap.IsCommitted(t2));
 
   // Start another transaction. This gets timestamp 3
-  Timestamp t3 = clock_->Now();
+  Timestamp t3 = clock_.Now();
   ASSERT_EQ(3, t3.value());
-  mgr.StartTransaction(t3);
+  ScopedTransaction txn3(&mgr, t3);
 
   // State should show 2 as committed, 1 and 4 as uncommitted.
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed="
             "{T|T < 1 or (T in {2})}]",
             snap.ToString());
@@ -154,11 +152,11 @@ TEST_F(MvccTest, TestMvccMultipleInFlight) {
   ASSERT_FALSE(snap.IsCommitted(t3));
 
   // Commit 3
-  mgr.StartApplyingTransaction(t3);
-  mgr.CommitTransaction(t3);
+  txn3.StartApplying();
+  txn3.Commit();
 
   // 2 and 3 committed
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed="
             "{T|T < 1 or (T in {2,3})}]",
             snap.ToString());
@@ -167,14 +165,14 @@ TEST_F(MvccTest, TestMvccMultipleInFlight) {
   ASSERT_TRUE(snap.IsCommitted(t3));
 
   // Commit 1
-  mgr.StartApplyingTransaction(t1);
-  mgr.CommitTransaction(t1);
+  txn1.StartApplying();
+  txn1.Commit();
 
-  // All transactions are committed, adjust the safe time.
-  mgr.AdjustSafeTime(t3);
+  // All transactions are committed, adjust the new transaction lower bound.
+  mgr.AdjustNewTransactionLowerBound(t3);
 
   // all committed
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_EQ("MvccSnapshot[committed={T|T < 3 or (T in {3})}]", snap.ToString());
   ASSERT_TRUE(snap.IsCommitted(t1));
   ASSERT_TRUE(snap.IsCommitted(t2));
@@ -182,83 +180,86 @@ TEST_F(MvccTest, TestMvccMultipleInFlight) {
 }
 
 TEST_F(MvccTest, TestOutOfOrderTxns) {
-  scoped_refptr<Clock> hybrid_clock(new HybridClock());
-  ASSERT_OK(hybrid_clock->Init());
+  MetricRegistry metric_registry;
+  auto metric_entity(METRIC_ENTITY_server.Instantiate(&metric_registry, "mvcc-test"));
+  clock::HybridClock hybrid_clock(metric_entity);
+  ASSERT_OK(hybrid_clock.Init());
   MvccManager mgr;
 
   // Start a normal non-commit-wait txn.
-  Timestamp normal_txn = hybrid_clock->Now();
-  mgr.StartTransaction(normal_txn);
+  Timestamp first_ts = hybrid_clock.Now();
+  ScopedTransaction first_txn(&mgr, first_ts);
 
-  MvccSnapshot s1(mgr);
+  // Take a snapshot that con
+  MvccSnapshot snap_with_nothing_committed(mgr);
 
   // Start a transaction as if it were using commit-wait (i.e. started in future)
-  Timestamp cw_txn = hybrid_clock->NowLatest();
-  mgr.StartTransaction(cw_txn);
+  Timestamp cw_ts = hybrid_clock.NowLatest();
+  ScopedTransaction cw_txn(&mgr, cw_ts);
 
   // Commit the original txn
-  mgr.StartApplyingTransaction(normal_txn);
-  mgr.CommitTransaction(normal_txn);
+  first_txn.StartApplying();
+  first_txn.Commit();
 
   // Start a new txn
-  Timestamp normal_txn_2 = hybrid_clock->Now();
-  mgr.StartTransaction(normal_txn_2);
+  Timestamp second_ts = hybrid_clock.Now();
+  ScopedTransaction second_txn(&mgr, second_ts);
 
   // The old snapshot should not have either txn
-  EXPECT_FALSE(s1.IsCommitted(normal_txn));
-  EXPECT_FALSE(s1.IsCommitted(normal_txn_2));
+  EXPECT_FALSE(snap_with_nothing_committed.IsCommitted(first_ts));
+  EXPECT_FALSE(snap_with_nothing_committed.IsCommitted(second_ts));
 
   // A new snapshot should have only the first transaction
-  MvccSnapshot s2(mgr);
-  EXPECT_TRUE(s2.IsCommitted(normal_txn));
-  EXPECT_FALSE(s2.IsCommitted(normal_txn_2));
+  MvccSnapshot snap_with_first_committed(mgr);
+  EXPECT_TRUE(snap_with_first_committed.IsCommitted(first_ts));
+  EXPECT_FALSE(snap_with_first_committed.IsCommitted(second_ts));
 
   // Commit the commit-wait one once it is time.
-  ASSERT_OK(hybrid_clock->WaitUntilAfter(cw_txn, MonoTime::Max()));
-  mgr.StartApplyingTransaction(cw_txn);
-  mgr.CommitTransaction(cw_txn);
+  ASSERT_OK(hybrid_clock.WaitUntilAfter(cw_ts, MonoTime::Max()));
+  cw_txn.StartApplying();
+  cw_txn.Commit();
 
   // A new snapshot at this point should still think that normal_txn_2 is uncommitted
-  MvccSnapshot s3(mgr);
-  EXPECT_FALSE(s3.IsCommitted(normal_txn_2));
+  MvccSnapshot snap_with_all_committed(mgr);
+  EXPECT_FALSE(snap_with_all_committed.IsCommitted(second_ts));
 }
 
 // Tests starting transaction at a point-in-time in the past and committing them while
-// adjusting safe time.
+// adjusting the new transaction timestamp lower bound.
 TEST_F(MvccTest, TestSafeTimeWithOutOfOrderTxns) {
   MvccManager mgr;
 
   // Set the clock to some time in the "future".
-  ASSERT_OK(clock_->Update(Timestamp(100)));
+  ASSERT_OK(clock_.Update(Timestamp(100)));
 
   // Start a transaction in the "past"
-  Timestamp txn_in_the_past(50);
-  mgr.StartTransaction(txn_in_the_past);
-  mgr.StartApplyingTransaction(txn_in_the_past);
+  Timestamp ts_in_the_past(50);
+  ScopedTransaction txn_in_the_past(&mgr, ts_in_the_past);
+  txn_in_the_past.StartApplying();
 
   ASSERT_EQ(Timestamp::kInitialTimestamp, mgr.GetCleanTimestamp());
 
-  // Committing 'txn_in_the_past' should not advance safe time or clean time.
-  mgr.CommitTransaction(txn_in_the_past);
+  // Committing 'txn_in_the_past' should not advance the new transaction lower
+  // bound or the clean time.
+  txn_in_the_past.Commit();
 
   // Now take a snapshot.
-  MvccSnapshot snap1;
-  mgr.TakeSnapshot(&snap1);
+  MvccSnapshot snap_with_first_txn(mgr);
 
-  // Because we did not advance the the safe or clean watermarkd, even though the only
-  // in-flight transaction was committed at time 50, a transaction at time 40 should still be
-  // considered uncommitted.
-  ASSERT_FALSE(snap1.IsCommitted(Timestamp(40)));
+  // Because we did not advance the the new transaction lower bound or clean
+  // time, even though the only in-flight transaction was committed at time 50,
+  // a transaction at time 40 should still be considered uncommitted.
+  ASSERT_FALSE(snap_with_first_txn.IsCommitted(Timestamp(40)));
 
-  // Now advance the both clean and safe watermarks to the last committed transaction.
-  mgr.AdjustSafeTime(Timestamp(50));
+  // Now advance the both clean and new transaction lower bound watermarks to
+  // the last committed transaction.
+  mgr.AdjustNewTransactionLowerBound(Timestamp(50));
 
-  ASSERT_EQ(txn_in_the_past, mgr.GetCleanTimestamp());
+  ASSERT_EQ(ts_in_the_past, mgr.GetCleanTimestamp());
 
-  MvccSnapshot snap2;
-  mgr.TakeSnapshot(&snap2);
+  MvccSnapshot snap_with_adjusted_clean_time(mgr);
 
-  ASSERT_TRUE(snap2.IsCommitted(Timestamp(40)));
+  ASSERT_TRUE(snap_with_adjusted_clean_time.IsCommitted(Timestamp(40)));
 }
 
 TEST_F(MvccTest, TestScopedTransaction) {
@@ -266,8 +267,8 @@ TEST_F(MvccTest, TestScopedTransaction) {
   MvccSnapshot snap;
 
   {
-    ScopedTransaction t1(&mgr, clock_->Now());
-    ScopedTransaction t2(&mgr, clock_->Now());
+    ScopedTransaction t1(&mgr, clock_.Now());
+    ScopedTransaction t2(&mgr, clock_.Now());
 
     ASSERT_EQ(1, t1.timestamp().value());
     ASSERT_EQ(2, t2.timestamp().value());
@@ -275,13 +276,13 @@ TEST_F(MvccTest, TestScopedTransaction) {
     t1.StartApplying();
     t1.Commit();
 
-    mgr.TakeSnapshot(&snap);
+    snap = MvccSnapshot(mgr);
     ASSERT_TRUE(snap.IsCommitted(t1.timestamp()));
     ASSERT_FALSE(snap.IsCommitted(t2.timestamp()));
   }
 
   // t2 going out of scope aborts it.
-  mgr.TakeSnapshot(&snap);
+  snap = MvccSnapshot(mgr);
   ASSERT_TRUE(snap.IsCommitted(Timestamp(1)));
   ASSERT_FALSE(snap.IsCommitted(Timestamp(2)));
 
@@ -289,7 +290,7 @@ TEST_F(MvccTest, TestScopedTransaction) {
   // scope while the MvccManager is closed.
   mgr.Close();
   {
-    ScopedTransaction t(&mgr, clock_->Now());
+    ScopedTransaction t(&mgr, clock_.Now());
     NO_FATALS(t.StartApplying());
   }
 }
@@ -387,50 +388,50 @@ TEST_F(MvccTest, TestMayHaveUncommittedTransactionsBefore) {
   ASSERT_FALSE(snap2.MayHaveUncommittedTransactionsAtOrBefore(Timestamp(10)));
 }
 
-TEST_F(MvccTest, TestAreAllTransactionsCommitted) {
+TEST_F(MvccTest, TestAreAllTransactionsCommittedForTests) {
   MvccManager mgr;
 
   // start several transactions and take snapshots along the way
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  Timestamp tx2 = clock_->Now();
-  mgr.StartTransaction(tx2);
-  Timestamp tx3 = clock_->Now();
-  mgr.StartTransaction(tx3);
-  mgr.AdjustSafeTime(clock_->Now());
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction txn1(&mgr, ts1);
+  Timestamp ts2 = clock_.Now();
+  ScopedTransaction txn2(&mgr, ts2);
+  Timestamp ts3 = clock_.Now();
+  ScopedTransaction txn3(&mgr, ts3);
+  mgr.AdjustNewTransactionLowerBound(clock_.Now());
 
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(1)));
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(2)));
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(3)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(1)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(2)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(3)));
 
   // commit tx3, should all still report as having as having uncommitted
   // transactions.
-  mgr.StartApplyingTransaction(tx3);
-  mgr.CommitTransaction(tx3);
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(1)));
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(2)));
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(3)));
+  txn3.StartApplying();
+  txn3.Commit();
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(1)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(2)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(3)));
 
   // commit tx1, first snap with in-flights should now report as all committed
   // and remaining snaps as still having uncommitted transactions
-  mgr.StartApplyingTransaction(tx1);
-  mgr.CommitTransaction(tx1);
-  ASSERT_TRUE(mgr.AreAllTransactionsCommitted(Timestamp(1)));
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(2)));
-  ASSERT_FALSE(mgr.AreAllTransactionsCommitted(Timestamp(3)));
+  txn1.StartApplying();
+  txn1.Commit();
+  ASSERT_TRUE(mgr.AreAllTransactionsCommittedForTests(Timestamp(1)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(2)));
+  ASSERT_FALSE(mgr.AreAllTransactionsCommittedForTests(Timestamp(3)));
 
   // Now they should all report as all committed.
-  mgr.StartApplyingTransaction(tx2);
-  mgr.CommitTransaction(tx2);
-  ASSERT_TRUE(mgr.AreAllTransactionsCommitted(Timestamp(1)));
-  ASSERT_TRUE(mgr.AreAllTransactionsCommitted(Timestamp(2)));
-  ASSERT_TRUE(mgr.AreAllTransactionsCommitted(Timestamp(3)));
+  txn2.StartApplying();
+  txn2.Commit();
+  ASSERT_TRUE(mgr.AreAllTransactionsCommittedForTests(Timestamp(1)));
+  ASSERT_TRUE(mgr.AreAllTransactionsCommittedForTests(Timestamp(2)));
+  ASSERT_TRUE(mgr.AreAllTransactionsCommittedForTests(Timestamp(3)));
 }
 
-TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapWithNoInflights) {
+TEST_F(MvccTest, WaitForCleanSnapshotSnapWithNoInflights) {
   MvccManager mgr;
-  Timestamp to_wait_for = clock_->Now();
-  mgr.AdjustSafeTime(clock_->Now());
+  Timestamp to_wait_for = clock_.Now();
+  mgr.AdjustNewTransactionLowerBound(clock_.Now());
   thread waiting_thread = thread(&MvccTest::WaitForSnapshotAtTSThread, this, &mgr, to_wait_for);
 
   // join immediately.
@@ -438,46 +439,47 @@ TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapWithNoInflights) {
   ASSERT_TRUE(HasResultSnapshot());
 }
 
-TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapBeforeSafeTimeWithInFlights) {
+TEST_F(MvccTest, WaitForCleanSnapshotSnapBeforeSafeTimeWithInFlights) {
   MvccManager mgr;
 
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  Timestamp tx2 = clock_->Now();
-  mgr.StartTransaction(tx2);
-  mgr.AdjustSafeTime(tx2);
-  Timestamp to_wait_for = clock_->Now();
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction txn1(&mgr, ts1);
+  Timestamp ts2 = clock_.Now();
+  ScopedTransaction txn2(&mgr, ts2);
+  mgr.AdjustNewTransactionLowerBound(ts2);
+  Timestamp to_wait_for = clock_.Now();
 
-  // Select a safe time that is after all transactions and after the the timestamp we'll wait for
-  // and adjust it on the MvccManager. This will cause "clean time" to move when tx1 and tx2 commit.
-  Timestamp safe_time = clock_->Now();
-  mgr.AdjustSafeTime(safe_time);
+  // Select a new transaction timestamp lower bound that is after all
+  // transactions and after the the timestamp we'll wait for.
+  // This will cause "clean time" to move when tx1 and tx2 commit.
+  Timestamp future_ts = clock_.Now();
+  mgr.AdjustNewTransactionLowerBound(future_ts);
 
   thread waiting_thread = thread(&MvccTest::WaitForSnapshotAtTSThread, this, &mgr, to_wait_for);
 
   ASSERT_FALSE(HasResultSnapshot());
-  mgr.StartApplyingTransaction(tx1);
-  mgr.CommitTransaction(tx1);
+  txn1.StartApplying();
+  txn1.Commit();
   ASSERT_FALSE(HasResultSnapshot());
-  mgr.StartApplyingTransaction(tx2);
-  mgr.CommitTransaction(tx2);
+  txn2.StartApplying();
+  txn2.Commit();
   waiting_thread.join();
   ASSERT_TRUE(HasResultSnapshot());
 }
 
-TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapAfterSafeTimeWithInFlights) {
+TEST_F(MvccTest, WaitForCleanSnapshotSnapAfterSafeTimeWithInFlights) {
   MvccManager mgr;
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  Timestamp tx2 = clock_->Now();
-  mgr.StartTransaction(tx2);
-  mgr.AdjustSafeTime(tx2);
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction txn1(&mgr, ts1);
+  Timestamp ts2 = clock_.Now();
+  ScopedTransaction txn2(&mgr, ts2);
+  mgr.AdjustNewTransactionLowerBound(ts2);
 
   // Wait should return immediately, since we have no transactions "applying"
   // yet.
   ASSERT_OK(mgr.WaitForApplyingTransactionsToCommit());
 
-  mgr.StartApplyingTransaction(tx1);
+  txn1.StartApplying();
 
   Status s;
   thread waiting_thread = thread([&] {
@@ -489,50 +491,51 @@ TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapAfterSafeTimeWithInFlights) {
   ASSERT_EQ(mgr.GetNumWaitersForTests(), 1);
 
   // Aborting the other transaction shouldn't affect our waiter.
-  mgr.AbortTransaction(tx2);
+  txn2.Abort();
   ASSERT_EQ(mgr.GetNumWaitersForTests(), 1);
 
   // Committing our transaction should wake the waiter.
-  mgr.CommitTransaction(tx1);
+  txn1.Commit();
   ASSERT_EQ(mgr.GetNumWaitersForTests(), 0);
   waiting_thread.join();
   ASSERT_OK(s);
 }
 
-TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapAtTimestampWithInFlights) {
+TEST_F(MvccTest, WaitForCleanSnapshotSnapAtTimestampWithInFlights) {
   MvccManager mgr;
 
   // Transactions with timestamp 1 through 3
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  Timestamp tx2 = clock_->Now();
-  mgr.StartTransaction(tx2);
-  Timestamp tx3 = clock_->Now();
-  mgr.StartTransaction(tx3);
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction tx1(&mgr, ts1);
+  Timestamp ts2 = clock_.Now();
+  ScopedTransaction tx2(&mgr, ts2);
+  Timestamp ts3 = clock_.Now();
+  ScopedTransaction tx3(&mgr, ts3);
 
   // Start a thread waiting for transactions with ts <= 2 to commit
-  thread waiting_thread = thread(&MvccTest::WaitForSnapshotAtTSThread, this, &mgr, tx2);
+  thread waiting_thread = thread(&MvccTest::WaitForSnapshotAtTSThread, this, &mgr, ts2);
   ASSERT_FALSE(HasResultSnapshot());
 
   // Commit tx 1 - thread should still wait.
-  mgr.StartApplyingTransaction(tx1);
-  mgr.CommitTransaction(tx1);
+  tx1.StartApplying();
+  tx1.Commit();
   SleepFor(MonoDelta::FromMilliseconds(1));
   ASSERT_FALSE(HasResultSnapshot());
 
   // Commit tx 3 - thread should still wait.
-  mgr.StartApplyingTransaction(tx3);
-  mgr.CommitTransaction(tx3);
+  tx3.StartApplying();
+  tx3.Commit();
   SleepFor(MonoDelta::FromMilliseconds(1));
   ASSERT_FALSE(HasResultSnapshot());
 
   // Commit tx 2 - thread should still wait.
-  mgr.StartApplyingTransaction(tx2);
-  mgr.CommitTransaction(tx2);
+  tx2.StartApplying();
+  tx2.Commit();
   ASSERT_FALSE(HasResultSnapshot());
 
-  // Advance safe time, thread should continue.
-  mgr.AdjustSafeTime(tx3);
+  // Advance new transaction lower bound and the clean time, thread should
+  // continue.
+  mgr.AdjustNewTransactionLowerBound(ts3);
   waiting_thread.join();
   ASSERT_TRUE(HasResultSnapshot());
 }
@@ -540,17 +543,17 @@ TEST_F(MvccTest, TestWaitForCleanSnapshot_SnapAtTimestampWithInFlights) {
 TEST_F(MvccTest, TestWaitForApplyingTransactionsToCommit) {
   MvccManager mgr;
 
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  Timestamp tx2 = clock_->Now();
-  mgr.StartTransaction(tx2);
-  mgr.AdjustSafeTime(tx2);
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction txn1(&mgr, ts1);
+  Timestamp ts2 = clock_.Now();
+  ScopedTransaction txn2(&mgr, ts2);
+  mgr.AdjustNewTransactionLowerBound(ts2);
 
   // Wait should return immediately, since we have no transactions "applying"
   // yet.
   ASSERT_OK(mgr.WaitForApplyingTransactionsToCommit());
 
-  mgr.StartApplyingTransaction(tx1);
+  txn1.StartApplying();
 
   thread waiting_thread = thread(&MvccManager::WaitForApplyingTransactionsToCommit, &mgr);
   while (mgr.GetNumWaitersForTests() == 0) {
@@ -559,11 +562,11 @@ TEST_F(MvccTest, TestWaitForApplyingTransactionsToCommit) {
   ASSERT_EQ(mgr.GetNumWaitersForTests(), 1);
 
   // Aborting the other transaction shouldn't affect our waiter.
-  mgr.AbortTransaction(tx2);
+  txn2.Abort();
   ASSERT_EQ(mgr.GetNumWaitersForTests(), 1);
 
   // Committing our transaction should wake the waiter.
-  mgr.CommitTransaction(tx1);
+  txn1.Commit();
   ASSERT_EQ(mgr.GetNumWaitersForTests(), 0);
   waiting_thread.join();
 }
@@ -572,10 +575,10 @@ TEST_F(MvccTest, TestWaitForApplyingTransactionsToCommit) {
 // instead return an error.
 TEST_F(MvccTest, TestDontWaitAfterClose) {
   MvccManager mgr;
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  mgr.AdjustSafeTime(tx1);
-  mgr.StartApplyingTransaction(tx1);
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction txn1(&mgr, ts1);
+  mgr.AdjustNewTransactionLowerBound(ts1);
+  txn1.StartApplying();
 
   // Spin up a thread to wait on the applying transaction.
   // Lock the changing status. This is only necessary in this test to read the
@@ -605,57 +608,56 @@ TEST_F(MvccTest, TestDontWaitAfterClose) {
   ASSERT_TRUE(s.IsAborted());
 }
 
-// Test that if we abort a transaction we don't advance the safe time and don't
-// add the transaction to the committed set.
+// Test that if we abort a transaction we don't advance the new transaction
+// lower bound and don't add the transaction to the committed set.
 TEST_F(MvccTest, TestTxnAbort) {
 
   MvccManager mgr;
 
   // Transactions with timestamps 1 through 3
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
-  Timestamp tx2 = clock_->Now();
-  mgr.StartTransaction(tx2);
-  Timestamp tx3 = clock_->Now();
-  mgr.StartTransaction(tx3);
-  mgr.AdjustSafeTime(tx3);
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction txn1(&mgr, ts1);
+  Timestamp ts2 = clock_.Now();
+  ScopedTransaction txn2(&mgr, ts2);
+  Timestamp ts3 = clock_.Now();
+  ScopedTransaction txn3(&mgr, ts3);
+  mgr.AdjustNewTransactionLowerBound(ts3);
 
   // Now abort tx1, this shouldn't move the clean time and the transaction
   // shouldn't be reported as committed.
-  mgr.AbortTransaction(tx1);
+  txn1.Abort();
   ASSERT_EQ(Timestamp::kInitialTimestamp, mgr.GetCleanTimestamp());
-  ASSERT_FALSE(mgr.cur_snap_.IsCommitted(tx1));
+  ASSERT_FALSE(mgr.cur_snap_.IsCommitted(ts1));
 
   // Committing tx3 shouldn't advance the clean time since it is not the earliest
-  // in-flight, but it should advance 'safe_time_' to 3.
-  mgr.StartApplyingTransaction(tx3);
-  mgr.CommitTransaction(tx3);
-  ASSERT_TRUE(mgr.cur_snap_.IsCommitted(tx3));
-  ASSERT_EQ(tx3, mgr.safe_time_);
+  // in-flight, but it should advance 'new_txn_timestamp_exc_lower_bound_' to 3.
+  txn3.StartApplying();
+  txn3.Commit();
+  ASSERT_TRUE(mgr.cur_snap_.IsCommitted(ts3));
+  ASSERT_EQ(ts3, mgr.new_txn_timestamp_exc_lower_bound_);
 
   // Committing tx2 should advance the clean time to 3.
-  mgr.StartApplyingTransaction(tx2);
-  mgr.CommitTransaction(tx2);
-  ASSERT_TRUE(mgr.cur_snap_.IsCommitted(tx2));
-  ASSERT_EQ(tx3, mgr.GetCleanTimestamp());
+  txn2.StartApplying();
+  txn2.Commit();
+  ASSERT_TRUE(mgr.cur_snap_.IsCommitted(ts2));
+  ASSERT_EQ(ts3, mgr.GetCleanTimestamp());
 }
 
 // This tests for a bug we were observing, where a clean snapshot would not
 // coalesce to the latest timestamp.
 TEST_F(MvccTest, TestAutomaticCleanTimeMoveToSafeTimeOnCommit) {
-
   MvccManager mgr;
-  clock_->Update(Timestamp(20));
+  clock_.Update(Timestamp(20));
 
-  mgr.StartTransaction(Timestamp(10));
-  mgr.StartTransaction(Timestamp(15));
-  mgr.AdjustSafeTime(Timestamp(15));
+  ScopedTransaction tx1(&mgr, Timestamp(10));
+  ScopedTransaction tx2(&mgr, Timestamp(15));
+  mgr.AdjustNewTransactionLowerBound(Timestamp(15));
 
-  mgr.StartApplyingTransaction(Timestamp(15));
-  mgr.CommitTransaction(Timestamp(15));
+  tx2.StartApplying();
+  tx2.Commit();
 
-  mgr.StartApplyingTransaction(Timestamp(10));
-  mgr.CommitTransaction(Timestamp(10));
+  tx1.StartApplying();
+  tx1.Commit();
   ASSERT_EQ(mgr.cur_snap_.ToString(), "MvccSnapshot[committed={T|T < 15 or (T in {15})}]");
 }
 
@@ -683,7 +685,7 @@ TEST_F(MvccTest, TestIllegalStateTransitionsCrash) {
     "Trying to commit a transaction with a future timestamp|"
     "Trying to remove timestamp which isn't in the in-flight set: 1");
 
-  clock_->Update(Timestamp(20));
+  clock_.Update(Timestamp(20));
 
   EXPECT_DEATH({
       mgr.CommitTransaction(Timestamp(1));
@@ -691,7 +693,7 @@ TEST_F(MvccTest, TestIllegalStateTransitionsCrash) {
 
   // Start a transaction, and try committing it without having moved to "Applying"
   // state.
-  Timestamp t = clock_->Now();
+  Timestamp t = clock_.Now();
   mgr.StartTransaction(t);
   EXPECT_DEATH({
       mgr.CommitTransaction(t);
@@ -706,9 +708,9 @@ TEST_F(MvccTest, TestIllegalStateTransitionsCrash) {
     }, "Trying to remove timestamp which isn't in the in-flight set: 21");
 
   // Start a new transaction. This time, mark it as Applying.
-  t = clock_->Now();
+  t = clock_.Now();
   mgr.StartTransaction(t);
-  mgr.AdjustSafeTime(t);
+  mgr.AdjustNewTransactionLowerBound(t);
   mgr.StartApplyingTransaction(t);
 
   // Can only call StartApplying once.
@@ -729,56 +731,57 @@ TEST_F(MvccTest, TestWaitUntilCleanDeadline) {
   MvccManager mgr;
 
   // Transactions with timestamp 1
-  Timestamp tx1 = clock_->Now();
-  mgr.StartTransaction(tx1);
+  Timestamp ts1 = clock_.Now();
+  ScopedTransaction tx1(&mgr, ts1);
 
   // Wait until the 'tx1' timestamp is clean -- this won't happen because the
   // transaction isn't committed yet.
   MonoTime deadline = MonoTime::Now() + MonoDelta::FromMilliseconds(10);
   MvccSnapshot snap;
-  Status s = mgr.WaitForSnapshotWithAllCommitted(tx1, &snap, deadline);
+  Status s = mgr.WaitForSnapshotWithAllCommitted(ts1, &snap, deadline);
   ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
 }
 
-// Test for a bug related to the initialization of the MvccManager without
-// any pending transactions, i.e. when there are only calls to AdvanceSafeTime().
-// Prior to the fix we would advance safe/clean time but not the
+// Test for a bug related to the initialization of the MvccManager without any
+// pending transactions, i.e. when there are only calls to
+// AdjustNewTransactionLowerBound().
+//
+// Prior to the fix we would advance clean time but not the
 // 'none_committed_at_or_after_' watermark, meaning the latter would become lower
-// than safe/clean time. This had the effect on compaction of culling delta files
+// than clean time. This had the effect on compaction of culling delta files
 // even though they shouldn't be culled.
 // This test makes sure that watermarks are advanced correctly and that delta
 // files are culled correctly.
 TEST_F(MvccTest, TestCorrectInitWithNoTxns) {
   MvccManager mgr;
 
-  MvccSnapshot snap;
-  mgr.TakeSnapshot(&snap);
+  MvccSnapshot snap(mgr);
   EXPECT_EQ(snap.all_committed_before_, Timestamp::kInitialTimestamp);
   EXPECT_EQ(snap.none_committed_at_or_after_, Timestamp::kInitialTimestamp);
   EXPECT_EQ(snap.committed_timestamps_.size(), 0);
 
   // Read the clock a few times to advance the timestamp
-  for (int i = 0; i < 10; i++) clock_->Now();
+  for (int i = 0; i < 10; i++) {
+    clock_.Now();
+  }
 
-  // Advance the safe timestamp.
-  Timestamp new_safe_time = clock_->Now();
-  mgr.AdjustSafeTime(new_safe_time);
+  // Advance the new transaction lower bound.
+  Timestamp new_ts_lower_bound = clock_.Now();
+  mgr.AdjustNewTransactionLowerBound(new_ts_lower_bound);
 
-  // Test that the snapshot reports that a timestamp lower than the safe time
-  // may have be committed transaction before that timestamp.
-  // Conversely, test that the snapshot reports that a timestamp greater
-  // than the safe time (and thus greater than none_committed_at_or_after_'
-  // as there are no other transactions committed) cannot have any committed
-  // transactions after that timestamp.
+  // Test that the snapshot reports that a timestamp lower than the new
+  // transaction lower bound may have committed transactions after that
+  // timestamp. Conversely, test that the snapshot reports that there are no
+  // committed transactions at or after the new lower bound.
   MvccSnapshot snap2;
-  mgr.TakeSnapshot(&snap2);
-  Timestamp before_safe_time(new_safe_time.value() - 1);
-  Timestamp after_safe_time(new_safe_time.value() + 1);
-  EXPECT_TRUE(snap2.MayHaveCommittedTransactionsAtOrAfter(before_safe_time));
-  EXPECT_FALSE(snap2.MayHaveCommittedTransactionsAtOrAfter(after_safe_time));
+  snap2 = MvccSnapshot(mgr);
+  Timestamp before_lb(new_ts_lower_bound.value() - 1);
+  Timestamp after_lb(new_ts_lower_bound.value() + 1);
+  EXPECT_TRUE(snap2.MayHaveCommittedTransactionsAtOrAfter(before_lb));
+  EXPECT_FALSE(snap2.MayHaveCommittedTransactionsAtOrAfter(after_lb));
 
-  EXPECT_EQ(snap2.all_committed_before_, new_safe_time);
-  EXPECT_EQ(snap2.none_committed_at_or_after_, new_safe_time);
+  EXPECT_EQ(snap2.all_committed_before_, new_ts_lower_bound);
+  EXPECT_EQ(snap2.none_committed_at_or_after_, new_ts_lower_bound);
   EXPECT_EQ(snap2.committed_timestamps_.size(), 0);
 }
 
