@@ -24,6 +24,7 @@
 #include <memory>
 #include <ostream>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <boost/optional/optional.hpp>
@@ -43,7 +44,6 @@
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/stl_util.h"
@@ -748,6 +748,53 @@ TYPED_TEST(TestTablet, TestInsertsPersist) {
   // TODO(unknown): add some more data, re-flush
 }
 
+TYPED_TEST(TestTablet, TestInsertIgnore) {
+  LocalTabletWriter writer(this->tablet().get(), &this->client_schema_);
+  KuduPartialRow row(&this->client_schema_);
+  vector<string> rows;
+
+  // Single batch, insert then insert ingore of same row, operation should succeed
+  this->setup_.BuildRow(&row, 0, 1000);
+  vector<LocalTabletWriter::Op> ops;
+  ops.emplace_back(LocalTabletWriter::Op(RowOperationsPB::INSERT, &row));
+  ops.emplace_back(LocalTabletWriter::Op(RowOperationsPB::INSERT_IGNORE, &row));
+  ASSERT_OK(writer.WriteBatch(ops));
+  ASSERT_OK(this->IterateToStringList(&rows));
+  ASSERT_EQ(1, rows.size());
+  EXPECT_EQ(vector<string>{ this->setup_.FormatDebugRow(0, 1000, false) }, rows);
+
+  ASSERT_OK(this->DeleteTestRow(&writer, 0));
+
+  ops.clear();
+  this->setup_.BuildRow(&row, 0, 1001);
+  ops.emplace_back(LocalTabletWriter::Op(RowOperationsPB::INSERT_IGNORE, &row));
+  ops.emplace_back(LocalTabletWriter::Op(RowOperationsPB::INSERT, &row));
+  Status s = writer.WriteBatch(ops);
+  ASSERT_STR_CONTAINS(s.ToString(), "key already present");
+  ASSERT_OK(this->IterateToStringList(&rows));
+  ASSERT_EQ(1, rows.size());
+  EXPECT_EQ(vector<string>{ this->setup_.FormatDebugRow(0, 1001, false) }, rows);
+
+  // INSERT IGNORE a row that is in MRS, ensure value doesn't change
+  this->InsertIgnoreTestRows(0, 1, 2000);
+  ASSERT_OK(this->IterateToStringList(&rows));
+  ASSERT_EQ(1, rows.size());
+  EXPECT_EQ(vector<string>{ this->setup_.FormatDebugRow(0, 1001, false) }, rows);
+
+  this->UpsertTestRows(0, 1, 1011);
+
+  // Flush it.
+  ASSERT_OK(this->tablet()->Flush());
+  ASSERT_OK(this->IterateToStringList(&rows));
+  EXPECT_EQ(vector<string>{ this->setup_.FormatDebugRow(0, 1011, false) }, rows);
+
+  // INSERT IGNORE a row that is in DRS, ensure value doesn't change.
+  this->InsertIgnoreTestRows(0, 1, 1000);
+  ASSERT_OK(this->IterateToStringList(&rows));
+  ASSERT_EQ(1, rows.size());
+  EXPECT_EQ(vector<string>{ this->setup_.FormatDebugRow(0, 1011, false) }, rows);
+}
+
 TYPED_TEST(TestTablet, TestUpsert) {
   vector<string> rows;
   const auto& upserts_as_updates = this->tablet()->metrics()->upserts_as_updates;
@@ -776,7 +823,6 @@ TYPED_TEST(TestTablet, TestUpsert) {
   NO_FATALS(this->CheckLiveRowsCount(1));
 }
 
-
 // Test that when a row has been updated many times, it always yields
 // the most recent value.
 TYPED_TEST(TestTablet, TestMultipleUpdates) {
@@ -1273,8 +1319,8 @@ TEST_F(TestTabletStringKey, TestSplitKeyRange) {
   }
   // test split key range with bound
   {
-    gscoped_ptr<EncodedKey> l_enc_key;
-    gscoped_ptr<EncodedKey> u_enc_key;
+    unique_ptr<EncodedKey> l_enc_key;
+    unique_ptr<EncodedKey> u_enc_key;
     Arena arena(256);
     KuduPartialRow lower_bound(&this->schema_);
     CHECK_OK(lower_bound.SetString("key", "1"));
