@@ -22,7 +22,6 @@
 #include <ctime>
 #include <new>
 #include <ostream>
-#include <type_traits>
 #include <vector>
 
 #include <boost/optional/optional.hpp>
@@ -138,7 +137,7 @@ WriteTransaction::WriteTransaction(unique_ptr<WriteTransactionState> state, Driv
   start_time_ = MonoTime::Now();
 }
 
-void WriteTransaction::NewReplicateMsg(gscoped_ptr<ReplicateMsg>* replicate_msg) {
+void WriteTransaction::NewReplicateMsg(unique_ptr<ReplicateMsg>* replicate_msg) {
   replicate_msg->reset(new ReplicateMsg);
   (*replicate_msg)->set_op_type(WRITE_OP);
   (*replicate_msg)->mutable_write_request()->CopyFrom(*state()->request());
@@ -155,7 +154,7 @@ Status WriteTransaction::Prepare() {
   RETURN_NOT_OK_PREPEND(SchemaFromPB(state_->request()->schema(), &client_schema),
                         "Cannot decode client schema");
   if (client_schema.has_column_ids()) {
-    // TODO: we have this kind of code a lot - add a new SchemaFromPB variant which
+    // TODO(unknown): we have this kind of code a lot - add a new SchemaFromPB variant which
     // does this check inline.
     Status s = Status::InvalidArgument("User requests should not have Column IDs");
     state_->completion_callback()->set_error(s, TabletServerErrorPB::INVALID_SCHEMA);
@@ -221,7 +220,7 @@ void WriteTransaction::UpdatePerRowErrors() {
 
 // FIXME: Since this is called as a void in a thread-pool callback,
 // it seems pointless to return a Status!
-Status WriteTransaction::Apply(gscoped_ptr<CommitMsg>* commit_msg) {
+Status WriteTransaction::Apply(unique_ptr<CommitMsg>* commit_msg) {
   TRACE_EVENT0("txn", "WriteTransaction::Apply");
   TRACE("APPLY: Starting.");
 
@@ -262,9 +261,10 @@ void WriteTransaction::Finish(TransactionResult result) {
 
   TabletMetrics* metrics = state_->tablet_replica()->tablet()->metrics();
   if (metrics) {
-    // TODO: should we change this so it's actually incremented by the
+    // TODO(unknown): should we change this so it's actually incremented by the
     // Tablet code itself instead of this wrapper code?
     metrics->rows_inserted->IncrementBy(state_->metrics().successful_inserts);
+    metrics->insert_ignore_errors->IncrementBy(state_->metrics().insert_ignore_errors);
     metrics->rows_upserted->IncrementBy(state_->metrics().successful_upserts);
     metrics->rows_updated->IncrementBy(state_->metrics().successful_updates);
     metrics->rows_deleted->IncrementBy(state_->metrics().successful_deletes);
@@ -319,7 +319,7 @@ WriteTransactionState::WriteTransactionState(TabletReplica* tablet_replica,
   }
 }
 
-void WriteTransactionState::SetMvccTx(gscoped_ptr<ScopedTransaction> mvcc_tx) {
+void WriteTransactionState::SetMvccTx(unique_ptr<ScopedTransaction> mvcc_tx) {
   DCHECK(!mvcc_tx_) << "Mvcc transaction already started/set.";
   mvcc_tx_ = std::move(mvcc_tx);
 }
@@ -417,6 +417,13 @@ void WriteTransactionState::UpdateMetricsForOp(const RowOp& op) {
     case RowOperationsPB::INSERT:
       tx_metrics_.successful_inserts++;
       break;
+    case RowOperationsPB::INSERT_IGNORE:
+      if (op.error_ignored) {
+        tx_metrics_.insert_ignore_errors++;
+      } else {
+        tx_metrics_.successful_inserts++;
+      }
+      break;
     case RowOperationsPB::UPSERT:
       tx_metrics_.successful_upserts++;
       break;
