@@ -19,7 +19,9 @@
 
 #include <algorithm>
 #include <atomic>
+#include <cstddef>
 #include <cstdint>
+#include <initializer_list>
 #include <map>
 #include <memory>
 #include <numeric>
@@ -34,6 +36,7 @@
 #include <boost/optional/optional.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
+#include <google/protobuf/stubs/common.h>
 #include <gtest/gtest.h>
 #include <rapidjson/document.h>
 
@@ -57,6 +60,7 @@ DECLARE_bool(checksum_scan);
 DECLARE_int32(checksum_idle_timeout_sec);
 DECLARE_int32(max_progress_report_wait_ms);
 DECLARE_string(color);
+DECLARE_string(flags_categories_to_check);
 DECLARE_string(ksck_format);
 DECLARE_uint32(truncate_server_csv_length);
 
@@ -83,12 +87,17 @@ namespace tools {
 
 class MockKsckMaster : public KsckMaster {
  public:
-  explicit MockKsckMaster(const string& address, const string& uuid)
+  explicit MockKsckMaster(const string& address, const string& uuid, bool is_get_flags_available)
       : KsckMaster(address),
-        fetch_info_status_(Status::OK()) {
+        fetch_info_status_(Status::OK()),
+        is_get_flags_available_(is_get_flags_available) {
     uuid_ = uuid;
     version_ = "mock-version";
-    flags_ = GetFlagsResponsePB{};
+    if (is_get_flags_available_) {
+      for (size_t cat = FlagsCategory::MIN; cat <= FlagsCategory::MAX; ++cat) {
+        flags_by_category_[cat].flags.emplace();
+      }
+    }
   }
 
   Status Init() override {
@@ -108,9 +117,16 @@ class MockKsckMaster : public KsckMaster {
     return fetch_cstate_status_;
   }
 
-  Status FetchUnusualFlags() override {
-    flags_state_ = KsckFetchState::FETCHED;
-    return Status::OK();
+  Status FetchFlags(const std::vector<FlagsCategory>& categories) override {
+    for (const auto cat : categories) {
+      if (is_get_flags_available_) {
+        flags_by_category_[cat].state = KsckFetchState::FETCHED;
+      } else {
+        flags_by_category_[cat].state = KsckFetchState::FETCH_FAILED;
+      }
+    }
+    return is_get_flags_available_
+        ? Status::OK() : Status::RemoteError("GetFlags not available");
   }
 
   // Public because the unit tests mutate these variables directly.
@@ -118,19 +134,26 @@ class MockKsckMaster : public KsckMaster {
   Status fetch_cstate_status_;
   using KsckMaster::uuid_;
   using KsckMaster::cstate_;
-  using KsckMaster::flags_;
+  using KsckMaster::flags_by_category_;
   using KsckMaster::version_;
+ private:
+  const bool is_get_flags_available_;
 };
 
 class MockKsckTabletServer : public KsckTabletServer {
  public:
-  explicit MockKsckTabletServer(const string& uuid)
+  explicit MockKsckTabletServer(const string& uuid, bool is_get_flags_available)
       : KsckTabletServer(uuid),
         fetch_info_status_(Status::OK()),
         fetch_info_health_(ServerHealth::HEALTHY),
-        address_("<mock>") {
+        address_("<mock>"),
+        is_get_flags_available_(is_get_flags_available) {
     version_ = "mock-version";
-    flags_ = GetFlagsResponsePB{};
+    if (is_get_flags_available_) {
+      for (size_t cat = FlagsCategory::MIN; cat <= FlagsCategory::MAX; ++cat) {
+        flags_by_category_[cat].flags.emplace();
+      }
+    }
   }
 
   Status FetchInfo(ServerHealth* health) override {
@@ -149,9 +172,16 @@ class MockKsckTabletServer : public KsckTabletServer {
     return Status::OK();
   }
 
-  Status FetchUnusualFlags() override {
-    flags_state_ = KsckFetchState::FETCHED;
-    return Status::OK();
+  Status FetchFlags(const std::vector<FlagsCategory>& categories) override {
+    for (const auto cat : categories) {
+      if (is_get_flags_available_) {
+        flags_by_category_[cat].state = KsckFetchState::FETCHED;
+      } else {
+        flags_by_category_[cat].state = KsckFetchState::FETCH_FAILED;
+      }
+    }
+    return is_get_flags_available_
+        ? Status::OK() : Status::RemoteError("GetFlags not available");
   }
 
   void FetchCurrentTimestampAsync() override {}
@@ -160,6 +190,8 @@ class MockKsckTabletServer : public KsckTabletServer {
     return Status::OK();
   }
 
+  void FetchQuiescingInfo() override {}
+
   void RunTabletChecksumScanAsync(
       const std::string& tablet_id,
       const Schema& /*schema*/,
@@ -183,12 +215,13 @@ class MockKsckTabletServer : public KsckTabletServer {
   // The fake progress amount for this mock server, used to mock checksum
   // progress for this server.
   int64_t checksum_progress_ = 10;
-  using KsckTabletServer::flags_;
+  using KsckTabletServer::flags_by_category_;
   using KsckTabletServer::location_;
   using KsckTabletServer::version_;
 
  private:
   const string address_;
+  const bool is_get_flags_available_;
 };
 
 class MockKsckCluster : public KsckCluster {
@@ -230,7 +263,9 @@ class KsckTest : public KuduTest {
       : cluster_(new MockKsckCluster()),
         ksck_(new Ksck(cluster_, &err_stream_)) {
     FLAGS_color = "never";
+  }
 
+  void SetUp() override {
     // Set up the master consensus state.
     consensus::ConsensusStatePB cstate;
     cstate.set_current_term(0);
@@ -244,7 +279,7 @@ class KsckTest : public KuduTest {
     for (int i = 0; i < 3; i++) {
       const string uuid = Substitute("master-id-$0", i);
       const string addr = Substitute("master-$0", i);
-      shared_ptr<MockKsckMaster> master = std::make_shared<MockKsckMaster>(addr, uuid);
+      auto master = std::make_shared<MockKsckMaster>(addr, uuid, IsGetFlagsAvailable());
       master->cstate_ = cstate;
       cluster_->masters_.push_back(master);
     }
@@ -252,7 +287,7 @@ class KsckTest : public KuduTest {
     KsckCluster::TSMap tablet_servers;
     for (int i = 0; i < 3; i++) {
       string name = Substitute("ts-id-$0", i);
-      shared_ptr<MockKsckTabletServer> ts(new MockKsckTabletServer(name));
+      auto ts = std::make_shared<MockKsckTabletServer>(name, IsGetFlagsAvailable());
       InsertOrDie(&tablet_servers, ts->uuid(), ts);
     }
     cluster_->tablet_servers_.swap(tablet_servers);
@@ -425,6 +460,10 @@ class KsckTest : public KuduTest {
     return json_stream.str();
   }
 
+  virtual bool IsGetFlagsAvailable() const {
+    return true;
+  }
+
   shared_ptr<MockKsckCluster> cluster_;
   shared_ptr<Ksck> ksck_;
   // This is used as a stack. First the unit test is responsible to create a plan to follow, that
@@ -437,6 +476,13 @@ class KsckTest : public KuduTest {
   std::ostringstream err_stream_;
 };
 
+class GetFlagsUnavailableKsckTest : public KsckTest {
+ protected:
+  bool IsGetFlagsAvailable() const override {
+    return false;
+  }
+};
+
 // Helpful macros for checking JSON fields vs. expected values.
 // In all cases, the meaning of the parameters are as follows:
 // 'reader' is the JsonReader that owns the parsed JSON data.
@@ -887,6 +933,12 @@ void CheckJsonStringVsKsckResults(const string& json,
   CheckJsonVsErrors(r, "errors", results.error_messages);
 }
 
+void CheckMessageNotPresent(const vector<Status>& messages, const string& msg) {
+  for (const auto& status : messages) {
+    ASSERT_STR_NOT_CONTAINS(status.ToString(), msg);
+  }
+}
+
 TEST_F(KsckTest, TestServersOk) {
   ASSERT_OK(RunKsck());
   const string err_string = err_stream_.str();
@@ -1032,6 +1084,9 @@ TEST_F(KsckTest, TestLeaderMasterUnavailable) {
 }
 
 TEST_F(KsckTest, TestMasterFlagCheck) {
+  // Check for the differences in the 'unusual' flags category.
+  FLAGS_flags_categories_to_check = "unusual";
+
   // Setup flags for each mock master.
   for (int i = 0; i < cluster_->masters().size(); i++) {
     server::GetFlagsResponsePB flags;
@@ -1060,12 +1115,14 @@ TEST_F(KsckTest, TestMasterFlagCheck) {
     }
     shared_ptr<MockKsckMaster> master =
         std::static_pointer_cast<MockKsckMaster>(cluster_->masters_.at(i));
-    master->flags_ = flags;
+    master->flags_by_category_[FlagsCategory::UNUSUAL].flags = std::move(flags);
   }
   ASSERT_OK(ksck_->CheckMasterHealth());
   ASSERT_OK(ksck_->CheckMasterUnusualFlags());
+  ASSERT_OK(ksck_->CheckMasterDivergedFlags());
   ASSERT_OK(ksck_->PrintResults());
   ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Unusual flags for Master:\n"
       "        Flag        | Value |     Tags      |         Master\n"
       "--------------------+-------+---------------+-------------------------\n"
       " experimental_flag  | x     | experimental  | all 3 server(s) checked\n"
@@ -1074,6 +1131,30 @@ TEST_F(KsckTest, TestMasterFlagCheck) {
       " hidden_flag        | 2     | hidden        | master-2\n"
       " hidden_unsafe_flag | 0     | hidden,unsafe | master-0, master-2\n"
       " hidden_unsafe_flag | 1     | hidden,unsafe | master-1");
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Some masters have unsafe, experimental, or hidden flags set");
+
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Flags of checked categories for Master:\n"
+      "        Flag        | Value |         Master\n"
+      "--------------------+-------+-------------------------\n"
+      " experimental_flag  | x     | all 3 server(s) checked\n"
+      " hidden_flag        | 0     | master-0\n"
+      " hidden_flag        | 1     | master-1\n"
+      " hidden_flag        | 2     | master-2\n"
+      " hidden_unsafe_flag | 0     | master-0, master-2\n"
+      " hidden_unsafe_flag | 1     | master-1");
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Different masters have different settings for same flags "
+      "of checked category 'unusual'");
+}
+
+TEST_F(GetFlagsUnavailableKsckTest, TestMasterFlagsUnavailable) {
+  ASSERT_OK(ksck_->CheckMasterHealth());
+  ASSERT_TRUE(ksck_->CheckMasterUnusualFlags().IsIncomplete());
+
+  static const string flags_msg = "unable to get flag information for master";
+  CheckMessageNotPresent(ksck_->results().warning_messages, flags_msg);
 }
 
 TEST_F(KsckTest, TestWrongUUIDTabletServer) {
@@ -1156,6 +1237,9 @@ TEST_F(KsckTest, TestTserverFlagCheck) {
   // Lower the truncation threshold to test truncation.
   FLAGS_truncate_server_csv_length = 1;
 
+  // Check for the differences in the 'unusual' flags category.
+  FLAGS_flags_categories_to_check = "unusual";
+
   // Setup flags for each mock tablet server.
   int i = 0;
   for (const auto& entry : cluster_->tablet_servers()) {
@@ -1185,13 +1269,15 @@ TEST_F(KsckTest, TestTserverFlagCheck) {
     }
     shared_ptr<MockKsckTabletServer> ts =
         std::static_pointer_cast<MockKsckTabletServer>(entry.second);
-    ts->flags_ = flags;
+    ts->flags_by_category_[FlagsCategory::UNUSUAL].flags = std::move(flags);
     i++;
   }
   ASSERT_OK(ksck_->FetchInfoFromTabletServers());
   ASSERT_OK(ksck_->CheckTabletServerUnusualFlags());
+  ASSERT_OK(ksck_->CheckTabletServerDivergedFlags());
   ASSERT_OK(ksck_->PrintResults());
   ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Unusual flags for Tablet Server:\n"
       "        Flag        | Value |     Tags      |         Tablet Server\n"
       "--------------------+-------+---------------+-------------------------------\n"
       " experimental_flag  | x     | experimental  | all 3 server(s) checked\n"
@@ -1200,6 +1286,138 @@ TEST_F(KsckTest, TestTserverFlagCheck) {
       " hidden_flag        | 2     | hidden        | <mock>\n"
       " hidden_unsafe_flag | 0     | hidden,unsafe | <mock>, and 1 other server(s)\n"
       " hidden_unsafe_flag | 1     | hidden,unsafe | <mock>");
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Some tablet servers have unsafe, experimental, or hidden flags set");
+
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Flags of checked categories for Tablet Server:\n"
+      "        Flag        | Value |         Tablet Server\n"
+      "--------------------+-------+-------------------------------\n"
+      " experimental_flag  | x     | all 3 server(s) checked\n"
+      " hidden_flag        | 0     | <mock>\n"
+      " hidden_flag        | 1     | <mock>\n"
+      " hidden_flag        | 2     | <mock>\n"
+      " hidden_unsafe_flag | 0     | <mock>, and 1 other server(s)\n"
+      " hidden_unsafe_flag | 1     | <mock>");
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Different tablet servers have different settings for same flags "
+      "of checked category 'unusual'");
+}
+
+TEST_F(KsckTest, FlagsCategoriesDifferenceBetweenMastersAndTabletServers) {
+  // Check for the differences in the 'time_source' flags category.
+  FLAGS_flags_categories_to_check = "time_source";
+
+  // Setup flags for mock masters.
+  for (const auto& master : cluster_->masters()) {
+    shared_ptr<MockKsckMaster> m =
+        std::static_pointer_cast<MockKsckMaster>(master);
+    // Set two flags in the 'time_source' category.
+    {
+      server::GetFlagsResponsePB flags;
+      {
+        auto* flag = flags.add_flags();
+        flag->set_name("time_source");
+        flag->set_value("builtin");
+      }
+      {
+        auto* flag = flags.add_flags();
+        flag->set_name("builtin_ntp_servers");
+        flag->set_value("mega.turbo.ntp");
+      }
+      m->flags_by_category_[FlagsCategory::TIME_SOURCE].flags = std::move(flags);
+    }
+
+    // Set a flag unrelated to the checked category.
+    {
+      server::GetFlagsResponsePB flags;
+      {
+        auto* flag = flags.add_flags();
+        flag->set_name("giga");
+        flag->set_value("hertz");
+      }
+      m->flags_by_category_[FlagsCategory::UNUSUAL].flags = std::move(flags);
+    }
+  }
+
+  // Setup flags for mock tablet servers.
+  for (const auto& entry : cluster_->tablet_servers()) {
+    shared_ptr<MockKsckTabletServer> ts =
+        std::static_pointer_cast<MockKsckTabletServer>(entry.second);
+    // Set one flag in the 'time_source' category.
+    {
+      server::GetFlagsResponsePB flags;
+      {
+        auto* flag = flags.add_flags();
+        flag->set_name("time_source");
+        flag->set_value("system");
+      }
+      ts->flags_by_category_[FlagsCategory::TIME_SOURCE].flags = std::move(flags);
+    }
+
+    // Set a flag unrelated to the 'time_source' category.
+    {
+      server::GetFlagsResponsePB flags;
+      {
+        auto* flag = flags.add_flags();
+        flag->set_name("foo");
+        flag->set_value("bar");
+      }
+      ts->flags_by_category_[FlagsCategory::UNUSUAL].flags = std::move(flags);
+    }
+  }
+
+  // Calling CheckMasterHealth() is a prerequisite for calling
+  // CheckMasterUnusualFlags().
+  ASSERT_OK(ksck_->CheckMasterHealth());
+  ASSERT_OK(ksck_->CheckMasterDivergedFlags());
+  // Calling FetchInfoFromTabletServers() is a prerequisite for calling
+  // CheckTabletServerDivergedFlags().
+  ASSERT_OK(ksck_->FetchInfoFromTabletServers());
+  ASSERT_OK(ksck_->CheckTabletServerDivergedFlags());
+  ASSERT_OK(ksck_->CheckDivergedFlags());
+  ASSERT_OK(ksck_->PrintResults());
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Flags of checked categories for Master:\n"
+      "        Flag         |     Value      |         Master\n"
+      "---------------------+----------------+-------------------------\n"
+      " builtin_ntp_servers | mega.turbo.ntp | all 3 server(s) checked\n"
+      " time_source         | builtin        | all 3 server(s) checked\n");
+  ASSERT_STR_NOT_CONTAINS(err_stream_.str(),
+      "Different masters have different settings for same flags "
+      "of checked category 'time_source'");
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Flags of checked categories for Master diverging from Tablet Server flags:\n"
+      "        Flag         |     Value      |         Master\n"
+      "---------------------+----------------+-------------------------\n"
+      " builtin_ntp_servers | mega.turbo.ntp | all 3 server(s) checked\n"
+      " time_source         | builtin        | all 3 server(s) checked");
+
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Flags of checked categories for Tablet Server:\n"
+      "    Flag     | Value  |      Tablet Server\n"
+      "-------------+--------+-------------------------\n"
+      " time_source | system | all 3 server(s) checked\n");
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+      "Flags of checked categories for Tablet Server diverging from Master flags:\n"
+      "    Flag     | Value  |      Tablet Server\n"
+      "-------------+--------+-------------------------\n"
+      " time_source | system | all 3 server(s) checked");
+  ASSERT_STR_NOT_CONTAINS(err_stream_.str(),
+      "Different tablet servers have different settings for same flags "
+      "of checked category 'time_source'");
+
+  ASSERT_STR_CONTAINS(err_stream_.str(),
+        "Same flags have different values between masters and tablet servers "
+        "for at least one checked flag category");
+}
+
+TEST_F(GetFlagsUnavailableKsckTest, TestTserverFlagsUnavailable) {
+  ASSERT_OK(ksck_->FetchInfoFromTabletServers());
+  ASSERT_TRUE(ksck_->CheckTabletServerUnusualFlags().IsIncomplete());
+
+  static const string flags_msg = "unable to get flag information for tablet server";
+  CheckMessageNotPresent(ksck_->results().warning_messages, flags_msg);
 }
 
 TEST_F(KsckTest, TestOneTableCheck) {
@@ -1252,7 +1470,7 @@ TEST_F(KsckTest, TestConsensusConflictExtraPeer) {
             error_messages[0].ToString());
   const string err_str = err_stream_.str();
   ASSERT_STR_CONTAINS(err_str, "Tablet tablet-id-0 of table 'test' is conflicted: "
-                               "3 replicas' active configs disagree with the leader master's");
+                               "1 replicas' active configs disagree with the leader master's");
   ASSERT_STR_CONTAINS(err_str,
       "The consensus matrix is:\n"
       " Config source |     Replicas     | Current term | Config index | Committed?\n"
@@ -1613,7 +1831,7 @@ TEST_F(KsckTest, TestChecksumWithAllPeersUnhealthy) {
   const char* const new_uuid = "new";
   EmplaceOrDie(&cluster_->tablet_servers_,
                new_uuid,
-               std::make_shared<MockKsckTabletServer>(new_uuid));
+               std::make_shared<MockKsckTabletServer>(new_uuid, IsGetFlagsAvailable()));
 
   // The checksum should fail for tablet because none of its replicas are
   // available to provide a timestamp.
@@ -1674,5 +1892,6 @@ TEST_F(KsckTest, TestSectionFilter) {
     CheckJsonStringVsKsckResults(json_output, ksck_->results(), selected_sections);
   }
 }
+
 } // namespace tools
 } // namespace kudu
