@@ -18,10 +18,9 @@
 #include "kudu/tools/tool_action_common.h"
 
 #include <stdlib.h>
-#include <unistd.h>
 
 #include <algorithm>
-#include <cerrno>
+#include <cstddef>
 #include <iomanip>
 #include <iostream>
 #include <iterator>
@@ -30,19 +29,19 @@
 #include <stack>
 #include <string>
 #include <unordered_map>
+#include <utility>
 #include <vector>
 
 #include <boost/algorithm/string/predicate.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
-#include <google/protobuf/util/json_util.h>
 // IWYU pragma: no_include <yaml-cpp/node/impl.h>
 // IWYU pragma: no_include <yaml-cpp/node/node.h>
 
 #include "kudu/client/client-internal.h"  // IWYU pragma: keep
 #include "kudu/client/client.h"
 #include "kudu/client/master_proxy_rpc.h"
-#include "kudu/client/shared_ptr.h"
+#include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/common/common.pb.h"
 #include "kudu/common/row_operations.h"
 #include "kudu/common/schema.h"
@@ -52,7 +51,6 @@
 #include "kudu/consensus/log.pb.h"
 #include "kudu/consensus/log_util.h"
 #include "kudu/consensus/opid.pb.h"
-#include "kudu/gutil/endian.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/join.h"
@@ -76,7 +74,6 @@
 #include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/async_util.h"
 #include "kudu/util/env.h"
-#include "kudu/util/faststring.h"
 #include "kudu/util/jsonwriter.h"
 #include "kudu/util/mem_tracker.pb.h"
 #include "kudu/util/memory/arena.h"
@@ -118,6 +115,9 @@ DEFINE_string(flag_tags, "", "Comma-separated list of tags used to restrict whic
                              "flags are returned. An empty value matches all tags");
 DEFINE_bool(all_flags, false, "Whether to return all flags, or only flags that "
                               "were explicitly set.");
+DEFINE_string(flags, "", "Comma-separated list of flags used to restrict which "
+                         "flags are returned. An empty value means no restriction. "
+                         "If non-empty, all_flags is ignored.");
 DEFINE_string(tables, "", "Tables to include (comma-separated list of table names)"
                           "If not specified, includes all tables.");
 
@@ -180,11 +180,6 @@ using std::vector;
 using strings::Split;
 using strings::Substitute;
 
-namespace boost {
-template <typename Signature>
-class function;
-} // namespace boost
-
 namespace kudu {
 namespace tools {
 
@@ -412,6 +407,7 @@ Status PrintSegment(const scoped_refptr<ReadableLogSegment>& segment) {
 Status GetServerFlags(const string& address,
                       uint16_t default_port,
                       bool all_flags,
+                      const string& flags_to_get,
                       const string& flag_tags,
                       vector<server::GetFlagsResponsePB_Flag>* flags) {
   unique_ptr<GenericServiceProxy> proxy;
@@ -426,6 +422,10 @@ Status GetServerFlags(const string& address,
   for (StringPiece tag : strings::Split(flag_tags, ",", strings::SkipEmpty())) {
     req.add_tags(tag.as_string());
   }
+  for (StringPiece flag: strings::Split(flags_to_get, ",", strings::SkipEmpty())) {
+    req.add_flags(flag.as_string());
+  }
+
   RETURN_NOT_OK(proxy->GetFlags(req, &resp, &rpc));
 
   flags->clear();
@@ -435,11 +435,12 @@ Status GetServerFlags(const string& address,
 
 Status PrintServerFlags(const string& address, uint16_t default_port) {
   vector<server::GetFlagsResponsePB_Flag> flags;
-  RETURN_NOT_OK(GetServerFlags(address, default_port, FLAGS_all_flags, FLAGS_flag_tags, &flags));
+  RETURN_NOT_OK(GetServerFlags(address, default_port, FLAGS_all_flags,
+      FLAGS_flags, FLAGS_flag_tags, &flags));
 
   std::sort(flags.begin(), flags.end(),
       [](const GetFlagsResponsePB::Flag& left,
-         const GetFlagsResponsePB::Flag& right) -> bool {
+         const GetFlagsResponsePB::Flag& right) {
         return left.name() < right.name();
       });
   DataTable table({ "flag", "value", "default value?", "tags" });
@@ -796,10 +797,10 @@ template<typename Req, typename Resp>
 Status LeaderMasterProxy::SyncRpc(const Req& req,
                                   Resp* resp,
                                   string func_name,
-                                  const boost::function<void(master::MasterServiceProxy*,
-                                                             const Req&, Resp*,
-                                                             rpc::RpcController*,
-                                                             const ResponseCallback&)>& func) {
+                                  const std::function<void(master::MasterServiceProxy*,
+                                                           const Req&, Resp*,
+                                                           rpc::RpcController*,
+                                                           const ResponseCallback&)>& func) {
   MonoTime deadline = MonoTime::Now() + MonoDelta::FromMilliseconds(FLAGS_timeout_ms);
   Synchronizer sync;
   AsyncLeaderMasterRpc<Req, Resp> rpc(deadline, client_.get(), BackoffType::EXPONENTIAL,
@@ -814,209 +815,41 @@ Status LeaderMasterProxy::SyncRpc(
     const master::ChangeTServerStateRequestPB& req,
     master::ChangeTServerStateResponsePB* resp,
     string func_name,
-    const boost::function<void(MasterServiceProxy*,
-                               const master::ChangeTServerStateRequestPB&,
-                               master::ChangeTServerStateResponsePB*,
-                               RpcController*,
-                               const ResponseCallback&)>& func);
+    const std::function<void(MasterServiceProxy*,
+                             const master::ChangeTServerStateRequestPB&,
+                             master::ChangeTServerStateResponsePB*,
+                             RpcController*,
+                             const ResponseCallback&)>& func);
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ListTabletServersRequestPB& req,
     master::ListTabletServersResponsePB* resp,
     string func_name,
-    const boost::function<void(MasterServiceProxy*,
-                               const master::ListTabletServersRequestPB&,
-                               master::ListTabletServersResponsePB*,
-                               RpcController*,
-                               const ResponseCallback&)>& func);
+    const std::function<void(MasterServiceProxy*,
+                             const master::ListTabletServersRequestPB&,
+                             master::ListTabletServersResponsePB*,
+                             RpcController*,
+                             const ResponseCallback&)>& func);
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ListMastersRequestPB& req,
     master::ListMastersResponsePB* resp,
     string func_name,
-    const boost::function<void(MasterServiceProxy*,
-                               const master::ListMastersRequestPB&,
-                               master::ListMastersResponsePB*,
-                               RpcController*,
-                               const ResponseCallback&)>& func);
+    const std::function<void(MasterServiceProxy*,
+                             const master::ListMastersRequestPB&,
+                             master::ListMastersResponsePB*,
+                             RpcController*,
+                             const ResponseCallback&)>& func);
 template
 Status LeaderMasterProxy::SyncRpc(
     const master::ReplaceTabletRequestPB& req,
     master::ReplaceTabletResponsePB* resp,
     string func_name,
-    const boost::function<void(MasterServiceProxy*,
-                               const master::ReplaceTabletRequestPB&,
-                               master::ReplaceTabletResponsePB*,
-                               RpcController*,
-                               const ResponseCallback&)>& func);
-
-const int ControlShellProtocol::kMaxMessageBytes = 1024 * 1024;
-
-ControlShellProtocol::ControlShellProtocol(SerializationMode serialization_mode,
-                                           CloseMode close_mode,
-                                           int read_fd,
-                                           int write_fd)
-    : serialization_mode_(serialization_mode),
-      close_mode_(close_mode),
-      read_fd_(read_fd),
-      write_fd_(write_fd) {
-}
-
-ControlShellProtocol::~ControlShellProtocol() {
-  if (close_mode_ == CloseMode::CLOSE_ON_DESTROY) {
-    int ret;
-    RETRY_ON_EINTR(ret, close(read_fd_));
-    RETRY_ON_EINTR(ret, close(write_fd_));
-  }
-}
-
-template <class M>
-Status ControlShellProtocol::ReceiveMessage(M* message) {
-  switch (serialization_mode_) {
-    case SerializationMode::JSON:
-    {
-      // Read and accumulate one byte at a time, looking for the newline.
-      //
-      // TODO(adar): it would be more efficient to read a chunk of data, look
-      // for a newline, and if found, store the remainder for the next message.
-      faststring buf;
-      faststring one_byte;
-      one_byte.resize(1);
-      while (true) {
-        RETURN_NOT_OK_PREPEND(DoRead(&one_byte), "unable to receive message byte");
-        if (one_byte[0] == '\n') {
-          break;
-        }
-        buf.push_back(one_byte[0]);
-      }
-
-      // Parse the JSON-encoded message.
-      const auto& google_status =
-          google::protobuf::util::JsonStringToMessage(buf.ToString(), message);
-      if (!google_status.ok()) {
-        return Status::InvalidArgument(
-            Substitute("unable to parse JSON: $0", buf.ToString()),
-            google_status.error_message().ToString());
-      }
-      break;
-    }
-    case SerializationMode::PB:
-    {
-      // Read four bytes of size (big-endian).
-      faststring size_buf;
-      size_buf.resize(sizeof(uint32_t));
-      RETURN_NOT_OK_PREPEND(DoRead(&size_buf), "unable to receive message size");
-      uint32_t body_size = NetworkByteOrder::Load32(size_buf.data());
-
-      if (body_size > kMaxMessageBytes) {
-        return Status::IOError(
-            Substitute("message size ($0) exceeds maximum message size ($1)",
-                       body_size, kMaxMessageBytes));
-      }
-
-      // Read the variable size body.
-      faststring body_buf;
-      body_buf.resize(body_size);
-      RETURN_NOT_OK_PREPEND(DoRead(&body_buf), "unable to receive message body");
-
-      // Parse the body into a PB request.
-      RETURN_NOT_OK_PREPEND(pb_util::ParseFromArray(
-          message, body_buf.data(), body_buf.length()),
-                            Substitute("unable to parse PB: $0", body_buf.ToString()));
-      break;
-    }
-    default: LOG(FATAL) << "Unknown mode";
-  }
-
-  VLOG(1) << "Received message: " << pb_util::SecureDebugString(*message);
-  return Status::OK();
-}
-
-template <class M>
-Status ControlShellProtocol::SendMessage(const M& message) {
-  VLOG(1) << "Sending message: " << pb_util::SecureDebugString(message);
-
-  faststring buf;
-  switch (serialization_mode_) {
-    case SerializationMode::JSON:
-    {
-      string serialized;
-      const auto& google_status =
-          google::protobuf::util::MessageToJsonString(message, &serialized);
-      if (!google_status.ok()) {
-        return Status::InvalidArgument(Substitute(
-            "unable to serialize JSON: $0", pb_util::SecureDebugString(message)),
-                                       google_status.error_message().ToString());
-      }
-
-      buf.append(serialized);
-      buf.append("\n");
-      break;
-    }
-    case SerializationMode::PB:
-    {
-      size_t msg_size = message.ByteSizeLong();
-      buf.resize(sizeof(uint32_t) + msg_size);
-      NetworkByteOrder::Store32(buf.data(), msg_size);
-      if (!message.SerializeWithCachedSizesToArray(buf.data() + sizeof(uint32_t))) {
-        return Status::Corruption("failed to serialize PB to array");
-      }
-      break;
-    }
-    default:
-      break;
-  }
-  RETURN_NOT_OK_PREPEND(DoWrite(buf), "unable to send message");
-  return Status::OK();
-}
-
-Status ControlShellProtocol::DoRead(faststring* buf) {
-  uint8_t* pos = buf->data();
-  size_t rem = buf->length();
-  while (rem > 0) {
-    ssize_t r;
-    RETRY_ON_EINTR(r, read(read_fd_, pos, rem));
-    if (r == -1) {
-      return Status::IOError("Error reading from pipe", "", errno);
-    }
-    if (r == 0) {
-      return Status::EndOfFile("Other end of pipe was closed");
-    }
-    DCHECK_GE(rem, r);
-    rem -= r;
-    pos += r;
-  }
-  return Status::OK();
-}
-
-Status ControlShellProtocol::DoWrite(const faststring& buf) {
-  const uint8_t* pos = buf.data();
-  size_t rem = buf.length();
-  while (rem > 0) {
-    ssize_t r;
-    RETRY_ON_EINTR(r, write(write_fd_, pos, rem));
-    if (r == -1) {
-      if (errno == EPIPE) {
-        return Status::EndOfFile("Other end of pipe was closed");
-      }
-      return Status::IOError("Error writing to pipe", "", errno);
-    }
-    DCHECK_GE(rem, r);
-    rem -= r;
-    pos += r;
-  }
-  return Status::OK();
-}
-
-// Explicit specialization for callers outside this compilation unit.
-template
-Status ControlShellProtocol::ReceiveMessage(ControlShellRequestPB* message);
-template
-Status ControlShellProtocol::ReceiveMessage(ControlShellResponsePB* message);
-template
-Status ControlShellProtocol::SendMessage(const ControlShellRequestPB& message);
-template
-Status ControlShellProtocol::SendMessage(const ControlShellResponsePB& message);
+    const std::function<void(MasterServiceProxy*,
+                             const master::ReplaceTabletRequestPB&,
+                             master::ReplaceTabletResponsePB*,
+                             RpcController*,
+                             const ResponseCallback&)>& func);
 
 } // namespace tools
 } // namespace kudu
