@@ -18,10 +18,12 @@
 #include <algorithm>
 #include <cstddef>
 #include <cstdint>
+#include <functional>
 #include <iostream>
 #include <map>
 #include <memory>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -50,6 +52,7 @@
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
 #include "kudu/fs/data_dirs.h"
+#include "kudu/fs/dir_manager.h"
 #include "kudu/fs/fs_manager.h"
 #include "kudu/fs/io_context.h"
 #include "kudu/gutil/map-util.h"
@@ -152,9 +155,11 @@ string Indent(int indent) {
   return string(indent, ' ');
 }
 
-Status FsInit(unique_ptr<FsManager>* fs_manager) {
+Status FsInit(bool skip_block_manager, unique_ptr<FsManager>* fs_manager) {
   FsManagerOpts fs_opts;
   fs_opts.read_only = true;
+  fs_opts.skip_block_manager = skip_block_manager;
+  fs_opts.update_instances = fs::UpdateInstanceBehavior::DONT_UPDATE;
   unique_ptr<FsManager> fs_ptr(new FsManager(Env::Default(), fs_opts));
   RETURN_NOT_OK(fs_ptr->Open());
   fs_manager->swap(fs_ptr);
@@ -185,10 +190,14 @@ Status ParseHostPortString(const string& hostport_str, HostPort* hostport) {
 Status FindLastLoggedOpId(FsManager* fs, const string& tablet_id,
                           OpId* last_logged_opid) {
   shared_ptr<LogReader> reader;
-  RETURN_NOT_OK(LogReader::Open(fs, scoped_refptr<log::LogIndex>(), tablet_id,
-                                scoped_refptr<MetricEntity>(), &reader));
+  RETURN_NOT_OK(LogReader::Open(fs,
+                                /*index*/nullptr,
+                                tablet_id,
+                                /*metric_entity*/nullptr,
+                                /*file_cache*/nullptr,
+                                &reader));
   SegmentSequence segs;
-  RETURN_NOT_OK(reader->GetSegmentsSnapshot(&segs));
+  reader->GetSegmentsSnapshot(&segs);
   // Reverse iterate the segments to find the 'last replicated' entry quickly.
   // Note that we still read the entries within a segment in sequential
   // fashion, so the last entry within the first 'found' segment will
@@ -230,7 +239,7 @@ Status ParsePeerString(const string& peer_str,
 
 Status PrintReplicaUuids(const RunnerContext& context) {
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/true, &fs_manager));
   scoped_refptr<ConsensusMetadataManager> cmeta_manager(
       new ConsensusMetadataManager(fs_manager.get()));
 
@@ -251,7 +260,7 @@ Status BackupConsensusMetadata(FsManager* fs_manager,
   string cmeta_filename = fs_manager->GetConsensusMetadataPath(tablet_id);
   string backup_filename = Substitute("$0.pre_rewrite.$1", cmeta_filename, env->NowMicros());
   WritableFileOptions opts;
-  opts.mode = Env::CREATE_NON_EXISTING;
+  opts.mode = Env::MUST_CREATE;
   opts.sync_on_close = true;
   RETURN_NOT_OK(env_util::CopyFile(env, cmeta_filename, backup_filename, opts));
   LOG(INFO) << "Backed up old consensus metadata to " << backup_filename;
@@ -278,7 +287,9 @@ Status RewriteRaftConfig(const RunnerContext& context) {
 
   // Make a copy of the old file before rewriting it.
   Env* env = Env::Default();
-  FsManager fs_manager(env, FsManagerOpts());
+  FsManagerOpts fs_opts = FsManagerOpts();
+  fs_opts.skip_block_manager = true;
+  FsManager fs_manager(env, std::move(fs_opts));
   RETURN_NOT_OK(fs_manager.Open());
   RETURN_NOT_OK(BackupConsensusMetadata(&fs_manager, tablet_id));
 
@@ -313,7 +324,9 @@ Status SetRaftTerm(const RunnerContext& context) {
 
   // Load the current metadata from disk and verify that the intended operation is safe.
   Env* env = Env::Default();
-  FsManager fs_manager(env, FsManagerOpts());
+  FsManagerOpts fs_opts = FsManagerOpts();
+  fs_opts.skip_block_manager = true;
+  FsManager fs_manager(env, fs_opts);
   RETURN_NOT_OK(fs_manager.Open());
   // Load the cmeta file and rewrite the raft config.
   scoped_refptr<ConsensusMetadataManager> cmeta_manager(new ConsensusMetadataManager(&fs_manager));
@@ -458,7 +471,7 @@ struct TabletSizeStats {
 Status SummarizeDataSize(const RunnerContext& context) {
   const string& tablet_id_pattern = FindOrDie(context.required_args, kTabletIdGlobArg);
   unique_ptr<FsManager> fs;
-  RETURN_NOT_OK(FsInit(&fs));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/false, &fs));
 
   vector<string> tablets;
   RETURN_NOT_OK(fs->ListTabletIds(&tablets));
@@ -515,18 +528,19 @@ Status SummarizeDataSize(const RunnerContext& context) {
 
 Status DumpWals(const RunnerContext& context) {
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/true, &fs_manager));
   const string& tablet_id = FindOrDie(context.required_args, kTabletIdArg);
 
   shared_ptr<LogReader> reader;
   RETURN_NOT_OK(LogReader::Open(fs_manager.get(),
-                                scoped_refptr<LogIndex>(),
+                                /*index*/nullptr,
                                 tablet_id,
-                                scoped_refptr<MetricEntity>(),
+                                /*metric_entity*/nullptr,
+                                /*file_cache*/nullptr,
                                 &reader));
 
   SegmentSequence segments;
-  RETURN_NOT_OK(reader->GetSegmentsSnapshot(&segments));
+  reader->GetSegmentsSnapshot(&segments);
 
   for (const scoped_refptr<ReadableLogSegment>& segment : segments) {
     RETURN_NOT_OK(PrintSegment(segment));
@@ -565,7 +579,7 @@ Status ListBlocksInRowSet(const Schema& schema,
 
 Status DumpBlockIdsForLocalReplica(const RunnerContext& context) {
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/false, &fs_manager));
   const string& tablet_id = FindOrDie(context.required_args, kTabletIdArg);
 
   scoped_refptr<TabletMetadata> meta;
@@ -617,7 +631,7 @@ Status DumpTabletMeta(FsManager* fs_manager,
 
 Status ListLocalReplicas(const RunnerContext& context) {
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/true, &fs_manager));
 
   vector<string> tablets;
   RETURN_NOT_OK(fs_manager->ListTabletIds(&tablets));
@@ -691,7 +705,7 @@ Status DumpRowSetInternal(const IOContext& ctx,
 Status DumpRowSet(const RunnerContext& context) {
   const int kIndent = 2;
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/false, &fs_manager));
   const string& tablet_id = FindOrDie(context.required_args, kTabletIdArg);
 
   scoped_refptr<TabletMetadata> meta;
@@ -729,14 +743,14 @@ Status DumpRowSet(const RunnerContext& context) {
 
 Status DumpMeta(const RunnerContext& context) {
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/false, &fs_manager));
   const string& tablet_id = FindOrDie(context.required_args, kTabletIdArg);
   return DumpTabletMeta(fs_manager.get(), tablet_id, 0);
 }
 
 Status DumpDataDirs(const RunnerContext& context) {
   unique_ptr<FsManager> fs_manager;
-  RETURN_NOT_OK(FsInit(&fs_manager));
+  RETURN_NOT_OK(FsInit(/*skip_block_manager*/false, &fs_manager));
   const string& tablet_id = FindOrDie(context.required_args, kTabletIdArg);
   // Load the tablet meta to make sure the tablet's data directories are loaded
   // into the manager.
