@@ -27,7 +27,7 @@
 #include <glog/logging.h>
 
 #include "kudu/client/client.h"
-#include "kudu/client/shared_ptr.h"
+#include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/common/wire_protocol.h"
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/consensus.proxy.h"
@@ -763,4 +763,3 @@ Status Is343SchemeCluster(const vector<string>& master_addresses,
 
 } // namespace tools
 } // namespace kudu
-
