@@ -17,20 +17,18 @@
 
 #include "kudu/tserver/tserver_path_handlers.h"
 
-#include <stdint.h>
-
 #include <algorithm>
+#include <cstdint>
+#include <functional>
 #include <iosfwd>
 #include <map>
 #include <memory>
 #include <sstream>
 #include <string>
-#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
-#include <boost/bind.hpp> // IWYU pragma: keep
 #include <glog/logging.h>
 
 #include "kudu/common/common.pb.h"
@@ -104,12 +102,6 @@ bool CompareByMemberType(const RaftPeerPB& a, const RaftPeerPB& b) {
   return a.member_type() < b.member_type();
 }
 
-string TabletLink(const string& id) {
-  return Substitute("<a href=\"/tablet?id=$0\">$1</a>",
-                    UrlEncodeToString(id),
-                    EscapeForHtmlToString(id));
-}
-
 bool IsTombstoned(const scoped_refptr<TabletReplica>& replica) {
   return replica->data_state() == tablet::TABLET_DATA_TOMBSTONED;
 }
@@ -148,7 +140,7 @@ bool GetTabletID(const Webserver::WebRequest& req,
                  Webserver::WebResponse* resp) {
   if (!FindCopy(req.parsed_args, "id", id)) {
     resp->status_code = HttpStatusCode::BadRequest;
-    resp->output->Set("error", "Request missing 'id' argument");
+    resp->output.Set("error", "Request missing 'id' argument");
     return false;
   }
   return true;
@@ -161,8 +153,8 @@ bool GetTabletReplica(TabletServer* tserver,
                       Webserver::WebResponse* resp) {
   if (!tserver->tablet_manager()->LookupTablet(tablet_id, replica)) {
     resp->status_code = HttpStatusCode::NotFound;
-    resp->output->Set("error",
-                      Substitute("Tablet $0 not found", tablet_id));
+    resp->output.Set("error",
+                     Substitute("Tablet $0 not found", tablet_id));
     return false;
   }
   return true;
@@ -173,8 +165,8 @@ bool TabletBootstrapping(const scoped_refptr<TabletReplica>& replica,
                          Webserver::WebResponse* resp) {
   if (replica->state() == tablet::BOOTSTRAPPING) {
     resp->status_code = HttpStatusCode::ServiceUnavailable;
-    resp->output->Set("error",
-                      Substitute("Tablet $0 is still bootstrapping", tablet_id));
+    resp->output.Set("error",
+                     Substitute("Tablet $0 is still bootstrapping", tablet_id));
     return true;
   }
   return false;
@@ -199,39 +191,57 @@ TabletServerPathHandlers::~TabletServerPathHandlers() {
 Status TabletServerPathHandlers::Register(Webserver* server) {
   server->RegisterPathHandler(
     "/scans", "Scans",
-    boost::bind(&TabletServerPathHandlers::HandleScansPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleScansPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
   server->RegisterPathHandler(
     "/tablets", "Tablets",
-    boost::bind(&TabletServerPathHandlers::HandleTabletsPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleTabletsPage(req, resp);
+    },
     true /* styled */, true /* is_on_nav_bar */);
   server->RegisterPathHandler(
     "/tablet", "",
-    boost::bind(&TabletServerPathHandlers::HandleTabletPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleTabletPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
   server->RegisterPrerenderedPathHandler(
     "/transactions", "",
-    boost::bind(&TabletServerPathHandlers::HandleTransactionsPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
+      this->HandleTransactionsPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
   server->RegisterPathHandler(
     "/tablet-rowsetlayout-svg", "",
-    boost::bind(&TabletServerPathHandlers::HandleTabletSVGPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleTabletSVGPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
   server->RegisterPathHandler(
     "/tablet-consensus-status", "",
-    boost::bind(&TabletServerPathHandlers::HandleConsensusStatusPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleConsensusStatusPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
   server->RegisterPathHandler(
     "/log-anchors", "",
-    boost::bind(&TabletServerPathHandlers::HandleLogAnchorsPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleLogAnchorsPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
-  server->RegisterPrerenderedPathHandler(
+  server->RegisterPathHandler(
     "/dashboards", "Dashboards",
-    boost::bind(&TabletServerPathHandlers::HandleDashboardsPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleDashboardsPage(req, resp);
+    },
     true /* styled */, true /* is_on_nav_bar */);
   server->RegisterPathHandler(
     "/maintenance-manager", "",
-    boost::bind(&TabletServerPathHandlers::HandleMaintenanceManagerPage, this, _1, _2),
+    [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
+      this->HandleMaintenanceManagerPage(req, resp);
+    },
     true /* styled */, false /* is_on_nav_bar */);
 
   return Status::OK();
@@ -239,7 +249,7 @@ Status TabletServerPathHandlers::Register(Webserver* server) {
 
 void TabletServerPathHandlers::HandleTransactionsPage(const Webserver::WebRequest& req,
                                                       Webserver::PrerenderedWebResponse* resp) {
-  ostringstream* output = resp->output;
+  ostringstream* output = &resp->output;
   bool as_text = ContainsKey(req.parsed_args, "raw");
 
   vector<scoped_refptr<TabletReplica> > replicas;
@@ -302,7 +312,7 @@ void TabletServerPathHandlers::HandleTransactionsPage(const Webserver::WebReques
 
 void TabletServerPathHandlers::HandleTabletsPage(const Webserver::WebRequest& /*req*/,
                                                  Webserver::WebResponse* resp) {
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   vector<scoped_refptr<TabletReplica>> replicas;
   tserver_->tablet_manager()->GetTabletReplicas(&replicas);
 
@@ -342,15 +352,14 @@ void TabletServerPathHandlers::HandleTabletsPage(const Webserver::WebRequest& /*
     EasyJson details_json = replicas_json->Set("replicas", EasyJson::kArray);
     for (const scoped_refptr<TabletReplica>& replica : replicas) {
       EasyJson replica_json = details_json.PushBack(EasyJson::kObject);
-      const auto* tablet = replica->tablet();
       const auto& tmeta = replica->tablet_metadata();
       TabletStatusPB status;
       replica->GetTabletStatusPB(&status);
       replica_json["table_name"] = status.table_name();
-      if (tablet != nullptr) {
-        replica_json["id_or_link"] = TabletLink(status.tablet_id());
-      } else {
-        replica_json["id_or_link"] = status.tablet_id();
+      replica_json["id"] = status.tablet_id();
+      if (replica->tablet() != nullptr) {
+        EasyJson link_json = replica_json.Set("link", EasyJson::kObject);
+        link_json["url"] = Substitute("/tablet?id=$0", status.tablet_id());
       }
       replica_json["partition"] =
           tmeta->partition_schema().PartitionDebugString(tmeta->partition(),
@@ -406,7 +415,7 @@ void TabletServerPathHandlers::HandleTabletPage(const Webserver::WebRequest& req
     role = consensus->role();
   }
 
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   output->Set("tablet_id", tablet_id);
   output->Set("state", replica->HumanReadableState());
   output->Set("role", RaftPeerPB::Role_Name(role));
@@ -434,7 +443,7 @@ void TabletServerPathHandlers::HandleTabletSVGPage(const Webserver::WebRequest&
   scoped_refptr<TabletReplica> replica;
   if (!LoadTablet(tserver_, req, &tablet_id, &replica, resp)) return;
   shared_ptr<Tablet> tablet = replica->shared_tablet();
-  auto* output = resp->output;
+  auto* output = &resp->output;
   if (!tablet) {
     output->Set("error", Substitute("Tablet $0 is not running", tablet_id));
     return;
@@ -452,7 +461,7 @@ void TabletServerPathHandlers::HandleLogAnchorsPage(const Webserver::WebRequest&
   scoped_refptr<TabletReplica> replica;
   if (!LoadTablet(tserver_, req, &tablet_id, &replica, resp)) return;
 
-  auto* output = resp->output;
+  auto* output = &resp->output;
   output->Set("tablet_id", tablet_id);
   output->Set("log_anchors", replica->log_anchor_registry()->DumpAnchorInfo());
 }
@@ -463,7 +472,7 @@ void TabletServerPathHandlers::HandleConsensusStatusPage(const Webserver::WebReq
   scoped_refptr<TabletReplica> replica;
   if (!LoadTablet(tserver_, req, &tablet_id, &replica, resp)) return;
   shared_ptr<consensus::RaftConsensus> consensus = replica->shared_consensus();
-  auto* output = resp->output;
+  auto* output = &resp->output;
   if (!consensus) {
     output->Set("error", Substitute("Tablet $0 not initialized", tablet_id));
     return;
@@ -550,6 +559,7 @@ void ScanToJson(const ScanDescriptor& scan, EasyJson* json) {
   json->Set("requestor", scan.remote_user.username());
 
   json->Set("duration", HumanReadableElapsedTime::ToShortString(duration.ToSeconds()));
+  json->Set("num_round_trips", scan.last_call_seq_id);
   json->Set("time_since_start",
             HumanReadableElapsedTime::ToShortString(time_since_start.ToSeconds()));
 
@@ -578,8 +588,8 @@ const char* kLongTimingTitle = "wall time, user cpu time, and system cpu time "
 
 void TabletServerPathHandlers::HandleScansPage(const Webserver::WebRequest& /*req*/,
                                                Webserver::WebResponse* resp) {
-  resp->output->Set("timing_title", kLongTimingTitle);
-  EasyJson scans = resp->output->Set("scans", EasyJson::kArray);
+  resp->output.Set("timing_title", kLongTimingTitle);
+  EasyJson scans = resp->output.Set("scans", EasyJson::kArray);
   vector<ScanDescriptor> descriptors = tserver_->scanner_manager()->ListScans();
 
   for (const auto& descriptor : descriptors) {
@@ -589,34 +599,12 @@ void TabletServerPathHandlers::HandleScansPage(const Webserver::WebRequest& /*re
 }
 
 void TabletServerPathHandlers::HandleDashboardsPage(const Webserver::WebRequest& /*req*/,
-                                                    Webserver::PrerenderedWebResponse* resp) {
-  ostringstream* output = resp->output;
-  *output << "<h3>Dashboards</h3>\n";
-  *output << "<table class='table table-striped'>\n";
-  *output << "  <thead><tr><th>Dashboard</th><th>Description</th></tr></thead>\n";
-  *output << "  <tbody\n";
-  *output << GetDashboardLine("scans", "Scans", "List of currently running and recently "
-                                                "completed scans.");
-  *output << GetDashboardLine("transactions", "Transactions", "List of transactions that are "
-                                                              "currently running.");
-  *output << GetDashboardLine("maintenance-manager", "Maintenance Manager",
-                              "List of operations that are currently running and those "
-                              "that are registered.");
-  *output << "</tbody></table>\n";
-}
-
-string TabletServerPathHandlers::GetDashboardLine(const std::string& link,
-                                                  const std::string& text,
-                                                  const std::string& desc) {
-  return Substitute("  <tr><td><a href=\"$0\">$1</a></td><td>$2</td></tr>\n",
-                    EscapeForHtmlToString(link),
-                    EscapeForHtmlToString(text),
-                    EscapeForHtmlToString(desc));
+                                                    Webserver::WebResponse* /*resp*/) {
 }
 
 void TabletServerPathHandlers::HandleMaintenanceManagerPage(const Webserver::WebRequest& req,
                                                             Webserver::WebResponse* resp) {
-  EasyJson* output = resp->output;
+  EasyJson* output = &resp->output;
   MaintenanceManager* manager = tserver_->maintenance_manager();
   MaintenanceManagerStatusPB pb;
   manager->GetMaintenanceManagerStatusDump(&pb);
