@@ -18,6 +18,7 @@
 #include "kudu/util/metrics.h"
 
 #include <cstdint>
+#include <functional>
 #include <map>
 #include <memory>
 #include <ostream>
@@ -32,8 +33,6 @@
 #include <gtest/gtest.h>
 #include <rapidjson/document.h>
 
-#include "kudu/gutil/bind.h"
-#include "kudu/gutil/bind_helpers.h"
 #include "kudu/gutil/casts.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
@@ -54,6 +53,8 @@ using std::vector;
 
 DECLARE_int32(metrics_retirement_age_ms);
 
+DECLARE_string(metrics_default_level);
+
 namespace kudu {
 
 METRIC_DEFINE_entity(test_entity);
@@ -83,7 +84,8 @@ class MetricsTest : public KuduTest {
 };
 
 METRIC_DEFINE_counter(test_entity, test_counter, "My Test Counter", MetricUnit::kRequests,
-                      "Description of test counter");
+                      "Description of test counter",
+                      kudu::MetricLevel::kInfo);
 
 TEST_F(MetricsTest, SimpleCounterTest) {
   scoped_refptr<Counter> requests =
@@ -115,7 +117,8 @@ TEST_F(MetricsTest, SimpleCounterMergeTest) {
 }
 
 METRIC_DEFINE_gauge_string(test_entity, test_string_gauge, "Test string Gauge",
-                           MetricUnit::kState, "Description of string Gauge");
+                           MetricUnit::kState, "Description of string Gauge",
+                           kudu::MetricLevel::kInfo);
 
 TEST_F(MetricsTest, SimpleStringGaugeTest) {
   scoped_refptr<StringGauge> state =
@@ -169,7 +172,8 @@ TEST_F(MetricsTest, SimpleStringGaugeForMergeTest) {
 }
 
 METRIC_DEFINE_gauge_double(test_entity, test_mean_gauge, "Test mean Gauge",
-                           MetricUnit::kUnits, "Description of mean Gauge");
+                           MetricUnit::kUnits, "Description of mean Gauge",
+                           kudu::MetricLevel::kInfo);
 
 TEST_F(MetricsTest, SimpleMeanGaugeTest) {
   scoped_refptr<MeanGauge> average_usage =
@@ -201,8 +205,37 @@ TEST_F(MetricsTest, SimpleMeanGaugeMergeTest) {
   ASSERT_EQ(4, average_usage_for_merge->value());
 }
 
+TEST_F(MetricsTest, TestMeanGaugeJsonPrint) {
+  scoped_refptr<MeanGauge> test_meangauge =
+    METRIC_test_mean_gauge.InstantiateMeanGauge(entity_);
+  const double kTotalSum = 5.0;
+  const double kTotalCount = 2.0;
+  test_meangauge->set_value(kTotalSum, kTotalCount);
+
+  std::ostringstream out;
+  JsonWriter writer(&out, JsonWriter::PRETTY);
+  CHECK_OK(entity_->WriteAsJson(&writer, MetricJsonOptions()));
+
+  JsonReader reader(out.str());
+  ASSERT_OK(reader.Init());
+  vector<const rapidjson::Value*> metrics;
+  ASSERT_OK(reader.ExtractObjectArray(reader.root(), "metrics", &metrics));
+  ASSERT_EQ(1, metrics.size());
+
+  double total_sum;
+  ASSERT_OK(reader.ExtractDouble(metrics[0], "total_sum", &total_sum));
+  double total_count;
+  ASSERT_OK(reader.ExtractDouble(metrics[0], "total_count", &total_count));
+  double value;
+  ASSERT_OK(reader.ExtractDouble(metrics[0], "value", &value));
+  ASSERT_EQ(total_sum, kTotalSum);
+  ASSERT_EQ(total_count, kTotalCount);
+  ASSERT_EQ(value, kTotalSum/kTotalCount);
+}
+
 METRIC_DEFINE_gauge_uint64(test_entity, test_gauge, "Test uint64 Gauge",
-                           MetricUnit::kBytes, "Description of Test Gauge");
+                           MetricUnit::kBytes, "Description of Test Gauge",
+                           kudu::MetricLevel::kInfo);
 
 TEST_F(MetricsTest, SimpleAtomicGaugeTest) {
   scoped_refptr<AtomicGauge<uint64_t> > mem_usage =
@@ -215,11 +248,11 @@ TEST_F(MetricsTest, SimpleAtomicGaugeTest) {
   ASSERT_EQ(5, mem_usage->value());
 }
 
-TEST_F(MetricsTest, SimpleAtomicGaugeMergeTest) {
+TEST_F(MetricsTest, SimpleAtomicGaugeSumTypeMergeTest) {
   scoped_refptr<AtomicGauge<uint64_t> > mem_usage =
-    METRIC_test_gauge.Instantiate(entity_, 2);
+      METRIC_test_gauge.Instantiate(entity_, 2);
   scoped_refptr<AtomicGauge<uint64_t> > mem_usage_for_merge =
-    METRIC_test_gauge.Instantiate(entity_same_attr_, 3);
+      METRIC_test_gauge.Instantiate(entity_same_attr_, 3);
   mem_usage_for_merge->MergeFrom(mem_usage);
   ASSERT_EQ(2, mem_usage->value());
   ASSERT_EQ(5, mem_usage_for_merge->value());
@@ -231,8 +264,41 @@ TEST_F(MetricsTest, SimpleAtomicGaugeMergeTest) {
   ASSERT_EQ(14, mem_usage_for_merge->value());
 }
 
+TEST_F(MetricsTest, SimpleAtomicGaugeMaxTypeMergeTest) {
+  scoped_refptr<AtomicGauge<uint64_t> > stop_time =
+      METRIC_test_gauge.Instantiate(entity_, 2, MergeType::kMax);
+  scoped_refptr<AtomicGauge<uint64_t> > stop_time_for_merge =
+      METRIC_test_gauge.Instantiate(entity_same_attr_, 3, MergeType::kMax);
+  stop_time_for_merge->MergeFrom(stop_time);
+  ASSERT_EQ(2, stop_time->value());
+  ASSERT_EQ(3, stop_time_for_merge->value());
+  stop_time->IncrementBy(7);
+  stop_time_for_merge->MergeFrom(stop_time);
+  ASSERT_EQ(9, stop_time->value());
+  ASSERT_EQ(9, stop_time_for_merge->value());
+  stop_time_for_merge->MergeFrom(stop_time_for_merge);
+  ASSERT_EQ(9, stop_time_for_merge->value());
+}
+
+TEST_F(MetricsTest, SimpleAtomicGaugeMinTypeMergeTest) {
+  scoped_refptr<AtomicGauge<uint64_t> > start_time =
+      METRIC_test_gauge.Instantiate(entity_, 3, MergeType::kMin);
+  scoped_refptr<AtomicGauge<uint64_t> > start_time_for_merge =
+      METRIC_test_gauge.Instantiate(entity_same_attr_, 5, MergeType::kMin);
+  start_time_for_merge->MergeFrom(start_time);
+  ASSERT_EQ(3, start_time->value());
+  ASSERT_EQ(3, start_time_for_merge->value());
+  start_time->DecrementBy(2);
+  start_time_for_merge->MergeFrom(start_time);
+  ASSERT_EQ(1, start_time->value());
+  ASSERT_EQ(1, start_time_for_merge->value());
+  start_time_for_merge->MergeFrom(start_time_for_merge);
+  ASSERT_EQ(1, start_time_for_merge->value());
+}
+
 METRIC_DEFINE_gauge_int64(test_entity, test_func_gauge, "Test Function Gauge",
-                          MetricUnit::kBytes, "Test Gauge 2");
+                          MetricUnit::kBytes, "Test Gauge 2",
+                          kudu::MetricLevel::kInfo);
 
 static int64_t MyFunction(int* metric_val) {
   return (*metric_val)++;
@@ -242,7 +308,7 @@ TEST_F(MetricsTest, SimpleFunctionGaugeTest) {
   int metric_val = 1000;
   scoped_refptr<FunctionGauge<int64_t> > gauge =
     METRIC_test_func_gauge.InstantiateFunctionGauge(
-      entity_, Bind(&MyFunction, Unretained(&metric_val)));
+        entity_, [&metric_val]() { return MyFunction(&metric_val); });
 
   ASSERT_EQ(1000, gauge->value());
   ASSERT_EQ(1001, gauge->value());
@@ -258,13 +324,14 @@ TEST_F(MetricsTest, SimpleFunctionGaugeTest) {
 }
 
 METRIC_DEFINE_gauge_int64(test_entity, test_func_gauge_snapshot, "Test Function Gauge snapshot",
-                          MetricUnit::kOperations, "Description of Function Gauge snapshot");
+                          MetricUnit::kOperations, "Description of Function Gauge snapshot",
+                          kudu::MetricLevel::kInfo);
 class FunctionGaugeOwner {
  public:
   explicit FunctionGaugeOwner(const scoped_refptr<MetricEntity>& entity) {
-    METRIC_test_func_gauge_snapshot.InstantiateFunctionGauge(entity,
-       Bind(&FunctionGaugeOwner::Count, Unretained(this)))
-       ->AutoDetach(&metric_detacher_);
+    METRIC_test_func_gauge_snapshot.InstantiateFunctionGauge(
+        entity, [this]() { return this->Count(); })
+        ->AutoDetach(&metric_detacher_);
   }
 
   int64_t Count() {
@@ -310,12 +377,14 @@ TEST_F(MetricsTest, SimpleFunctionGaugeMergeTest) {
   int metric_val = 1000;
   scoped_refptr<FunctionGauge<int64_t> > gauge =
     METRIC_test_func_gauge.InstantiateFunctionGauge(
-      entity_, Bind(&MyFunction, Unretained(&metric_val)));
+        entity_, [&metric_val]() { return MyFunction(&metric_val); });
 
   int metric_val_for_merge = 1234;
   scoped_refptr<FunctionGauge<int64_t> > gauge_for_merge =
     METRIC_test_func_gauge.InstantiateFunctionGauge(
-      entity_same_attr_, Bind(&MyFunction, Unretained(&metric_val_for_merge)));
+        entity_same_attr_, [&metric_val_for_merge]() {
+          return MyFunction(&metric_val_for_merge);
+        });
 
   gauge_for_merge->MergeFrom(gauge);
   ASSERT_EQ(1001, gauge->value());
@@ -333,7 +402,7 @@ TEST_F(MetricsTest, AutoDetachToLastValue) {
   int metric_val = 1000;
   scoped_refptr<FunctionGauge<int64_t> > gauge =
     METRIC_test_func_gauge.InstantiateFunctionGauge(
-        entity_, Bind(&MyFunction, Unretained(&metric_val)));
+        entity_, [&metric_val]() { return MyFunction(&metric_val); });
 
   ASSERT_EQ(1000, gauge->value());
   ASSERT_EQ(1001, gauge->value());
@@ -352,7 +421,7 @@ TEST_F(MetricsTest, AutoDetachToConstant) {
   int metric_val = 1000;
   scoped_refptr<FunctionGauge<int64_t> > gauge =
     METRIC_test_func_gauge.InstantiateFunctionGauge(
-        entity_, Bind(&MyFunction, Unretained(&metric_val)));
+        entity_, [&metric_val]() { return MyFunction(&metric_val); });
 
   ASSERT_EQ(1000, gauge->value());
   ASSERT_EQ(1001, gauge->value());
@@ -368,13 +437,16 @@ TEST_F(MetricsTest, AutoDetachToConstant) {
 
 METRIC_DEFINE_gauge_uint64(test_entity, counter_as_gauge, "Gauge exposed as Counter",
                            MetricUnit::kBytes, "Gauge exposed as Counter",
+                           kudu::MetricLevel::kInfo,
                            EXPOSE_AS_COUNTER);
 TEST_F(MetricsTest, TEstExposeGaugeAsCounter) {
   ASSERT_EQ(MetricType::kCounter, METRIC_counter_as_gauge.type());
 }
 
 METRIC_DEFINE_histogram(test_entity, test_hist, "Test Histogram",
-                        MetricUnit::kMilliseconds, "foo", 1000000, 3);
+                        MetricUnit::kMilliseconds, "foo",
+                        MetricLevel::kInfo,
+                        1000000, 3);
 
 TEST_F(MetricsTest, SimpleHistogramTest) {
   scoped_refptr<Histogram> hist = METRIC_test_hist.Instantiate(entity_);
@@ -702,6 +774,7 @@ TEST_F(MetricsTest, TestDumpJsonPrototypes) {
     "            \"type\": \"gauge\",\n"
     "            \"unit\": \"bytes\",\n"
     "            \"description\": \"Test Gauge 2\",\n"
+    "            \"level\": \"info\",\n"
     "            \"entity_type\": \"test_entity\"\n"
     "        }";
   ASSERT_STR_CONTAINS(json, expected);
@@ -771,7 +844,7 @@ TEST_F(MetricsTest, TestDontDumpUntouched) {
   scoped_refptr<Histogram> hist = METRIC_test_hist.Instantiate(entity_);
   scoped_refptr<FunctionGauge<int64_t> > function_gauge =
     METRIC_test_func_gauge.InstantiateFunctionGauge(
-        entity_, Bind(&MyFunction, Unretained(&metric_val)));
+        entity_, [&metric_val]() { return MyFunction(&metric_val); });
   scoped_refptr<AtomicGauge<uint64_t> > atomic_gauge =
     METRIC_test_gauge.Instantiate(entity_, 0);
 
@@ -789,6 +862,14 @@ TEST_F(MetricsTest, TestDontDumpUntouched) {
   ASSERT_STR_CONTAINS(out.str(), "test_gauge");
 }
 
+METRIC_DEFINE_counter(test_entity, warn_counter, "Warn Metric", MetricUnit::kRequests,
+                      "Description of warn metric",
+                      kudu::MetricLevel::kWarn);
+
+METRIC_DEFINE_counter(test_entity, debug_counter, "Debug Metric", MetricUnit::kRequests,
+                      "Description of debug metric",
+                      kudu::MetricLevel::kDebug);
+
 TEST_F(MetricsTest, TestFilter) {
   const int32_t kNum = 4;
   vector<string> id_uuids;
@@ -817,6 +898,14 @@ TEST_F(MetricsTest, TestFilter) {
     attr2_uuids.emplace_back(attr2_uuid);
   }
 
+  // Add 1 warn and 1 debug entity and metric.
+  scoped_refptr<MetricEntity> warnEntity =
+      METRIC_ENTITY_test_entity.Instantiate(&registry_, oid.Next());
+  scoped_refptr<Counter> warnMetric = METRIC_warn_counter.Instantiate(warnEntity);
+  scoped_refptr<MetricEntity> debugEntity =
+      METRIC_ENTITY_test_entity.Instantiate(&registry_, oid.Next());
+  scoped_refptr<Counter> debugMetric = METRIC_debug_counter.Instantiate(debugEntity);
+
   // 2.Check the filter.
   Random rand(SeedRandom());
   const string not_exist_string = "not_exist_string";
@@ -840,7 +929,7 @@ TEST_F(MetricsTest, TestFilter) {
       ASSERT_OK(registry_.WriteAsJson(&w, opts));
       rapidjson::Document d;
       d.Parse<0>(out.str().c_str());
-      ASSERT_EQ(kNum + kEntityCount, d.Size());
+      ASSERT_EQ(kNum + kEntityCount + 2, d.Size());
       ASSERT_EQ(entity_type, d[rand.Next() % kNum]["type"].GetString());
     }
   }
@@ -918,14 +1007,42 @@ TEST_F(MetricsTest, TestFilter) {
       ASSERT_EQ(kNum, d.Size());
     }
   }
-  // 2.5 Default filter condition.
+  // 2.5 Filter the 'level'
+  {
+    // Higher level.
+    {
+      const string entity_level = "warn";
+      std::ostringstream out;
+      MetricJsonOptions opts;
+      opts.filters.entity_level = entity_level;
+      JsonWriter w(&out, JsonWriter::PRETTY);
+      ASSERT_OK(registry_.WriteAsJson(&w, opts));
+      rapidjson::Document d;
+      d.Parse<0>(out.str().c_str());
+      ASSERT_EQ(kEntityCount + 1, d.Size());
+    }
+    // Debug level includes info and warn level.
+    {
+      const string entity_level = "debug";
+      std::ostringstream out;
+      MetricJsonOptions opts;
+      opts.filters.entity_level = entity_level;
+      JsonWriter w(&out, JsonWriter::PRETTY);
+      ASSERT_OK(registry_.WriteAsJson(&w, opts));
+      rapidjson::Document d;
+      d.Parse<0>(out.str().c_str());
+      ASSERT_EQ(kNum + kEntityCount + 2, d.Size());
+    }
+  }
+
+  // 2.6 Default filter condition.
   {
     std::ostringstream out;
     JsonWriter w(&out, JsonWriter::PRETTY);
     ASSERT_OK(registry_.WriteAsJson(&w, MetricJsonOptions()));
     rapidjson::Document d;
     d.Parse<0>(out.str().c_str());
-    ASSERT_EQ(kNum + kEntityCount, d.Size());
+    ASSERT_EQ(kNum + kEntityCount + 2, d.Size());
   }
 }
 
