@@ -31,19 +31,16 @@
 #include <vector>
 
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 
 #include "kudu/gutil/atomic_refcount.h"
 #include "kudu/gutil/atomicops.h"
 #include "kudu/gutil/bits.h"
 #include "kudu/gutil/dynamic_annotations.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/hash/city.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/sysinfo.h"
 #include "kudu/util/cache.h"
@@ -68,23 +65,39 @@ DEFINE_string(nvm_cache_path, "/pmem",
               "The path at which the NVM cache will try to allocate its memory. "
               "This can be a tmpfs or ramfs for testing purposes.");
 
-DEFINE_int32(nvm_cache_allocation_retry_count, 0,
-             "The number of times that the NVM cache will retry attempts to allocate "
-             "memory for new entries. In between attempts, a cache entry will be "
-             "evicted.");
-TAG_FLAG(nvm_cache_allocation_retry_count, advanced);
-TAG_FLAG(nvm_cache_allocation_retry_count, experimental);
-
 DEFINE_bool(nvm_cache_simulate_allocation_failure, false,
             "If true, the NVM cache will inject failures in calls to memkind_malloc "
             "for testing.");
 TAG_FLAG(nvm_cache_simulate_allocation_failure, unsafe);
 
+DEFINE_double(nvm_cache_usage_ratio, 1.25,
+             "A ratio to set the usage of nvm cache. The charge of an item in the nvm "
+             "cache is equal to the results of memkind_malloc_usable_size multiplied by "
+             "the ratio.");
+TAG_FLAG(nvm_cache_usage_ratio, advanced);
+
 using std::string;
 using std::unique_ptr;
 using std::vector;
 using strings::Substitute;
 
+static bool ValidateNVMCacheUsageRatio(const char* flagname, double value) {
+  if (value < 1.0) {
+    LOG(ERROR) << Substitute("$0 must be greater than or equal to 1.0, value $1 is invalid.",
+                             flagname, value);
+    return false;
+  }
+  if (value < 1.25) {
+    LOG(WARNING) << Substitute("The value of $0 is $1, it is less than recommended "
+                               "value (1.25). Due to memkind fragmentation, an improper "
+                               "ratio will cause allocation failures while capacity-based "
+                               "evictions are not triggered. Raise --nvm_cache_usage_ratio.",
+                               flagname, value);
+  }
+  return true;
+}
+DEFINE_validator(nvm_cache_usage_ratio, &ValidateNVMCacheUsageRatio);
+
 namespace kudu {
 
 namespace {
@@ -305,7 +318,7 @@ class NvmLRUCache {
   void Release(Cache::Handle* handle);
   void Erase(const Slice& key, uint32_t hash);
   size_t Invalidate(const Cache::InvalidationControl& ctl);
-  void* AllocateAndRetry(size_t size);
+  void* Allocate(size_t size);
 
  private:
   void NvmLRU_Remove(LRUHandle* e);
@@ -388,37 +401,9 @@ void NvmLRUCache::FreeEntry(LRUHandle* e) {
   CALL_MEMKIND(memkind_free, vmp_, e);
 }
 
-// Allocate nvm memory. Try until successful or FLAGS_nvm_cache_allocation_retry_count
-// has been exceeded.
-void *NvmLRUCache::AllocateAndRetry(size_t size) {
-  void *tmp;
-  // There may be times that an allocation fails. With NVM we have
-  // a fixed size to allocate from. If we cannot allocate the size
-  // that was asked for, we will remove entries from the cache and
-  // retry up to the configured number of retries. If this fails, we
-  // return nullptr, which will cause the caller to not insert anything
-  // into the cache.
-  LRUHandle *to_remove_head = nullptr;
-  tmp = MemkindMalloc(size);
-
-  if (tmp == nullptr) {
-    std::unique_lock<MutexType> l(mutex_);
-
-    int retries_remaining = FLAGS_nvm_cache_allocation_retry_count;
-    while (tmp == nullptr && retries_remaining-- > 0 && lru_.next != &lru_) {
-      EvictOldestUnlocked(&to_remove_head);
-
-      // Unlock while allocating memory.
-      l.unlock();
-      tmp = MemkindMalloc(size);
-      l.lock();
-    }
-  }
-
-  // we free the entries here outside of mutex for
-  // performance reasons
-  FreeLRUEntries(to_remove_head);
-  return tmp;
+// Allocate NVM memory.
+void* NvmLRUCache::Allocate(size_t size) {
+  return MemkindMalloc(size);
 }
 
 void NvmLRUCache::NvmLRU_Remove(LRUHandle* e) {
@@ -604,7 +589,7 @@ int DetermineShardBits() {
 class ShardedLRUCache : public Cache {
  private:
   unique_ptr<CacheMetrics> metrics_;
-  vector<NvmLRUCache*> shards_;
+  vector<unique_ptr<NvmLRUCache>> shards_;
 
   // Number of bits of hash used to determine the shard.
   const int shard_bits_;
@@ -629,14 +614,14 @@ class ShardedLRUCache : public Cache {
     int num_shards = 1 << shard_bits_;
     const size_t per_shard = (capacity + (num_shards - 1)) / num_shards;
     for (int s = 0; s < num_shards; s++) {
-      gscoped_ptr<NvmLRUCache> shard(new NvmLRUCache(vmp_));
+      unique_ptr<NvmLRUCache> shard(new NvmLRUCache(vmp_));
       shard->SetCapacity(per_shard);
-      shards_.push_back(shard.release());
+      shards_.emplace_back(std::move(shard));
     }
   }
 
   virtual ~ShardedLRUCache() {
-    STLDeleteElements(&shards_);
+    shards_.clear();
     // Per the note at the top of this file, our cache is entirely volatile.
     // Hence, when the cache is destructed, we delete the underlying
     // memkind pool.
@@ -673,8 +658,8 @@ class ShardedLRUCache : public Cache {
 
   virtual void SetMetrics(unique_ptr<CacheMetrics> metrics) OVERRIDE {
     metrics_ = std::move(metrics);
-    for (NvmLRUCache* cache : shards_) {
-      cache->SetMetrics(metrics_.get());
+    for (const auto& shard : shards_) {
+      shard->SetMetrics(metrics_.get());
     }
   }
   virtual UniquePendingHandle Allocate(Slice key, int val_len, int charge) OVERRIDE {
@@ -685,9 +670,9 @@ class ShardedLRUCache : public Cache {
     // Try allocating from each of the shards -- if memkind is tight,
     // this can cause eviction, so we might have better luck in different
     // shards.
-    for (NvmLRUCache* cache : shards_) {
+    for (const auto& shard : shards_) {
       UniquePendingHandle ph(static_cast<PendingHandle*>(
-          cache->AllocateAndRetry(sizeof(LRUHandle) + key_len + val_len)),
+          shard->Allocate(sizeof(LRUHandle) + key_len + val_len)),
           Cache::PendingHandleDeleter(this));
       if (ph) {
         LRUHandle* handle = reinterpret_cast<LRUHandle*>(ph.get());
@@ -695,8 +680,11 @@ class ShardedLRUCache : public Cache {
         handle->kv_data = &buf[sizeof(LRUHandle)];
         handle->val_length = val_len;
         handle->key_length = key_len;
+        // Multiply the results of memkind_malloc_usable_size by a ratio
+        // due to the fragmentation is not counted in.
         handle->charge = (charge == kAutomaticCharge) ?
-            CALL_MEMKIND(memkind_malloc_usable_size, vmp_, buf) : charge;
+            CALL_MEMKIND(memkind_malloc_usable_size, vmp_, buf) * FLAGS_nvm_cache_usage_ratio :
+            charge;
         handle->hash = HashSlice(key);
         memcpy(handle->kv_data, key.data(), key.size());
         return ph;
@@ -712,7 +700,7 @@ class ShardedLRUCache : public Cache {
 
   size_t Invalidate(const InvalidationControl& ctl) override {
     size_t invalidated_count = 0;
-    for (auto& shard: shards_) {
+    for (const auto& shard: shards_) {
       invalidated_count += shard->Invalidate(ctl);
     }
     return invalidated_count;
@@ -737,6 +725,9 @@ Cache* NewCache<Cache::EvictionPolicy::LRU,
     << "configured capacity " << capacity << " bytes is less than "
     << "the minimum capacity for an NVM cache: " << MEMKIND_PMEM_MIN_SIZE;
 
+  LOG(INFO) << 1 / FLAGS_nvm_cache_usage_ratio * 100 << "% capacity "
+            << "from nvm block cache is available due to memkind fragmentation.";
+
   memkind* vmp;
   int err = CALL_MEMKIND(memkind_create_pmem, FLAGS_nvm_cache_path.c_str(), capacity, &vmp);
   // If we cannot create the cache pool we should not retry.
