@@ -101,6 +101,11 @@ else
       "sentry")       F_SENTRY=1 ;;
       "yaml")         F_YAML=1 ;;
       "chrony")       F_CHRONY=1 ;;
+      "gumbo-parser") F_GUMBO_PARSER=1 ;;
+      "gumbo-query")  F_GUMBO_QUERY=1 ;;
+      "postgres")     F_POSTGRES=1 ;;
+      "psql-jdbc")    F_POSTGRES_JDBC=1 ;;
+      "ranger")       F_RANGER=1 ;;
       *)              echo "Unknown module: $arg"; exit 1 ;;
     esac
   done
@@ -250,6 +255,15 @@ if [ -n "$F_COMMON" -o -n "$F_CHRONY" ]; then
   build_chrony
 fi
 
+if [ -n "$F_COMMON" -o -n "$F_POSTGRES" ]; then
+  build_postgres
+fi
+
+if [ -n "$F_COMMON" -o -n "$F_POSTGRES_JDBC" ]; then
+  mkdir -p $PREFIX/opt/jdbc
+  ln -nsf $POSTGRES_JDBC_SOURCE/$POSTGRES_JDBC_NAME.jar $PREFIX/opt/jdbc/postgresql.jar
+fi
+
 # Install Hadoop, Hive, and Sentry by symlinking their source directories (which
 # are pre-built) into $PREFIX/opt.
 if [ -n "$F_COMMON" -o -n "$F_HADOOP" ]; then
@@ -271,6 +285,19 @@ if [ -n "$F_COMMON" -o -n "$F_SENTRY" ]; then
   ln -nsf $SENTRY_SOURCE $PREFIX/opt/sentry
 fi
 
+if [ -n "$F_COMMON" -o -n "$F_RANGER" ]; then
+  mkdir -p $PREFIX/opt
+  # Remove any hadoop jars included in the Ranger package to avoid unexpected
+  # runtime behavior due to different versions of hadoop jars, similarly to
+  # Sentry.
+  rm -rf $RANGER_SOURCE/lib/hadoop-[a-z-]*.jar
+  ln -nsf $RANGER_SOURCE $PREFIX/opt/ranger
+
+  # Symlink conf.dist to conf
+  ln -nsf $PREFIX/opt/ranger/ews/webapp/WEB-INF/classes/conf.dist \
+  $PREFIX/opt/ranger/ews/webapp/WEB-INF/classes/conf
+fi
+
 ### Build C dependencies without instrumentation
 
 PREFIX=$PREFIX_DEPS
@@ -320,6 +347,15 @@ if [ -n "$F_COMMON" -o -n "$F_LLVM" ]; then
   build_llvm normal
 fi
 
+# From this point forward, clang is available for us to use if needed.
+if which ccache >/dev/null ; then
+  CLANG="$TP_DIR/../build-support/ccache-clang/clang"
+  CLANGXX="$TP_DIR/../build-support/ccache-clang/clang++"
+else
+  CLANG="$TP_DIR/clang-toolchain/bin/clang"
+  CLANGXX="$TP_DIR/clang-toolchain/bin/clang++"
+fi
+
 save_env
 
 # Enable debug symbols so that stacktraces and linenumbers are available at
@@ -328,6 +364,15 @@ save_env
 EXTRA_CFLAGS="-g $EXTRA_CFLAGS"
 EXTRA_CXXFLAGS="-g $EXTRA_CXXFLAGS"
 
+# Build libc++abi first as it is a dependency for libc++.
+if [ -n "$F_UNINSTRUMENTED" -o -n "$F_LLVM" ]; then
+  build_libcxxabi
+fi
+
+if [ -n "$F_UNINSTRUMENTED" -o -n "$F_LLVM" ]; then
+  build_libcxx normal
+fi
+
 if [ -n "$F_UNINSTRUMENTED" -o -n "$F_GFLAGS" ]; then
   build_gflags
 fi
@@ -376,6 +421,15 @@ if [ -n "$F_UNINSTRUMENTED" -o -n "$F_YAML" ]; then
   build_yaml
 fi
 
+if [ -n "$F_UNINSTRUMENTED" -o -n "$F_GUMBO_PARSER" ]; then
+  # Although it's a C library its tests are written in C++.
+  build_gumbo_parser
+fi
+
+if [ -n "$F_UNINSTRUMENTED" -o -n "$F_GUMBO_QUERY" ]; then
+  build_gumbo_query
+fi
+
 restore_env
 
 # If we're on macOS best to exit here, otherwise single dependency builds will try to
@@ -411,13 +465,6 @@ fi
 #   * -Wl,-rpath,... - Add instrumented libc++ location to the rpath so that it
 #                      can be found at runtime.
 
-if which ccache >/dev/null ; then
-  CLANG="$TP_DIR/../build-support/ccache-clang/clang"
-  CLANGXX="$TP_DIR/../build-support/ccache-clang/clang++"
-else
-  CLANG="$TP_DIR/clang-toolchain/bin/clang"
-  CLANGXX="$TP_DIR/clang-toolchain/bin/clang++"
-fi
 export CC=$CLANG
 export CXX=$CLANGXX
 
@@ -556,6 +603,15 @@ if [ -n "$F_TSAN" -o -n "$F_YAML" ]; then
   build_yaml
 fi
 
+if [ -n "$F_TSAN" -o -n "$F_GUMBO_PARSER" ]; then
+  # Although it's a C library its tests are written in C++.
+  build_gumbo_parser
+fi
+
+if [ -n "$F_TSAN" -o -n "$F_GUMBO_QUERY" ]; then
+  build_gumbo_query
+fi
+
 restore_env
 
 finish
