@@ -104,6 +104,9 @@ fetch_and_expand() {
         rm "$FILENAME"
         continue
       fi
+    elif [[ "$FILENAME" =~ \.jar$ ]]; then
+      mkdir ${FILENAME%.jar}
+      cp $FILENAME ${FILENAME%.jar}/
     else
       echo "Error: unknown file format: $FILENAME"
       exit 1
@@ -167,7 +170,7 @@ fetch_and_patch() {
 mkdir -p $TP_SOURCE_DIR
 cd $TP_SOURCE_DIR
 
-GLOG_PATCHLEVEL=3
+GLOG_PATCHLEVEL=4
 fetch_and_patch \
  glog-${GLOG_VERSION}.tar.gz \
  $GLOG_SOURCE \
@@ -175,6 +178,7 @@ fetch_and_patch \
  "patch -p0 < $TP_DIR/patches/glog-issue-198-fix-unused-warnings.patch" \
  "patch -p0 < $TP_DIR/patches/glog-issue-54-dont-build-tests.patch" \
  "patch -p1 < $TP_DIR/patches/glog-fix-symbolization.patch" \
+ "patch -p1 < $TP_DIR/patches/glog-support-stacktrace-for-aarch64.patch" \
  "autoreconf -fvi"
 
 GMOCK_PATCHLEVEL=0
@@ -297,12 +301,11 @@ fetch_and_patch \
  $CURL_PATCHLEVEL \
  "autoreconf -fvi"
 
-CRCUTIL_PATCHLEVEL=1
+CRCUTIL_PATCHLEVEL=0
 fetch_and_patch \
  crcutil-${CRCUTIL_VERSION}.tar.gz \
  $CRCUTIL_SOURCE \
- $CRCUTIL_PATCHLEVEL \
- "patch -p0 < $TP_DIR/patches/crcutil-fix-libtoolize-on-osx.patch"
+ $CRCUTIL_PATCHLEVEL
 
 LIBUNWIND_PATCHLEVEL=1
 fetch_and_patch \
@@ -317,19 +320,14 @@ fetch_and_patch \
  $PYTHON_SOURCE \
  $PYTHON_PATCHLEVEL
 
-LLVM_PATCHLEVEL=4
+LLVM_PATCHLEVEL=5
 fetch_and_patch \
  llvm-${LLVM_VERSION}-iwyu-${IWYU_VERSION}.src.tar.gz \
  $LLVM_SOURCE \
  $LLVM_PATCHLEVEL \
-  "patch -p1 < $TP_DIR/patches/llvm-fix-amazon-linux.patch" \
   "patch -p1 < $TP_DIR/patches/llvm-add-iwyu.patch" \
-  "patch -p1 < $TP_DIR/patches/llvm-iwyu-nocurses.patch" \
   "patch -p1 < $TP_DIR/patches/llvm-iwyu-include-picker.patch" \
-  "patch -d tools/clang/tools/include-what-you-use -p1 < $TP_DIR/patches/llvm-iwyu-llvm-6-compat.patch" \
-  "patch -d projects/compiler-rt -p1 < $TP_DIR/patches/llvm-tsan-disable-trace-switching-after-multithreaded-for.patch" \
-  "patch -d projects -p1 < $TP_DIR/patches/llvm-ustat-removal.patch" \
-  "patch -d projects -p1 < $TP_DIR/patches/llvm-fix-944-destruction-of-a-locked-mutex.patch"
+  "patch -p0 < $TP_DIR/patches/llvm-iwyu-sized-deallocation.patch"
 
 LZ4_PATCHLEVEL=0
 fetch_and_patch \
@@ -349,12 +347,13 @@ fetch_and_patch \
  $TRACE_VIEWER_SOURCE \
  $TRACE_VIEWER_PATCHLEVEL
 
-BOOST_PATCHLEVEL=1
+BOOST_PATCHLEVEL=2
 fetch_and_patch \
  boost_${BOOST_VERSION}.tar.gz \
  $BOOST_SOURCE \
  $BOOST_PATCHLEVEL \
- "patch -p0 < $TP_DIR/patches/boost-issue-12179-fix-compilation-errors.patch"
+ "patch -p0 < $TP_DIR/patches/boost-issue-12179-fix-compilation-errors.patch" \
+ "patch -p0 < $TP_DIR/patches/boost-issue-440-darwin-version.patch"
 
 # Return 0 if the current system appears to be el6 (either CentOS or proper RHEL)
 needs_openssl_workaround() {
@@ -376,12 +375,13 @@ fetch_and_patch \
  $BREAKPAD_PATCHLEVEL \
  "patch -p1 < $TP_DIR/patches/breakpad-add-basic-support-for-dwz-dwarf-extension.patch"
 
-SPARSEHASH_PATCHLEVEL=2
+SPARSEHASH_PATCHLEVEL=3
 fetch_and_patch \
  sparsehash-c11-${SPARSEHASH_VERSION}.tar.gz \
  $SPARSEHASH_SOURCE \
  $SPARSEHASH_PATCHLEVEL \
- "patch -p1 < $TP_DIR/patches/sparsehash-0001-Add-compatibily-for-gcc-4.x-in-traits.patch"
+ "patch -p1 < $TP_DIR/patches/sparsehash-0001-Add-compatibily-for-gcc-4.x-in-traits.patch" \
+ "patch -p1 < $TP_DIR/patches/sparsehash-0002-Add-workaround-for-dense_hashtable-move-constructor-.patch"
 
 SPARSEPP_PATCHLEVEL=0
 fetch_and_patch \
@@ -435,5 +435,42 @@ fetch_and_patch \
  "patch -p1 < $TP_DIR/patches/chrony-no-superuser.patch" \
  "patch -p1 < $TP_DIR/patches/chrony-reuseport.patch"
 
+GUMBO_PARSER_PATCHLEVEL=1
+fetch_and_patch \
+ $GUMBO_PARSER_NAME.tar.gz \
+ $GUMBO_PARSER_SOURCE \
+ $GUMBO_PARSER_PATCHLEVEL \
+ "patch -p1 < $TP_DIR/patches/gumbo-parser-autoconf-263.patch" \
+ "autoreconf -fvi"
+
+GUMBO_QUERY_PATCHLEVEL=1
+fetch_and_patch \
+ $GUMBO_QUERY_NAME.tar.gz \
+ $GUMBO_QUERY_SOURCE \
+ $GUMBO_QUERY_PATCHLEVEL \
+ "patch -p1 < $TP_DIR/patches/gumbo-query-namespace.patch"
+
+POSTGRES_PATCHLEVEL=1
+fetch_and_patch \
+ $POSTGRES_NAME.tar.gz \
+ $POSTGRES_SOURCE \
+ $POSTGRES_PATCHLEVEL \
+ "patch -p0 < $TP_DIR/patches/postgres-root-can-run-initdb.patch" \
+ "patch -p0 < $TP_DIR/patches/postgres-no-check-root.patch"
+
+POSTGRES_JDBC_PATCHLEVEL=0
+fetch_and_patch \
+ $POSTGRES_JDBC_NAME.jar \
+ $POSTGRES_JDBC_SOURCE \
+ $POSTGRES_JDBC_PATCHLEVEL
+
+RANGER_PATCHLEVEL=2
+fetch_and_patch \
+ $RANGER_NAME.tar.gz \
+ $RANGER_SOURCE \
+ $RANGER_PATCHLEVEL \
+ "patch -p1 < $TP_DIR/patches/ranger-python3.patch" \
+ "patch -p0 < $TP_DIR/patches/ranger-fixscripts.patch"
+
 echo "---------------"
 echo "Thirdparty dependencies downloaded successfully"
